#        #!/bin/sh
#        # the next line restarts using wish \
#        exec wish "$0" "$@"

#*************************************************************************
# Copyright (c) 2002 The University of Chicago, as Operator of Argonne
# National Laboratory.
# Copyright (c) 2002 The Regents of the University of California, as
# Operator of Los Alamos National Laboratory.
# This file is distributed subject to a Software License Agreement found
# in the file LICENSE that is included with this distribution. 
#*************************************************************************
# DCT - Data Configuration Tool V2.1

foreach i [winfo child .] {
    catch {destroy $i}
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.8  1996/02/27 22:55:22  saunders
# Strip $ chars out of given version message.
#
# Revision 1.7  1996/02/14  16:40:07  saunders
# Removed APSMp dummy routines from APSMisc.tcl. This will fix the problem
# with using APSUniqueName from within a machine procedure.
#
# Revision 1.6  1996/01/16  21:19:54  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSApplicationHelp {} {
    return "Usage: APSApplication widget \n\[-name <string>\]\t\n\[-version <string>\]\t\n\[-overview <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$widget.\n\tuserFrame\n\tmenu.file.menu\n\tmenu.help.menu"
}

proc APSApplication {widget args} {
    global apsContextHelp
    set name "Untitled"
    set version " Revision: 1.0 Author: unknown"
    set overview "None"
    set contextHelp "No context help available."
    APSStrictParseArguments {name version overview contextHelp}

    APSStandardSetup

    regsub -all {[$]} $version " " version

    set apsContextHelp(.) $contextHelp
    if {$widget == "."} {
	wm title . $name
	APSMenubar menu -parent . -name $name -version $version \
	  -overview $overview
	frame .userFrame -width 200 -height 100
	pack .userFrame -side top -fill both -expand 1
    } else {
	toplevel $widget
	wm title $widget $name
	APSMenubar .menu -parent $widget -name $name -version $version \
	  -overview $overview
	frame $widget.userFrame -width 200 -height 100
	pack $widget.userFrame -side top -fill both -expand 1
    }
    focus $widget
}

proc APSMenubarHelp {} {
    return "Usage: APSMenubar widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string>\]\t\n\[-version <string>\]\t\n\[-overview <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tfile.menu\n\thelp.menu"
}

proc APSMenubar {widget args} {
    global apsMenuForeground apsMenuBackground
    global apsContextHelp apsWidgetCursorType apsContextHelpEnable
    global apsContextHelpExcludeList
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "Untitled"
    set version "1.0"
    set overview "None"
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption name version overview \
			 contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$parent == ""} {
	toplevel $widget -relief raised -bd 2 -bg $apsMenuBackground
	wm title $widget $name
    } else {
	frame $parent$widget -relief raised -bd 2 -bg $apsMenuBackground
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    menubutton $parent$widget.file -text File -underline 0 \
      -menu $parent$widget.file.menu  \
      -bg $apsMenuBackground -fg $apsMenuForeground
    menu $parent$widget.file.menu -bg $apsMenuBackground \
      -fg $apsMenuForeground -activeforeground $apsMenuForeground

    $parent$widget.file.menu add command -label Quit -underline 0 -command exit
    pack $parent$widget.file -side left

    menubutton $parent$widget.help -text Help -underline 0 \
      -menu $parent$widget.help.menu \
      -bg $apsMenuBackground -fg $apsMenuForeground
    menu $parent$widget.help.menu -bg $apsMenuBackground \
      -fg $apsMenuForeground -activeforeground $apsMenuForeground

    $parent$widget.help.menu add command -label Overview -underline 0 -command\
      [list APSInfoWindow $parent$widget.overview -name Overview \
	 -infoMessage $overview]
    $parent$widget.help.menu add command -label Version -underline 0 -command \
      [list APSInfoWindow $parent$widget.version -name Version \
	 -infoMessage $version]
    
    set apsContextHelpExcludeList \
      [lappend apsContextHelpExcludeList $parent$widget.help]
    bind contextHelpTag <Button-1> { 
	APSInfoWindow .contextHelp -name "Context Help" \
	  -infoMessage [APSNearestContextHelp %W] 
	break 
    }
    # Make <Btn-3> be help in all windows
    bind all <Double-Button-3> { 
	APSInfoWindow .contextHelp -name "Context Help" \
	  -infoMessage [APSNearestContextHelp %W]
    }
    bind all <KeyPress-Help> {
	if {$apsContextHelpEnable == 0} {
	    set apsContextHelpEnable 1
	} else {
	    set apsContextHelpEnable 0
	}
	if {$apsContextHelpEnable} { 
	    APSPushHeadBindList . contextHelpTag
	    foreach x [APSListAllWidgets .] {
		set apsWidgetCursorType($x) \
		  [lindex [$x configure -cursor] end] 
		$x configure -cursor question_arrow 
	    } 
	} else { 
	    APSPopHeadBindList .
	    foreach x [APSListAllWidgets .] {
		if {[info exists apsWidgetCursorType($x)]} {
		    $x configure -cursor $apsWidgetCursorType($x) 
		}
	    }
	    if {[winfo exists .contextHelp]} {
		destroy .contextHelp
	    }
	} 
    }
    $parent$widget.help.menu add check -label "Context Help" \
      -variable apsContextHelpEnable -command {
	  if {$apsContextHelpEnable} { 
	      APSPushHeadBindList . contextHelpTag
	      foreach x [APSListAllWidgets .] {
		  set apsWidgetCursorType($x) \
		    [lindex [$x configure -cursor] end] 
		  $x configure -cursor question_arrow 
	      } 
	  } else { 
	      APSPopHeadBindList .
	      foreach x [APSListAllWidgets .] {
		  if {[info exists apsWidgetCursorType($x)]} {
		      $x configure -cursor $apsWidgetCursorType($x) 
		  }
	      } 
	      if {[winfo exists .contextHelp]} {
		  destroy .contextHelp
	      }
	  } 
      }
    pack $parent$widget.help -side right
    focus $parent$widget
}
    
proc APSMenubarAddMenuHelp {} {
    return "Usage: APSMenubarAddMenu widget \n\[-parent <string>\]\t\n\[-text <string>\]\t\n\[-packOption <list>\]\t\n\[-underline 1\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tmenu"
}

proc APSMenubarAddMenu {widget args} {
    global apsMenuForeground apsMenuBackground apsContextHelp
    set parent ""
    set packOption "-side left"
    set text "noentry"
    set underline 0
    set contextHelp ""
    APSStrictParseArguments {parent text underline packOption contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    menubutton $parent$widget -text $text -underline $underline \
      -menu $parent$widget.menu  \
      -bg $apsMenuBackground -fg $apsMenuForeground
    menu $parent$widget.menu -bg $apsMenuBackground \
      -fg $apsMenuForeground \
      -activeforeground $apsMenuForeground
    eval pack $parent$widget $packOption

}

proc APSListAllWidgets {root} {
    set list {}
    set childlist [winfo children $root]
    foreach x $childlist {
	set list [concat $list [APSListAllWidgets $x]]
    }
    return [concat $root $list]
}

proc APSPushHeadBindList {widget tag} {
    global apsContextHelpExcludeList
    set childList [winfo children $widget]
    foreach x $childList {
	if {[lsearch -exact $apsContextHelpExcludeList $x] == -1} {
	    bindtags $x "$tag [bindtags $x]"
	    APSPushHeadBindList $x $tag
	}
    }
}

proc APSPopHeadBindList {widget} {
    global apsContextHelpExcludeList
    set childList [winfo children $widget]
    foreach x $childList {
	if {[lsearch -exact $apsContextHelpExcludeList $x] == -1} {
	    bindtags $x [lrange [bindtags $x] 1 end]
	    APSPopHeadBindList $x
	}
    }
}

proc APSNearestContextHelp {widget} {
    global apsContextHelp

    while {$widget != "" &&
	   ![info exists apsContextHelp($widget)]} {
	set splitWidget [split $widget .]
	set widget [join [lrange $splitWidget 0 \
			  [expr [llength $splitWidget]-2]] .]
    }
    if {$widget != ""} {
	return $apsContextHelp($widget)
    } elseif {[info exists apsContextHelp(.)]} {
	return $apsContextHelp(.)
    } else {
	return ""
    }
}
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.3  1996/01/16 21:19:52  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSAlertBoxHelp {} {
    return "Usage: APSAlertBox widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-contextHelp <string>\]\t\n\[-errorMessage <string>\]\t\n\[-modeless 1\]\t\n\nCreates\t\$parent\$widget.\n\tmsg\n\tbuttonRow.ok.button\n\t"
}

proc APSAlertBox {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "Alert"
    set errorMessage "No Message Given"
    set modeless 0
    set contextHelp ""

    APSStrictParseArguments {parent noPack packOption errorMessage modeless \
			 contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$parent == ""} {
	toplevel $widget -bg Red
	wm title $widget $name
    } else {
	frame $parent$widget -bg Red
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    label $parent$widget.label -relief ridge -text Error -bg red -fg white
    pack $parent$widget.label -side top -fill x
    message $parent$widget.msg -relief ridge -justify center -bd 2 \
      -width 400 -text "$errorMessage\n                            "
    pack $parent$widget.msg -side top -fill both -expand 1

    frame $parent$widget.buttonRow -relief raised -bd 2
    pack $parent$widget.buttonRow -side bottom -fill x

    APSButton .ok -parent $parent$widget.buttonRow -text OK \
      -command "destroy $parent$widget" -highlight 1 -contextHelp \
      "Dismiss this warning and continue with application."

    if {!$modeless} {
	focus $parent$widget
	grab $parent$widget
#        if {$parent == ""} {
#        bind $widget <Visibility> "DCTraise $widget"
#    } else {
#        bind $parent$widget <Visibility> "DCTraise $parent$widget"
#	}
	tkwait window $parent$widget
    }
}    




#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.15  1996/02/14 20:48:04  saunders
# Added -commandList option to RadioButtonFrame procedure.
#
# Revision 1.14  1996/02/09  21:24:50  saunders
# Added -geometry option to APSFrame and APSMenuFrame to allow relative
# positioning of submenus.
#
# Revision 1.13  1996/02/09  17:12:13  saunders
# Additions to support APSMenuFrame procedure.
#
# Revision 1.12  1996/02/07  20:14:29  saunders
# Added -commandList to APSCheckButtonFrame procedure.
#
# Revision 1.11  1996/01/16  21:19:56  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSButtonHelp {} {
    return "Usage: APSButton widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-text <string>\]\t\n\[-command <string>\]\t\n\[-highlight 1\]\t\n\[-size <string>\] where <string> is small or medium (default)\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tbutton"
}

proc APSButton {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side left -padx 1 -pady 1"
    set text "notext"
    set command ""
    set highlight 0
    set size medium
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption text command highlight \
			 size contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    if {$parent == ""} {
	if {$highlight} {
	    toplevel $widget -bd 2 -bg black
	} else {
	    toplevel $widget -bd 2
	}
	wm title $widget $text
    } else {
	if {$highlight} {
	    frame $parent$widget -bd 2 -bg black
	} else {
	    frame $parent$widget -bd 2
	}
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }
    if {$size == "small"} {
       button $parent$widget.button -text $text -command $command \
          -highlightthickness 0 -padx 3 -pady 1 -font \
          -adobe-courier-medium-r-normal-*-10-*-*-*-*-*-*-*
    } else {
       button $parent$widget.button -text $text -command $command \
          -highlightthickness 0 -padx 6 -pady 3
    }
    pack $parent$widget.button -side top -fill x
}

proc APSEnableButtonHelp {} {
    return "Usage: APSEnableButton widget \n\tSets button widget state to normal."
}
proc APSEnableButton {widget} {
    $widget configure -state normal
}

proc APSDisableButtonHelp {} {
    return "Usage: APSDisableButton widget \n\tSets button widget state to disabled."
}
proc APSDisableButton {widget} {
    $widget configure -state disabled
}

proc APSSecureButtonAlert {} {
    APSAlertBox [APSUniqueName .uniqueNameError] -errorMessage "You do not have permission to run that application.  Send email to borland to be put on the permission list."
}

proc APSSecureButton {widget args} {
    global env
    set allowedUsers ""
    APSParseArguments {allowedUsers }
    set userList [split $allowedUsers]
    if [llength $userList]!=0  {
        if [lsearch -exact $userList $env(USER)]==-1 {
            eval APSButton $widget $args \
              -command APSSecureButtonAlert 
            return
        }
    }
    eval APSButton $widget $args
}

proc APSLabeledEntryHelp {} {
    return "Usage: APSLabeledEntry widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-textVariable <string>\]\t\n\[-width <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tentry"
}
proc APSLabeledEntry {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set label "NoLabel"
    set textVariable ""
    set width 20
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label textVariable width \
		     contextHelp }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$parent == ""} {
	toplevel $widget -bd 1
	wm title $widget $label
    } else {
	frame $parent$widget -bd 1
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }
    label $parent$widget.label -text $label
    pack $parent$widget.label -side left -fill x
    entry $parent$widget.entry -relief sunken -width $width \
      -textvariable $textVariable
    pack $parent$widget.entry -side right -fill x
}

proc APSLabeledEntryFrameHelp {} {
    return "Usage: APSLabeledEntryFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-variableList <list>\]\t\n\[-width <string>\]\t\n\[-orientation <string>\] where <string> is horizontal or vertical (default)\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tframe.entry1\n\tframe.entry2,...,frame.entry<n>\n"
}
proc APSLabeledEntryFrame {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set label ""
    set variableList ""
    set width 20
    set orientation vertical
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label variableList \
			       width orientation contextHelp }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$orientation == "vertical"} {
	set packside top
    } else {
	set packside left
    }
    APSFrame $widget -parent $parent -label $label -orientation $orientation \
      -packOption $packOption -noPack $noPack
    set i 1
    foreach var $variableList {
	entry $parent$widget.frame.entry$i -relief sunken -width $width \
	  -textvariable $var
	pack $parent$widget.frame.entry$i -side $packside
	incr i
    }
}

proc APSLabeledOutputHelp {} {
    return "Usage: APSLabeledOutput widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-textVariable <string>\]\t\n\[-width <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tentry"
}

proc APSLabeledOutput {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set label "NoLabel"
    set textVariable ""
    set width 20
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label textVariable \
			       width contextHelp }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }
    if {$parent == ""} {
	toplevel $widget -bd 1
	wm title $widget $label
    } else {
	frame $parent$widget -bd 1
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }
    label $parent$widget.label -text $label
    pack $parent$widget.label -side left -fill x
    entry $parent$widget.entry -relief ridge -width $width \
      -textvariable $textVariable -state disabled
    pack $parent$widget.entry -side right -fill x
}

proc APSLabeledOutputFrameHelp {} {
    return "Usage: APSLabeledOutputFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-variableList <list>\]\t\n\[-width <string>\]\t\n\[-orientation <string>\] where <string> is horizontal or vertical (default)\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tframe.entry1\n\tframe.entry2,...,frame.entry<n>\n"
}
proc APSLabeledOutputFrame {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set label ""
    set variableList ""
    set width 20
    set orientation vertical
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label variableList width \
			 orientation contextHelp }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$orientation == "vertical"} {
	set packside top
    } else {
	set packside left
    }
    APSFrame $widget -parent $parent -label $label -orientation $orientation \
      -packOption $packOption -noPack $noPack
    set i 1
    foreach var $variableList {
	entry $parent$widget.frame.entry$i -relief ridge -width $width \
	  -textvariable $var -state disabled
	pack $parent$widget.frame.entry$i -side $packside
	incr i
    }
}

proc APSRadioButtonFrameHelp {} {
    return "Usage: APSRadioButtonFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-variable <string>\]\t\n\[-buttonList <list>\]\t\n\[-valueList <list>\]\t\n\[-orientation <string>\] where <string> is horizontal or vertical\n\[-commandList <list>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tframe\n\tframe.button1\n\tframe.button2,...\n"
}

proc APSRadioButtonFrame {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top"
    set label "NoLabel"
    set variable ""
    set buttonList ""
    set valueList ""
    set orientation vertical
    set commandList ""
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label variable \
			       buttonList valueList orientation contextHelp \
			       commandList}

    if {$commandList != "" &&
	[llength $buttonList] != [llength $commandList]} {
	puts stderr "commandList must be same length as buttonList"
	return
    }    
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }
    if {$orientation == "vertical"} {
	set packside top
	set packOption "$packOption -fill y"
    } else {
	set packside left
	set packOption "$packOption -fill x"
    }

    APSFrame $widget -parent $parent -packOption $packOption -noPack $noPack\
      -label $label -orientation $orientation

    set valueIndex 0
    set buttonNumber 1
    foreach button $buttonList {
	set cmd [lindex $commandList $valueIndex]
	radiobutton $parent$widget.frame.button$buttonNumber -text $button \
	  -variable $variable -value [lindex $valueList $valueIndex] \
	  -anchor w -command $cmd
	pack $parent$widget.frame.button$buttonNumber -side $packside -fill x
	incr valueIndex
	incr buttonNumber
    }
}

proc APSCheckButtonFrameHelp {} {
    return "Usage: APSCheckButtonFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-buttonList <list>\]\t\n\[-variableList <list>\]\t\n\[-commandList <list>\]\t\n\[-orientation <string>\] where <string> is horizontal or vertical\n\[-allNone 1] adds All and None buttons\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tframe\n\tframe.button1\n\tframe.button2,...\n"
}

proc APSCheckButtonFrame {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top"
    set label "NoLabel"
    set buttonList ""
    set variableList ""
    set commandList ""
    set orientation vertical
    set allNone 0
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label buttonList \
			       variableList orientation allNone commandList \
			       contextHelp }

    if {$commandList != "" &&
	[llength $buttonList] != [llength $commandList]} {
	puts stderr "commandList must be same length as buttonList"
	return
    }
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }
    if {$orientation == "vertical"} {
	set packside top
	set packOption "$packOption -fill y"
    } else {
	set packside left
	set packOption "$packOption -fill x"
    }
    APSFrame $widget -parent $parent -packOption $packOption -noPack $noPack\
      -label $label -orientation $orientation

    set variableIndex 0
    set buttonNumber 1
    foreach button $buttonList {
	set cmd [lindex $commandList $variableIndex]
	checkbutton $parent$widget.frame.button$buttonNumber -text $button \
	  -variable [lindex $variableList $variableIndex] -anchor w \
	  -command $cmd
	pack $parent$widget.frame.button$buttonNumber -side $packside -fill x
	incr variableIndex
	incr buttonNumber
    }
    if {$allNone} {
       set setBody {
          global $var
          set $var 1
       }
       set setCmd [list foreach var $variableList $setBody]
       set clrBody {
          global $var
          set $var 0
       }
       set clrCmd [list foreach var $variableList $clrBody]
       APSButton .all -parent $parent$widget.frame -text All -size small \
          -command $setCmd
       APSButton .none -parent $parent$widget.frame -text None -size small \
          -command $clrCmd
   }
}

proc APSFrameHelp {} {
    return "Usage: APSFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-label <string>\]\t\n\[-width <num>\]\t\n\[-height <num>\]\t\n\[-orientation <string>\] where <string> is horizontal or vertical (default)\n\[-geometry <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlabel\n\tframe\n"
}

proc APSFrame {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top"
    set label ""
    set name ""
    set width 150
    set height 120
    set orientation vertical
    set geometry ""
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption label name width height \
			       orientation geometry contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }

    if {$parent == ""} {
	toplevel $widget -bd 1
	wm title $widget $name
	if {$geometry != ""} {
	    wm geometry $widget $geometry
	}
    } else {
	frame $parent$widget -bd 1
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }
    if {$orientation == "vertical"} {
	set packside top
    } else {
	set packside left
    }
    if {$label != ""} {
	label $parent$widget.label -text $label
	pack $parent$widget.label -side $packside -fill x
    }
    frame $parent$widget.frame -relief ridge -bd 2 -width $width \
      -height $height
    pack $parent$widget.frame -side $packside -fill both
}

proc APSFrameGridHelp {} {
    return "Usage: APSFrameGrid widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-xList <list>\]\t\n\[-yList <list>\]\t\n\[-width <num>\]\t\n\[-height <num>\]\t\n\[-relief <string>\]\t\n\[-bd <string>\]\t\n\[-contextHelp <string>\]\t\n\nIf you supply just -xList or just -yList, it creates:\n\$parent\$widget.\n\t<name1>,...,<namen> where names are from xList or yList\n\nIf both -xList and -yList specified, it creates:\n\$parent\$widget.\n\t<xname1>.<yname1>,...,<xname1>.<ynamen>\n\t...\n\t<xnamen>.<yname1>,...,<xnamen>.<ynamen>\n"
}
proc APSFrameGrid {widget args} {
    global apsContextHelp
    set xList ""
    set yList ""
    set parent ""
    set width 50m
    set height 8m
    set packOption "-side top -fill both -expand 1"
    set noPack 0
    set relief flat
    set bd 1
    set contextHelp ""
    APSStrictParseArguments {parent xList yList packOption noPack width \
			       height relief bd contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }
    if {$parent == ""} {
	toplevel $widget -bd 1
	wm title $widget $label
    } else {
	frame $parent$widget -bd 1
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }
    if {[llength $xList] == 0} {
       foreach yname $yList {
          frame $parent$widget.$yname -width $width -height $height \
              -relief $relief -bd $bd
          pack $parent$widget.$yname -side top -fill both -expand 1
       }       
    } elseif {[llength $yList] == 0} {
       foreach xname $xList {
          frame $parent$widget.$xname -width $width -height $height \
              -relief $relief -bd $bd
          pack $parent$widget.$xname -side left -fill both -expand 1
       }       
    } else {
       foreach xname $xList {
	   frame $parent$widget.$xname 
           pack $parent$widget.$xname -side left -fill both -expand 1
           foreach yname $yList {
	     frame $parent$widget.$xname.$yname -width $width -height $height \
                  -relief $relief -bd $bd
	     pack $parent$widget.$xname.$yname -side top -fill both -expand 1
	   }
       }
    }
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.6  1996/01/16 21:19:58  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSDialogBoxHelp {} {
    return "Usage: APSDialogBox widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string>\]\t\n\[-width <string>\]\t\n\[-height <string>\]\t\n\[-contextHelp <string\]\n\t\nCreates\t\$parent\$widget.\n\tuserFrame\n\tbuttonRow.ok.button\n\tbuttonRow.cancel.button"
}

proc APSDialogBox {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "NoName"
    set width 200
    set height 100
    set contextHelp ""
    APSStrictParseArguments {parent name noPack packOption width height contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    if {$parent == ""} {
	toplevel $widget
	wm title $widget $name
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    frame $parent$widget.userFrame -relief raised -bd 1 -width $width \
      -height $height
    pack $parent$widget.userFrame -side top -fill both -expand 1

    frame $parent$widget.buttonRow -relief raised -bd 1
    pack $parent$widget.buttonRow -side bottom -fill x 

    APSButton .ok -parent $parent$widget.buttonRow -text OK \
      -command "destroy $parent$widget" -highlight 1 -contextHelp \
      "Done interacting with this dialog box. Remove window."
    APSButton .cancel -parent $parent$widget.buttonRow -text Cancel \
      -command "destroy $parent$widget" -contextHelp \
      "Cancel interaction with dialog box. Remove window."
    APSDisableButton $parent$widget.buttonRow.ok.button

}

proc APSDialogBoxR {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "NoName"
    set width 200
    set height 100
    set contextHelp ""
    APSStrictParseArguments {parent name noPack packOption width height contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    if {$parent == ""} {
	toplevel $widget
	wm title $widget $name
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    frame $parent$widget.userFrame -relief raised -bd 1 -width $width \
      -height $height
    pack $parent$widget.userFrame -side top -fill both -expand 1

    frame $parent$widget.buttonRow -relief raised -bd 1
    pack $parent$widget.buttonRow -side bottom -fill x 

    APSButton .ok -parent $parent$widget.buttonRow -text OK \
      -command "destroy $parent$widget" -highlight 1 -contextHelp \
      "Done interacting with this dialog box. Remove window."
    APSButton .cancel -parent $parent$widget.buttonRow -text Cancel \
      -command "destroy $parent$widget" -contextHelp \
      "Cancel interaction with dialog box. Remove window."
    APSDisableButton $parent$widget.buttonRow.ok.button

  bind $parent$widget <Visibility> "DCTraise $parent$widget"
}



proc APSDialogBoxAddButtonHelp {} {
    return "Usage: APSDialogBoxAddButton widget \n\[-parent <string>\]\t\n\[-text <string>\]\t\n\[-command <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent.buttonRow.\$widget"
}
proc APSDialogBoxAddButton {widget args} {
    set parent ""
    set text "NoName"
    set command ""
    set contextHelp ""
    APSStrictParseArguments {parent text command contextHelp}
    APSButton $widget -parent $parent.buttonRow -text $text \
      -command $command -contextHelp $contextHelp
}
    
proc APSWindowHelp {} {
    return "Usage: APSWindow widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string>\]\t\n\[-contextHelp <string\]\n\t\nCreates\t\$parent\$widget.\n\tuserFrame\n\tbuttonRow.close.button\n"
}

proc APSWindow {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "NoName"
    set contextHelp ""
    APSStrictParseArguments {parent name noPack packOption contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    set windowExists [winfo exists $parent$widget]
    if {$parent == ""} {
	if {$windowExists} {
	    raise $widget
	} else {
	    toplevel $widget
	    wm title $widget $name
	}
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    if {!$windowExists} {
	frame $parent$widget.userFrame -relief raised -bd 2 \
	  -width 200 -height 100
	pack $parent$widget.userFrame -side top -fill both -expand 1
	
	frame $parent$widget.buttonRow -relief raised -bd 2
	pack $parent$widget.buttonRow -side bottom -fill x
	
	APSButton .close -parent $parent$widget.buttonRow -text Close \
	  -command "destroy $parent$widget" -highlight 1 -contextHelp \
	  "Close Window."
    }
}
#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.16  1996/02/07 23:23:27  saunders
# Restricted form of -unixCommand to permit process aborting.
#
# Revision 1.15  1996/01/31  18:59:42  saunders
# Callback now performed at global level.
#
# Revision 1.14  1996/01/23  20:51:19  saunders
# Improved scroll behavior added by Ken Evans. Autoraise disabled.
#
# Revision 1.13  1996/01/16  23:40:50  saunders
# Further refinement of scrolling behavior.
#
# Revision 1.12  1996/01/16  23:13:36  saunders
# Window now forces scroll to bottom when new command is used on
# an existing exec log window.
#
# Revision 1.11  1996/01/16  21:20:00  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSExecLogHelp {} {
    return "Usage: APSExecLog widget \n\[-name <string>\]\t\n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-unixCommand <string>\]\t\n\[-callback <script>\]\t\n\[-width <string>\]\t\n\[-height <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tuserFrame\n\tbuttonRow.ok.button\n\tbuttonRow.cancel.button\n\tbuttonRow.print.button\n\tbuttonRow.enscript.button\n\tuserFrame.text.text\n\tuserFrame.text.scroll"
}

proc APSExecLog {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set unixCommand ""
    set contextHelp ""
    set callback ""
    set name "Unix Command Execution"
    set width 40
    set height 15
    APSStrictParseArguments {parent unixCommand noPack packOption contextHelp \
		     callback name width height}

    set windowExists [winfo exists $parent$widget]

    if {!$windowExists} {
	APSDialogBox $widget -parent $parent -name $name \
	  -noPack $noPack -packOption $packOption \
	  -contextHelp $contextHelp
	
	APSScrolledText .text -parent $parent$widget.userFrame -width $width \
	  -height $height
    } else {
	wm title $parent$widget $name
    }

    set log $parent$widget.userFrame.text.text

    if {!$windowExists} {
	# Put in a null command until the process pipe is open
	APSDialogBoxAddButton .abort -parent $parent$widget -text Abort \
	  -command "" \
	  -contextHelp "Abort command execution, but leave window up."
	
	APSDialogBoxAddButton .print -parent $parent$widget -text Print \
	  -command "APSExecLogPrint $log lpr" \
	  -contextHelp "Print contents of window using lpr command."
	
	APSDialogBoxAddButton .enscript -parent $parent$widget -text Enscript \
	  -command "APSExecLogPrint $log enscript" \
	  -contextHelp "Print contents of window using enscript command."
    } else {
	APSDisableButton $parent$widget.buttonRow.ok.button
    }

#    set cmd1 "csh -f -c \"$unixCommand\" |& cat -u"
    set cmd1 "$unixCommand |& cat -u"
    # If at end now, then reset to end afterward
    if {[lindex [$log yview] 1] == 1} {
	set gotoend 1
    } else {
	set gotoend 0
    }
    if [catch {open |$cmd1 r} input] {
	$log insert end $input\n
	if {$gotoend} {$log see end}
	set pipePid -1
    } else {
	set pipePid [pid $input]
	fileevent $input readable \
	  [list APSExecLogCallback $parent$widget $input $log $callback]
	$log insert end $unixCommand\n
	if {$gotoend} {$log see end}
    }
    $parent$widget.buttonRow.cancel.button configure -command \
      [list APSExecLogStop $parent$widget $input $pipePid]
    $parent$widget.buttonRow.abort.button configure \
      -command [list APSExecLogAbort $input $pipePid]
}

proc APSExecLogCallback {widget input log callback} {
    if [eof $input] {
	catch {close $input}
	APSEnableButton $widget.buttonRow.ok.button
	# used to be 'eval callback', really want it done at global level
	uplevel #0 $callback
    } else {
	# Add new output, but don't disturb view if user has scrolled up
	set oldend [lindex [$log yview] 1]
	gets $input line
	$log insert end $line\n
	if {$oldend == 1} {
	    $log see end
	}
    }
}

proc APSExecLogStop {widget input pipePid} {
    catch "exec kill $pipePid"
    catch {close $input}
    destroy $widget
}

proc APSExecLogAbort {input pipePid} {
    if [catch "exec kill $pipePid" result] {
	puts stderr $result
    }
    catch {close $input}
}

proc APSExecLogPrint {textwidget cmd} {
    catch {exec echo [$textwidget get 1.0 end] | $cmd}
}

proc APSExecHelp {} {
    return "Usage: APSExec\n\[-unixCommand <string>\]\t\n\[-callback <script>\]\t\n\[-outputVariable <variable>\]\t\n\nExecutes command given by -unixCommand option.\nIf -callback is given, <script> will be executed upon\ncompletion of unixCommand. Output of unixCommand\nis normally thrown away. If -outputVariable is given,\ncommand output will be stored in global <variable>."
}
proc APSExec {args} {
    set unixCommand ""
    set callback ""
    set outputVariable ""
    APSStrictParseArguments {unixCommand callback outputVariable}

    if {$outputVariable != ""} {
	set saveOutput 1
	global $outputVariable
    } else {
	set saveOutput 0
    }
#    set cmd1 "csh -f -c \"$unixCommand\" |& cat"
    set cmd1 "$unixCommand |& cat -u"
    if [catch {open |$cmd1 r} input] {
	if {$saveOutput} {
	    set $outputVariable $input
	}
    } else {
	fileevent $input readable \
	  [list APSExecCallback $input $callback $outputVariable $saveOutput]
	set $outputVariable ""
    }
}
proc APSExecCallback {input callback outputVariable saveOutput} {
    if {$saveOutput} {
	global $outputVariable
    }
    if [eof $input] {
	catch {close $input}
	uplevel #0 $callback
	# used to be 'eval $callback', really want this done at global level
    } else {
	gets $input line
	if {$saveOutput} {
	    set $outputVariable [subst $$outputVariable]$line\n
	}
    }
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.12  1996/02/28 19:54:30  saunders
# Entirely new dialog submitted by Ken Evans. Much more Motif-like.
#
# Revision 1.11  1996/01/21  21:12:47  saunders
# Added -listFilter option to APSFileSelectDialog. Improved behavior
# of dialog when various patterns are entered in File: widget.
#
# Revision 1.10  1996/01/17  17:23:12  saunders
# Added width option and made default 80, not 40. Added Print and Enscript
# buttons.
#
# Revision 1.9  1996/01/16  21:20:02  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSFileSelectDialogHelp {} {
    return "Needs to be fixed"
}

proc APSFileSelectDialog {widget args} {
    global apsContextHelp
    global apsFileSelect
    set parent ""
    set width 50
    set checkValidity 1
    set noPack 0
    set packOption "-side top -fill x"
    if {[info exists apsFileSelect(dir)]} {
	set listDir $apsFileSelect(dir)
    } else {
	set listDir ""
    }
    if {[info exists apsFileSelect(pattern)]} {
	set listFilter $apsFileSelect(pattern)
    } else {
	set listFilter "*"
    }
    set title "Select File:"
    set contextHelp ""

    APSStrictParseArguments {parent width checkValidity noPack packOption \
			       listDir listFilter title contextHelp}

    # Check if there is an APSFileSelectDialog up
    if {[info exists apsFileSelect]} {
	if {!$apsFileSelect(done)} {
	    APSAlertBox .alert -errorMessage \
	      "There is already a File Selection\
              in use.  Close it to use this one."
	    return
	}
    }

    # Set context help
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    # Set validity check
    set apsFileSelect(validity) $checkValidity

    APSDialogBox $widget -parent $parent -name "$title" \
      -noPack $noPack -packOption $packOption -contextHelp $contextHelp
    set userFrame $parent$widget.userFrame

    # Create a read-only entry for the current directory
#     APSLabeledOutput .dir -parent $userFrame -label Path: \
#       -textVariable apsFileSelect(dir) -width $width -contextHelp \
#       "This displays the currently selected directory."
#     set apsFileSelect(dirEnt) $userFrame.dir.entry

    # Create an entry for the filter. Value kept in apsFileSelect(filter).
    APSLabeledEntry .filter -parent $userFrame -label Filter: \
      -textVariable apsFileSelect(filter) -width $width -contextHelp \
      "The files that appear under Files will be those that match this\
       filter.  Typically the filter should contain wildcards.  Enter\
       the desired filter or build it by clicking in the Directories\
       list.  <Return> or the clicking Filter button will generate new\
       Directories and Files lists using the filter."
    eval pack $userFrame.filter.entry [pack info $userFrame.filter.entry] -expand 1 -fill x

    # Create a frame to hold the listboxes
    APSFrame .listFrame -parent $userFrame \
      -packOption "-side top -fill both -expand 1"
    set w $userFrame.listFrame.frame
    eval pack $w [pack info $w] -expand 1 -fill both

    # Create a listbox to hold the directories
    APSFrame .dirlistFrame -parent $w -label Directories \
      -packOption "-side left -fill both -expand 1"
    set w1 $w.dirlistFrame.frame
    # Repack to expand
    eval pack $w1 [pack info $w1] -expand 1 -fill both

    APSScrolledList .dirList -parent $w1 \
      -contextHelp \
      "A click on a directory will add it to the\
       filter.  A double click will add it to the filter and then\
       process the filter.  If the focus is in the Directories\
       listbox then <Space> is the same as a click, and <Return>\
       is the same as a double click."
    set dirlb $w1.dirList.listbox

    # Create a listbox to hold the filenames
    APSFrame .filelistFrame -parent $w -label Files \
      -packOption "-side left -fill both -expand 1"
    set w1 $w.filelistFrame.frame
    # Repack to expand
    eval pack $w1 [pack info $w1] -expand 1 -fill both

    APSScrolledList .fileList -parent $w1 \
      -contextHelp \
      "A click on a file will add it to the selection.  A\
       double click will add it to the selection and process\
       the selection.  If the focus is in the Files listbox then\
       <Space> is the same as a click, and <Return> is the same as\
       a double click."
    set filelb $w1.fileList.listbox

    # Create an entry for the selection. Value kept in apsFileSelect(selection).
    APSLabeledEntry .select -parent $userFrame -label Selection: \
      -textVariable apsFileSelect(select) -width $width -contextHelp \
      "Your selection appears here.  You may edit it or build it by\
       clicking in the Directories or Files lists.  A <Return> will\
       process the  selection.  Use the <Esc> key for filename\
       completion.  (The selection may contain wildcards when used for\
       filename completion.)  All possible completions will be displayed."
    eval pack $userFrame.select.entry [pack info $userFrame.select.entry] -expand 1 -fill x

    # Add filter button
    APSButton .filter -parent $parent$widget.buttonRow -text Filter \
      -packOption "-after $parent$widget.buttonRow.ok\
        -side left -padx 1 -pady 1" \
      -command "APSFileSelectDialogFilter $dirlb $filelb" -contextHelp \
      "Generate new Directories and Files lists using the filter."

    # Configure the OK and Cancel buttons
    $parent$widget.buttonRow.ok.button configure \
      -command "APSFileSelectDialogOK $dirlb $filelb"
    APSEnableButton $parent$widget.buttonRow.ok.button
    $parent$widget.buttonRow.cancel.button configure \
      -command APSFileSelectDialogCancel
    
    APSFileSelectDialogBindings \
      $userFrame.filter.entry $dirlb $filelb $userFrame.select.entry

    # Initialize variables and list the directory
    if {[string length $listDir] == 0} {
	set apsFileSelect(pattern) $listFilter
	set dir [pwd]
	set apsFileSelect(filter) "$dir/$listFilter"
	set apsFileSelect(select) "$dir/"
    } else {
	set apsFileSelect(pattern) $listFilter
	set dir $listDir
	if {[string compare $dir "/"]} {
	    # Not top of tree
	    set apsFileSelect(filter) "$dir/$listFilter"
	    set apsFileSelect(select) "$dir/"
	} else {
	    set apsFileSelect(filter) "/$listFilter"
	    set apsFileSelect(select) "/"
	}
    }
    set apsFileSelect(dir) {}
    set apsFileSelect(done) 0

    # Wait for the listbox to be visible
    tkwait visibility $dirlb
    tkwait visibility $filelb
    APSFileSelectDialogList $dirlb $filelb $dir

    tkwait variable apsFileSelect(done)
    destroy $parent$widget
    return $apsFileSelect(select)
}

proc APSFileSelectDialogBindings { f dirlb filelb s } {
    global apsFileSelect
    
    # Filter bindings
    bind $f <Return> "APSFileSelectDialogFilter $dirlb $filelb"

    # Directory Listbox bindings
    bind $dirlb <space> "APSFileSelectDialogTake dir $dirlb $filelb"
    bind $dirlb <Button-1> "APSFileSelectDialogClick dir $dirlb $filelb %y"
    bind $dirlb <Return> \
      "APSFileSelectDialogTake dir $dirlb $filelb;\
       APSFileSelectDialogFilter $dirlb $filelb"
    bind $dirlb <Double-Button-1> \
      "APSFileSelectDialogDoubleClick dir $dirlb $filelb %y"

    # File Listbox bindings
    bind $filelb <space> "APSFileSelectDialogTake file $dirlb $filelb"
    bind $filelb <Button-1> "APSFileSelectDialogClick file $dirlb $filelb %y"
    bind $filelb <Return> \
      "APSFileSelectDialogTake file $dirlb $filelb;\
      APSFileSelectDialogOK $dirlb $filelb"
    bind $filelb <Double-Button-1> \
      "APSFileSelectDialogDoubleClick file $dirlb $filelb %y"

    # Selection bindings
    bind $s <Return> "APSFileSelectDialogOK $dirlb $filelb"
    bind $s <KeyPress-Escape> "APSFileSelectDialogComplete $dirlb $filelb"

    focus $s
}
    
proc APSFileSelectDialogFilter { dirlb filelb } {
    global apsFileSelect

    # If the filter is a directory, add *
    if {[file isdirectory $apsFileSelect(filter)]} {
	if {[regexp {/$} $apsFileSelect(filter)]} {
	    set apsFileSelect(filter) "$apsFileSelect(filter)*"
	} else {
	    set apsFileSelect(filter) "$apsFileSelect(filter)/*"
	}
    }
    # Separate out a path if given and use it for the path
    set pattern [file tail $apsFileSelect(filter)]
    set dirname [file dirname $apsFileSelect(filter)]
    set pwd [pwd]
    set pwdparent [file dirname $pwd]
    # Trim ../ (the parent) out of the pathname
    if {[regsub {^\.\.} $dirname $pwdparent newpath] != 0} {
	set dirname $newpath
    }
    # Trim ./ (the cwd) out of the pathname
    if {[regsub {^\.} $dirname $pwd newpath] != 0} {
	set dirname $newpath
    }
    # If the pattern is blank, make it be *
    if {![string compare $pattern ""]} {
	set pattern *
    }
    if {[string compare $dirname "/"]} {
	# Not top of tree
	set apsFileSelect(dir) $dirname
	set apsFileSelect(pattern) $pattern
	set apsFileSelect(filter) $dirname/$pattern
	set apsFileSelect(select) $dirname/
    } else {
	set apsFileSelect(dir) /
	set apsFileSelect(pattern) $pattern
	set apsFileSelect(filter) /$pattern
	set apsFileSelect(select) /
    }

    APSFileSelectDialogList $dirlb $filelb $apsFileSelect(dir)
}

proc APSFileSelectDialogList { dirlb filelb dir {files {}} } {
    global apsFileSelect

    set apsFileSelect(dir) $dir

# Directory
    $dirlb delete 0 end
    $filelb delete 0 end
    # Check for bogus names
    if ![file isdirectory $dir] {
	$dirlb insert 0 "Bad Directory"
	return
    }
    # Give the user some feedback
    $dirlb insert 0 Listing...
    $filelb insert 0 Listing...
    update idletasks
    # Find files
    set dirfiles [glob -nocomplain "$dir/*"]
    $dirlb delete 0
    $dirlb insert end ./
    $dirlb insert end ../
    # Sort out the directories
    foreach f [lsort $dirfiles] {
	if {[file isdirectory $f]} {
	    $dirlb insert end [file tail $f]/
	}
    }
    
# File
    # Find files
    if {[string length $files] == 0} {
	set files [glob -nocomplain $apsFileSelect(filter)]
    }
    $filelb delete 0
    # Sort out the files
    foreach f [lsort $files] {
	if {[file isfile $f]} {
	    $filelb insert end [file tail $f]
	}
    }
}

proc APSFileSelectDialogOK { dirlb filelb} {
    global apsFileSelect

    set pwd [pwd]
    if {[string compare $pwd "/"]} {
	# Not the top level
	set pwdparent "[file dirname $pwd]/"
	set pwd "$pwd/"
    } else {
	set pwdparent /
    }
    # Trim ./ (the cwd) out of the pathname
    if {[regsub {^\./} $apsFileSelect(select) $pwd newpath] != 0} {
	set apsFileSelect(select) $newpath
	set apsFileSelect(dir) [file dirname $newpath]
    }
    # Trim ../ (the parent) out of the pathname
    if {[regsub {^\.\./} $apsFileSelect(select) $pwdparent newpath] != 0} {
	set apsFileSelect(select) $newpath
	set apsFileSelect(dir) [file dirname $newpath]
    }
    # Check that it is not a directory
    if {[file isdirectory $apsFileSelect(select)]} {
	APSAlertBox .alert -errorMessage \
	  "You have choosen a directory, not a file.  Will use this\
           directory to refilter."
	set apsFileSelect(filter) \
	  [string trimright $apsFileSelect(select) /]/$apsFileSelect(pattern)
	APSFileSelectDialogFilter $dirlb $filelb
	return
    }
    # Glob the result
    set globsel [glob -nocomplain $apsFileSelect(select)]
    if {[llength $globsel] > 1} {
	APSAlertBox .alert -errorMessage \
	  "Selection is not unique.  Will do filename completion."
	APSFileSelectDialogComplete $dirlb $filelb
	return
    } elseif {[llength $globsel] < 1} {
	if {![file exists $apsFileSelect(select)]} {
	    APSAlertBox .alert -errorMessage \
	      "Selection is not a valid file."
	    return
	}
    } else {
	set apsFileSelect(select) $globsel
    }
    # Check validity
    if {$apsFileSelect(validity) } {
	if {![file exists $apsFileSelect(select)]} {
	    APSAlertBox .alert -errorMessage \
	      "Selection is not a valid file."
	    return
	}
    }
    # Selection is acceptable, finish up
    # All done
    set apsFileSelect(done) 1
    return
}

proc APSFileSelectDialogCancel {} {
    global apsFileSelect
    set apsFileSelect(done) 1
    set apsFileSelect(select) {}
}

proc APSFileSelectDialogDoubleClick { mode dirlb filelb y } {
    switch -exact $mode {
	dir {
	    set result [$dirlb get [$dirlb nearest $y]]
	    APSFileSelectDialogClick $mode $dirlb $filelb $y
	    APSFileSelectDialogFilter $dirlb $filelb
	}
	file {
	    set result [$filelb get [$filelb nearest $y]]
	    APSFileSelectDialogClick $mode $dirlb $filelb $y
	    APSFileSelectDialogOK $dirlb $filelb
	}
    }
    global apsFileSelect
}

proc APSFileSelectDialogClick { mode dirlb filelb y } {
    # Take the item the user clicked on
    global apsFileSelect
    
    switch -exact $mode {
	dir {
	    set result [$dirlb get [$dirlb nearest $y]]
	    if {![string compare $result "./"]} {
		# Is the current directory
		set apsFileSelect(filter)  \
		  $apsFileSelect(dir)/$apsFileSelect(pattern)
	    } elseif {![string compare $result "../"]} {
		# Is the parent directory
		set dirname [file dirname $apsFileSelect(dir)]
		if {[string compare $dirname "/"]} {
		    # Not top of tree
		    set apsFileSelect(filter) \
		      $dirname/$apsFileSelect(pattern)
		} else {
		    set apsFileSelect(filter) /$apsFileSelect(pattern)
		}
	    } elseif {[regexp {/$} $result]} {
		# Is a subdirectory
		set dirname [file dirname $apsFileSelect(dir)]
		if {[string compare $dirname "/"]} {
		    # Not top of tree
		    set apsFileSelect(filter) \
		      $apsFileSelect(dir)/$result$apsFileSelect(pattern)
		} else {
		    set apsFileSelect(filter) /$result$apsFileSelect(pattern)
		}
	    }
	}
	file {
	    set result [$filelb get [$filelb nearest $y]]
	    set dirname $apsFileSelect(dir)
	    if {[string compare $dirname "/"]} {
		# Not top of tree
		set apsFileSelect(select) $apsFileSelect(dir)/$result
	    } else {
		set apsFileSelect(select) /$result
	    }
	}
    }
}

proc APSFileSelectDialogTake { mode dirlb filelb } {
    # Take the currently selected list item
    global apsFileSelect
    
    switch -exact $mode {
	dir {
	    set result [$dirlb get [$dirlb curselection]]
	    if {![string compare $result "./"]} {     
		# Is the current directory
		set apsFileSelect(filter) \
		  $apsFileSelect(dir)/$apsFileSelect(pattern)
	    } elseif {![string compare $result "../"]} {     
		# Is the parent directory
		set dirname [file dirname $apsFileSelect(dir)]
		if {[string compare $dirname "/"]} {     
		    # Not top of tree
		    set apsFileSelect(filter) $dirname/$apsFileSelect(pattern)
		} else {
		    set apsFileSelect(filter) /$apsFileSelect(pattern)
		}
	    } elseif {[regexp {/$} $result]} {     
		# Is a subdirectory
		set dirname [file dirname $apsFileSelect(dir)]
		if {[string compare $dirname "/"]} {
		    # Not top of tree
		    set apsFileSelect(filter) \
		      $apsFileSelect(dir)/$result$apsFileSelect(pattern)
		} else {
		    set apsFileSelect(filter) /$result$apsFileSelect(pattern)
		}
	    }
	}
	file {
	    set result [$filelb get [$filelb curselection]]
	    set dirname $apsFileSelect(dir)
	    if {[string compare $dirname "/"]} {
		# Not top of tree
		set apsFileSelect(select) $apsFileSelect(dir)/$result
	    } else {
		set apsFileSelect(select) /$result
	    }
	}
    }
}

proc APSFileSelectDialogComplete { dirlb filelb } {
    # Do file name completion
    global apsFileSelect
    
    # See what files are there
    set globlist [glob -nocomplain $apsFileSelect(select)*]
    if {[llength $globlist] == 0} {
	# No match
	$filelb delete 0 end
	return
    }
    if {[llength $globlist] == 1} {
	# Matched a single file
	set apsFileSelect(select) $globlist
    }
    set dir [file dirname $apsFileSelect(select)]
    APSFileSelectDialogList $dirlb $filelb $dir $globlist
}

proc APSFileDisplayWindowHelp {} {
    return "Usage: APSFileDisplayWindow widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-comment <string>\]\t\n\[-fileName <string>\]\t\n\[-deleteOnClose 1\]\t\n\[-width <string>\]\t\n\[-contextHelp <string\]\n\t\nCreates\t\$parent\$widget.\n\tuserFrame\n\tuserFrame.file.text\n\tbuttonRow.close.button\n"
}

proc APSFileDisplayWindow {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top"
    set comment ""
    set fileName ""
    set deleteOnClose 0
    set width 80
    set contextHelp ""
    APSStrictParseArguments {parent comment noPack packOption fileName \
			       deleteOnClose width contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    set name "Display File: $comment ($fileName)"
    APSWindow $widget -parent $parent -name $name
    
    APSScrolledText .file -parent $parent$widget.userFrame -width $width
    set text $parent$widget.userFrame.file.text

    APSDialogBoxAddButton .print -parent $parent$widget -text Print \
      -command "APSFileDisplayPrint $text lpr" \
      -contextHelp "Print contents of window using lpr command."
    
    APSDialogBoxAddButton .enscript -parent $parent$widget -text Enscript \
      -command "APSFileDisplayPrint $text enscript" \
      -contextHelp "Print contents of window using enscript command."

    set f [open $fileName r]
    while {![eof $f]} {
	$text insert end [read $f 1000]
    }
    close $f

    # Add file deletion to close button callback
    if {$deleteOnClose} {
	set cmd {
	    catch {exec rm $fileName}
	    destroy $parent$widget
	}
	$parent$widget.buttonRow.close.button configure -command \
	  [subst $cmd]
    }
}
    
proc APSFileDisplayPrint {textwidget cmd} {
    catch {exec echo [$textwidget get 1.0 end] | $cmd}
}
    





#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.1  1996/02/22 21:57:53  borland
# First version committed.  A routine to find standard monitoring output files
# between a range of dates.
#

#
# Finds files between given dates, with defaults for sddsmonitor files
# for the data logging jobs.
#
proc APSFindFilesBetweenDates {args} {
    set rootname ""
    set directory "."
    set tailsOnly 0
    set startDateList {}
    set endDateList {}
    set extensionList {"" ".\[0-9\]\[0-9\]\[0-9\]\[0-9\]" ".gz"}
    APSStrictParseArguments {rootname directory startDateList endDateList extensionList tailsOnly}
    if {[string length $rootname]==0 || [llength $startDateList]==0 || [llength $endDateList]==0} {
        return 0
    }
    set startYear [APSMakeFourDigitYear [lindex $startDateList 0]]
    set endYear   [APSMakeFourDigitYear [lindex $endDateList 0  ]]
    scan [lindex $startDateList 1] "%ld" startJDay
    scan [lindex $endDateList   1] "%ld" endJDay

    set fileList {}
    for {set year $startYear} {$year<=$endYear} {incr year} {
        if {$year!=$endYear} {
            set toYearEnd 1
            set jDayLimit [expr 365+([APSLeapYear $year]?1:0)]
        } else {
            set toYearEnd 0
            set jDayLimit $endJDay
        }
        set yearList [glob -nocomplain $directory/${rootname}-${year}-???-????*]
        if [llength $yearList] {
            if {$startJDay==1 && $toYearEnd==1} {
                set fileList [concat $fileList $yearList]
            } else {
                set newList {}
                for {set jDay $startJDay} {$jDay<=$jDayLimit} {incr jDay} {
                    set jDay3 [format %03ld $jDay]
                    while [llength $yearList] {
                        set index [lsearch -glob $yearList $directory/${rootname}-${year}-${jDay3}-????*]
                        if $index==-1 break
                        set newName [lindex $yearList $index]
                        if $tailsOnly {
                            set newName [file tail $newName]
                        }
                        if [llength $extensionList] {
                            set match 0
                            set extension [file extension $newName]
                            foreach extensionPattern $extensionList {
                                if [string match $extensionPattern $extension] {
                                    set match 1
                                    break
                                }
                            }
                            if $match {
                                lappend newList $newName
                            }
                        } else {
                            lappend newList $newName
                        }
                        set yearList [lreplace $yearList $index $index]
                    }
                }
                set fileList [concat $fileList $newList]
            }
        }
        set startJDay 1
    }
    return $fileList
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.8  1996/02/13 15:52:10  saunders
# APSInfoWindow uses -width option for short messages, and -aspect option
# for long ones.
#
# Revision 1.7  1996/02/06  21:34:40  saunders
# Fixed so that aspect ratio is honored.
#
# Revision 1.6  1996/01/16  21:20:04  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSInfoWindowHelp {} {
    return "Usage: APSInfoWindow widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string\]\t\n\[-infoMessage <string>\]\t\n\[-modal 1]\t\n\[-contextHelp <string>\]\n\nCreates\t\$parent\$widget.\n\tmsg\n\tbuttonRow.ok.button\n\t"
}

proc APSInfoWindow {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "UntitledInfo"
    set infoMessage "No Info"
    set modal 0
    set contextHelp ""

    APSStrictParseArguments {parent noPack packOption name infoMessage \
			       contextHelp modal}
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    set windowExists [winfo exists $parent$widget]
    if {$parent == ""} {
	if {$windowExists} {
	    raise $widget
	} else {
	    toplevel $widget
	    wm title $widget $name
	}
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    if {$windowExists} {
	$parent$widget.msg configure -text "$infoMessage\n                            "
    } else {
	if {[string length $infoMessage] < 40} {
	    message $parent$widget.msg -relief raised -justify left -bd 2 \
	      -width 40c -text "$infoMessage"
	} else {
	    message $parent$widget.msg -relief raised -justify left -bd 2 \
	      -aspect 200 -text "$infoMessage"
	}
	pack $parent$widget.msg -side top -fill both -expand 1
	
	frame $parent$widget.buttonRow -relief raised -bd 2
	pack $parent$widget.buttonRow -side bottom -fill x
	
	APSButton .ok -parent $parent$widget.buttonRow -text OK \
	  -command "destroy $parent$widget" -highlight 1
    }
    if {$modal == 1} {
	focus $parent$widget
	grab $parent$widget
	tkwait window $parent$widget
    }
}    


#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.14  1996/03/22 22:40:09  saunders
# Added APSMenuFrameFindSubMenu procedure.
#
# Revision 1.13  1996/02/14  16:40:09  saunders
# Removed APSMp dummy routines from APSMisc.tcl. This will fix the problem
# with using APSUniqueName from within a machine procedure.
#
# Revision 1.12  1996/02/13  22:35:54  saunders
# Added Close Submenus and Kill buttons to main menu widget.
#
# Revision 1.11  1996/02/13  22:16:28  saunders
# Fixed help text for menu button.
#
# Revision 1.10  1996/02/13  22:00:18  saunders
# Added "command" option to APSMenuFrameMake.
#
# Revision 1.9  1996/02/13  21:11:41  saunders
# Improved APSMenuFrameMake procedure.
#
# Revision 1.8  1996/02/13  20:45:57  saunders
# Added "Close SubMenus" button and "Kill" button.
#
# Revision 1.7  1996/02/09  23:47:41  saunders
# Minor change.
#
# Revision 1.6  1996/02/09  21:24:52  saunders
# Added -geometry option to APSFrame and APSMenuFrame to allow relative
# positioning of submenus.
#
# Revision 1.5  1996/02/09  18:09:02  saunders
# Stupid bug fix.
#
# Revision 1.4  1996/02/09  18:06:16  saunders
# Removed APSFrame title. Window manager title is sufficient.
#
# Revision 1.3  1996/02/09  17:16:41  saunders
# Added name to Frame.
#
# Revision 1.2  1996/02/09  17:11:28  saunders
# First functional release of new menu procedure.
#
# Revision 1.1  1996/02/08  23:35:59  saunders
# New procedure APSMenuFrame. Not quite operational yet.
#
#

proc APSMenuFrameHelp {} {
    return "Usage: APSMenuFrame widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-menuList <string>\]\t\n\[-parentLabel <string>\]\t\n\[-subMenu 1\]\t\n\[-contextHelp <string\]\n\t\n"
}

proc APSMenuFrame {widget args} {
    global apsContextHelp apsMenuFrame
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set menuList ""
    set parentLabel ""
    set subMenu ""
    set closeButton 0
    set geometry ""
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption menuList subMenu \
			       closeButton parentLabel geometry contextHelp}

    if {[winfo exists $parent$widget]} {
	raise $parent$widget
	return
    }
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    if {$parentLabel != ""} {
	set labelDelimiter :
	set labelPrefix $parentLabel$labelDelimiter
    } else {
	set labelPrefix ""
    }
    set label ""
    set maxWidth 0
    foreach item $menuList {
	switch [lindex $item 0] {
	    label {
		set label "$labelPrefix[lindex $item 1]"
		set len [string length $label]
	    }
	    exec {
		set text "[lindex $item 1]..."
		set len [string length $text]
	    }
	    secureExec {
		set text "[lindex $item 1]..."
		set len [string length $text]
	    }
	    command {
		set text [lindex $item 1]
		set len [string length $text]
	    }
	    menu {
		set text "[lindex $item 1]->"
		set len [string length $text]
	    }
	    default {

	    }
	}
	if {$len > $maxWidth} {
	    set maxWidth $len
	}
    }
    set maxWidth [expr $maxWidth + 2]

    APSFrame $widget -name $label -parent $parent -packOption $packOption \
      -noPack $noPack -geometry $geometry

    # Initialize list for keeping track of sub-menu widgets and exec pids
    if {![info exists apsMenuFrame($parent$widget)]} {
	set apsMenuFrame($parent$widget) {}
    }
    foreach item $menuList {
	switch [lindex $item 0] {
	    exec {
		set text "[lindex $item 1]..."
		set text [APSMenuFramePadLength $text $maxWidth]
		set cmd "[lindex $item 2] &"
		set help [lindex $item 3]
		set buttonWidget [APSUniqueName .xMenuFrame]
		APSButton $buttonWidget -packOption "-side top -fill x" \
		  -parent $parent$widget.frame -text $text -command \
		  "APSMenuFrameExec [list $cmd] $parent$widget" \
		  -contextHelp $help
	    }
	    secureExec {
		set text "[lindex $item 1]..."
		set text [APSMenuFramePadLength $text $maxWidth]
		set cmd "[lindex $item 2] &"
		set help [lindex $item 3]
		set users [lindex $item 4]
		set buttonWidget [APSUniqueName .xMenuFrame]
		APSSecureButton $buttonWidget -packOption "-side top -fill x" \
		  -parent $parent$widget.frame -text $text -command \
		  "APSMenuFrameExec [list $cmd] $parent$widget"  \
		  -contextHelp $help -allowedUsers $users
	    }
	    command {
		set text [lindex $item 1]
		set text [APSMenuFramePadLength $text $maxWidth]
		set cmd [lindex $item 2]
		set help [lindex $item 3]
		set buttonWidget [APSUniqueName .xMenuFrame]
		APSButton $buttonWidget -packOption "-side top -fill x" \
		  -parent $parent$widget.frame -text $text -command $cmd \
		  -contextHelp $help
	    }
	    menu {
		set text [lindex $item 1]
		set helpText $text
		set text [APSMenuFramePadLength $text $maxWidth]
		set text "[string range $text 0 \
			    [expr [string length $text] - 2]]->"
		set menuList [lindex $item 2]
		set buttonWidget [APSUniqueName .xMenuFrame]
		set menuWidget [APSUniqueName .xMenuFrame]
		APSButton $buttonWidget -packOption "-side top -fill x" \
		  -parent $parent$widget.frame -text $text -command \
		  "APSMenuFrameMenu $menuWidget [list $menuList] [list $label] $parent$widget" \
		  -contextHelp "Brings up menu of choices for $helpText"
	    }
	}
    }

    frame $parent$widget.frame.buttonRow -relief raised -bd 2
    pack $parent$widget.frame.buttonRow -side top -fill x
    set buttonRow $parent$widget.frame.buttonRow
    if {$closeButton} {
	APSButton .close -parent $buttonRow -packOption "-side left" \
	  -text "Close" -command "APSMenuFrameClose $parent$widget 1" \
	  -highlight 1 -contextHelp "Closes this menu and all submenus."
    }
    APSButton .closeSubmenus -parent $buttonRow -packOption "-side left" \
      -text "Close SubMenus" -command \
      "APSMenuFrameClose $parent$widget 0" -contextHelp "Closes submenus."
    APSButton .kill -parent $buttonRow -packOption "-side left" \
      -text "Kill" -command "APSMenuFrameKill $parent$widget" \
      -contextHelp "Kills all applications started from this menu."
}

proc APSMenuFrameExec {command parentMenuWidget} {
    global apsMenuFrame
    if [catch {eval exec $command} result] {
	puts "error exec'ing $command in APSMenuFrame: $result"
	return
    } else {
	set pid $result
	lappend apsMenuFrame($parentMenuWidget) [list pid $pid]
    }
}

proc APSMenuFrameMenu {menuWidget menuList parentLabel parentMenuWidget} {
    global apsMenuFrame
    if {![info exists apsMenuFrame($parentMenuWidget)]} {
	set apsMenuFrame($parentMenuWidget) {}
    }    
    set apsMenuFrame($parentMenuWidget) \
      [APSMenuFrameListAdd $apsMenuFrame($parentMenuWidget) \
	 [list widget $menuWidget]]
    APSMenuFrame $menuWidget -menuList $menuList -parentLabel $parentLabel \
      -subMenu 1 -geometry [APSGeometryRightRelative $parentMenuWidget]
}

proc APSMenuFrameClose {parentMenuWidget {closeRoot 1}} {
    global apsMenuFrame

    if {[info exists apsMenuFrame($parentMenuWidget)]} {
	set closeList $apsMenuFrame($parentMenuWidget)
	foreach item $closeList {
	    switch [lindex $item 0] {
		widget {
		    set widget [lindex $item 1]
		    APSMenuFrameClose $widget 1
		    set apsMenuFrame($parentMenuWidget) \
		      [APSMenuFrameListDelete \
			 $apsMenuFrame($parentMenuWidget) $item]
		}
		default {
		}
	    }
	}
	if {[llength $apsMenuFrame($parentMenuWidget)] == 0} {
	    unset apsMenuFrame($parentMenuWidget)
	}
    }
    if {$closeRoot} {
	catch {destroy $parentMenuWidget}
    }
    return
}

proc APSMenuFrameKill {parentMenuWidget} {
    global apsMenuFrame

    if {[info exists apsMenuFrame($parentMenuWidget)]} {
	set closeList $apsMenuFrame($parentMenuWidget)
	foreach item $closeList {
	    switch [lindex $item 0] {
		pid {
		    set pid [lindex $item 1]
		    catch {exec kill $pid}
		    set apsMenuFrame($parentMenuWidget) \
		      [APSMenuFrameListDelete \
			 $apsMenuFrame($parentMenuWidget) $item]
		}
		default {
		}
	    }
	}
	if {[llength $apsMenuFrame($parentMenuWidget)] == 0} {
	    unset apsMenuFrame($parentMenuWidget)
	}
    }
    return
}

proc APSMenuFrameListAdd {list value} {
    set ix [lsearch -exact $list $value]
    if {$ix >= 0} {
	return $list
    } else {
	return [linsert $list 0 $value]
    }
}

proc APSMenuFrameListDelete {list value} {
    set ix [lsearch -exact $list $value]
    if {$ix >= 0} {
	return [lreplace $list $ix $ix]
    } else {
	return $list
    }
}

proc APSMenuFramePadLength {text maxWidth} {
    return [format "%-${maxWidth}s" $text]
}

proc APSMenuFrameMakeHelp {} {
    return "Usage: APSMenuFrameMake menu \n\[-type <string>\]\t\n\[-text <string>\]\t\n\[-executable <string>\]\t\n\[-contextHelp <string>\]\t\n\[-allowedUsers <string>\]\t\n\[-menuList <list>\]\t\n\[-command <string>\]\t\n"
}
proc APSMenuFrameMake {menu args} {
    upvar $menu localMenu
    set menuList ""
    set type ""
    set text ""
    set executable ""
    set command ""
    set contextHelp ""
    set allowedUsers ""
    APSStrictParseArguments {menuList type text executable contextHelp \
			       command allowedUsers}

    switch $type {
	label {
	    set item [list label $text]
	}
	exec {
	    set item [list exec $text $executable $contextHelp]
	}
	secureExec {
	    set item [list exec $text $executable $contextHelp $allowedUsers]
	}
	menu {
	    # Label this menu button using label specification of submenu.
	    # If user hasn't supplied one, use text option of procedure.
	    set subMenuLabel ""
	    foreach entry $menuList {
		if {[lindex $entry 0] == "label"} {
		    set subMenuLabel [lindex $entry 1]
		}
	    }
	    if {$subMenuLabel == ""} {
		set item [list menu $text $menuList]
	    } else {
		set item [list menu $subMenuLabel $menuList]
	    }
	}
	command {
	    set item [list command $text $command $contextHelp]
	}
	default {
	    puts stderr "APSMenuFrameMake: unknown -type $type"
	    return $menu
	}
    }
    if {![info exists localMenu]} {
	set localMenu {}
    }
    set localMenu [linsert $localMenu end $item]
}

proc APSMenuFrameFindSubMenu {subMenu menuList} {
    
    set first ""
    set second ""
    set rest ""
    regexp {([^:]*)(.*)} $subMenu match first rest
    regexp {:([^:]*)(.*)} $rest match second rest

    set menuLabel ""
    foreach entry $menuList {
	if {[lindex $entry 0] == "label"} {
	    set menuLabel [lindex $entry 1]
	}
    }
    if {$first != $menuLabel} {
	return {}
    } elseif {$second == ""} {
	return $menuList
    } else {
	set list {}
	foreach item $menuList {
	    if {([lindex $item 0] == "menu") && \
		  ($second == [lindex $item 1])} {
		set list \
		  [APSMenuFrameFindSubMenu $second$rest [lindex $item 2]]
	    }
	}
	return $list
    }
}	

   
	#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.12  1996/03/05 19:41:14  saunders
# Added APSRepack procedure.
#
# Revision 1.11  1996/02/14  16:40:10  saunders
# Removed APSMp dummy routines from APSMisc.tcl. This will fix the problem
# with using APSUniqueName from within a machine procedure.
#
# Revision 1.10  1996/02/14  16:26:23  saunders
# Made small mod to UniqueName procs in effort to fix odd problem with pem.
#
# Revision 1.9  1996/02/09  23:41:58  saunders
# Added new APSUnmap and APSRemap procedures.
#
# Revision 1.8  1996/01/31  19:42:43  saunders
# APSMpReturn dummy routine uses "return -code return" now.
#
# Revision 1.7  1996/01/21  19:51:37  saunders
# Removed ~borland/tcl/rel duplicate procedures. Accidentally left in
# from previous week's work.
#
# Revision 1.6  1996/01/20  22:05:34  borland
# Removed debugging statement in APSDateBreakDown.
#
# Revision 1.5  1996/01/20  21:55:19  borland
# Replaced APSDateBreakDown with new version from mbdateConversion.tcl.
#
# Revision 1.4  1996/01/16  21:20:06  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSUniqueNumberHelp {} {
    return "Usage: APSUniqueNumber\nReturns a unique number.\n"
}

proc APSUniqueNumber {} {
    global apsUniqueNumber
    incr apsUniqueNumber
    return $apsUniqueNumber
}

proc APSUniqueNameHelp {} {
    return "Usage: APSUniqueName prefix\nReturns a unique name which begins with\nthe given prefix.\n"
}

proc APSUniqueName {prefix} {
    global apsUniqueName

    if {[info exists apsUniqueName($prefix)]} {
	incr apsUniqueName($prefix)
    } else {
	set apsUniqueName($prefix) 1
    }
    return $prefix$apsUniqueName($prefix)
}

proc APSGeometryRightRelative {parentWidget} {
    return "+[expr [winfo rootx $parentWidget] + [winfo width $parentWidget]]+[winfo rooty $parentWidget]"
}

proc APSUnmap {widget} {
    global apsMap

    set apsMap($widget) [pack info $widget]
    pack forget $widget
}

proc APSRemap {widget} {
    global apsMap

    if {[info exists apsMap($widget)]} {
	eval pack $widget $apsMap($widget)
    }
}

proc APSRepack {widget args} {
    eval pack $widget [pack info $widget] $args
}



#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.1  1996/02/14 16:40:11  saunders
# Removed APSMp dummy routines from APSMisc.tcl. This will fix the problem
# with using APSUniqueName from within a machine procedure.
#
#

# Dummy routines which allow Mp procedures to execute without
# the Procedure Execution Manager (PEM)

proc APSMpReturn {code {results ""}} {
    return -code return [list $code $results]
}

proc APSMpStep {stepName args} {
    return
}
#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.2  1996/03/22 20:31:57  borland
# Removed debugging statements.
#
# Revision 1.1  1996/03/22  20:15:38  borland
# First version.
#
#

proc APSMultipleChoiceHelp {} {
    return "Usage: APSMultipleChoice widget \n-question <string>\t\n-labelList <list>\t\n-returnList <list>\t\n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string\]\t\n\[-contextHelp <string>\]\n\nCreates\t\$parent\$widget.\n\tmsg\n\tbuttonRow.*.button\nReturns a string value"
}

proc APSMultipleChoice {widget args} {
    global apsContextHelp apsMultipleChoiceValue
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "UntitledChoice"
    set question "Undefined question"
    set contextHelp ""
    set width 20
    set returnList {}
    set labelList {}

    APSParseArguments {parent noPack packOption name contextHelp width \
                     question labelList returnList}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    set windowExists [winfo exists $parent$widget]
    if {$parent == ""} {
	if {$windowExists} {
	    raise $widget
	} else {
	    toplevel $widget
	    wm title $widget $name
	}
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    if {$windowExists} {
        destroy $parent$widget
        } 
    message $parent$widget.msg -relief raised -justify center -bd 2 \
	  -width 10c -text "$question ?\n                            "
    pack $parent$widget.msg -side top -fill both -expand 1
	
    frame $parent$widget.buttonRow -relief raised -bd 2
    pack $parent$widget.buttonRow -side bottom -fill x

    set length [llength $returnList]
    if {[llength $labelList]<$length} {
        set length [llength $labelList]
    }

    if !$length {
        return -code error -errorinfo "No items supplied" ""
    }

    for {set i 0} {$i<$length} {incr i} {
        set labelValue [lindex $labelList $i]
        set returnValue [lindex $returnList $i]
        APSButton .button$i -parent $parent$widget.buttonRow -text "$labelValue" \
	  -command "set apsMultipleChoiceValue $returnValue" 
    }

    focus $parent$widget
    grab $parent$widget
    tkwait variable apsMultipleChoiceValue
    destroy $parent$widget
    return $apsMultipleChoiceValue
}
#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.5  1996/02/06 21:55:14  saunders
# APSStrictParseArguments now returns -1 if unknown option given, 0 otherwise.
#
# Revision 1.4  1996/01/16  21:20:08  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSParseArgumentsHelp {} {
    return "Usage: APSParseArguments <optionKeywordList>\n\
<optionKeywordList> is a list of option keywords that are accepted\n\
by the procedure requesting the parsing.  The parsing scans the args\n\
list of the calling procedure.  Only options from the keyword list\n\
will be processed; others are left in the args list.\n\
The effect of APSParseArguments is to translate a sequence like\n\
\"-keyword value\" into \"set keyword value\"; that is, the keyword\n\
names are variable names in the calling procedure.\n"
}

proc APSParseArguments {optlist} {
    upvar args arguments
    set length [llength $arguments]
    set index 0
    set leftovers {}
    while {$index<$length} {
	set arg [lindex $arguments $index]
	if {[string index $arg 0]=="-"} {
	    set keywordName [string range $arg 1 end]
	    if {[lsearch -exact $optlist $keywordName]!=-1} {
		incr index
		if {$index==$length} {
		    lappend leftovers $arg
		} else {
		    set valueString [lindex $arguments $index]
		    uplevel "set $keywordName {$valueString}"
		    incr index
		}
	    } else {
		incr index
		lappend leftovers $arg
	    }
	} else {
	    lappend leftovers $arg
	    incr index
	}
    }
    set arguments [concat $leftovers]
}

proc APSStrictParseArgumentsHelp {} {
    return "Usage: APSStrictParseArguments <optionKeywordList>\n\
Same as APSParseArguments, only it prints an error to stderr if user\n\
passes extra or unknown options not given in optlist. Also, returns\n\
-1 if unknown option given, 0 otherwise.\n\
<optionKeywordList> is a list of option keywords that are accepted\n\
by the procedure requesting the parsing.  The parsing scans the args\n\
list of the calling procedure.  Only options from the keyword list\n\
will be processed; others are left in the args list.\n\
The effect of APSParseArguments is to translate a sequence like\n\
\"-keyword value\" into \"set keyword value\"; that is, the keyword\n\
names are variable names in the calling procedure.\n"
}

proc APSStrictParseArguments {optlist} {
    upvar args arguments
    set length [llength $arguments]
    set index 0
    set leftovers {}
    while {$index<$length} {
	set arg [lindex $arguments $index]
	if {[string index $arg 0]=="-"} {
	    set keywordName [string range $arg 1 end]
	    if {[lsearch -exact $optlist $keywordName]!=-1} {
		incr index
		if {$index==$length} {
		    lappend leftovers $arg
		} else {
		    set valueString [lindex $arguments $index]
		    uplevel "set $keywordName {$valueString}"
		    incr index
		}
	    } else {
		incr index
		lappend leftovers $arg
	    }
	} else {
	    lappend leftovers $arg
	    incr index
	}
    }
    set arguments [concat $leftovers]
    if {$arguments != ""} {
	set procName [lindex [info level [expr [info level] - 1]] 0]
	puts stderr "Unknown option(s) given to $procName \"$arguments\""
	return -1
    } else {
	return 0
    }
}


proc APSClearOptions {} {
    upvar args arguments
    set length [llength $arguments]
    set index 0
    set leftovers {}
    while {$index<$length} {
	set arg [lindex $arguments $index]
	if {[string index $arg 0]=="-"} {
	    incr index 2
	} else {
	    lappend leftovers $arg
	    incr index
	}
    }
    set arguments [concat $leftovers]
}



#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.13  1996/02/13 15:55:00  saunders
# Modified APSScrolledText to do word wrap, not char wrap.
#
# Revision 1.12  1996/01/19  20:27:30  saunders
# Fixed APSScrolledList so that multiple listboxes can have independent
# selections.
#
# Revision 1.11  1996/01/18 00:42:54  saunders
# Added "-fill x" as default to APSScrolled procedure.
#
# Revision 1.10  1996/01/16  21:20:16  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#


proc APSScrolledTextHelp {} {
    return "Usage: APSScrolledText widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-width <string>\]\t\n\[-height <string>\]\t\n\[-name <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\ttext\n\tscroll\n"
}

proc APSScrolledText {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill both -expand 1"
    set name "Scroll Text"
    set width 40
    set height 15
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption width height name contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$parent == ""} {
	toplevel $widget -bd 1
	wm title $widget $name
    } else {
	frame $parent$widget -bd 2 -relief ridge
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    text $parent$widget.text -width $width -height $height \
      -bd 1 -relief raised -setgrid 1 -wrap word \
      -yscrollcommand "$parent$widget.scroll set"
    scrollbar $parent$widget.scroll \
      -command "$parent$widget.text yview"
    pack $parent$widget.scroll -side right -fill y
    pack $parent$widget.text -side left -fill both -expand 1

}

proc APSScrolledListHelp {} {
    return "Usage: APSScrolledList widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-height <string>\]\t\n\[-name <string>\]\t\n\[-itemList <list>\]\t\n\[-callback <proc>] note: procedure must be <proc> listboxItem doubleClick\n\[-itemList <list>\]\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tlistbox\n\tscroll\n"
}

proc APSScrolledList {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill both -expand 1"
    set name "Scroll Text"
    set height 15
    set callback ""
    set itemList ""
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption height name \
			 callback itemList contextHelp }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }
    if {$parent == ""} {
	toplevel $widget -bd 2
	wm title $widget $name
    } else {
	frame $parent$widget -bd 2 -relief ridge
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    listbox $parent$widget.listbox  -height $height -width 20 \
      -bd 1 -relief raised -setgrid 1 -exportselection 0 \
      -yscrollcommand "$parent$widget.scroll set"
    scrollbar $parent$widget.scroll \
      -command "$parent$widget.listbox yview"
    pack $parent$widget.scroll -side right -fill y
    pack $parent$widget.listbox -side left -fill both -expand 1

    # Set listbox width so that longest item fits
    set maxWidth 20 
    foreach item $itemList {
	set itemWidth [string length $item]
	if {$itemWidth > $maxWidth} {
	    set maxWidth $itemWidth
	}
    }
    $parent$widget.listbox configure -width $maxWidth
    eval {$parent$widget.listbox insert end} $itemList

    if {$callback != ""} {
	bind $parent$widget.listbox <ButtonRelease-1> \
	  [subst -nocommands {
	      $callback [$parent$widget.listbox get \
			   [$parent$widget.listbox nearest %y]] 0
	  }]
	bind $parent$widget.listbox <Double-1> \
	  [subst -nocommands {
	      $callback [$parent$widget.listbox get \
			   [$parent$widget.listbox nearest %y]] 1
	  }]
    }
}

proc APSScrolledListWindowHelp {} {
    return "Usage: APSScrolledListWindow widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-height <string>\]\t\n\[-name <string>\]\t\n\[-label <string>\]\n\[-itemList <list>\]\n\[-selectionVar <string>\]\n\[-callback <script>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tuserFrame\n\tuserFrame.sl.listbox\n\tbuttonRow.close.button\n\tbuttonRow.accept.button"
}
proc APSScrolledListWindowCb {listbox script} {
    set list ""
    foreach item [$listbox curselection] {
	lappend list [$listbox get $item]
    }
    eval $script {$list}
}
proc APSScrolledListWindow {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "NoName"
    set label ""
    set itemList ""
    set selectionVar ""
    set callback ""
    set height 15
    set contextHelp ""
    APSStrictParseArguments {parent name label noPack packOption itemList \
                         selectionVar height callback contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    APSWindow $widget -parent $parent -noPack $noPack -packOption $packOption \
      -name $name

    if {$label != ""} {
	label $parent$widget.userFrame.label -text $label
	pack $parent$widget.userFrame.label -side top -fill x -padx 10 -pady 1
    }

    APSScrolledList .sl -parent $parent$widget.userFrame -itemList $itemList \
      -height $height -contextHelp "Click on selection, and drag to select a block of items. Use Ctrl-Click to pick out disjoint items from list. Press Accept to accept your selection, but keep the window open. Press Close to accept your selection and close the window."

    set list $parent$widget.userFrame.sl.listbox
    $list configure -selectmode extended

    set acceptCmd ""
    if {$selectionVar != ""} {
	set foreachList "\[$list curselection\]"
	set foreachBody "lappend $selectionVar \[$list get \$item\]"
	set acceptCmd [subst {
	    set $selectionVar ""
	    foreach item $foreachList {$foreachBody}
	}]
    }
    if {$callback != ""} {
	set acceptCmd [subst {
	    $acceptCmd
	    APSScrolledListWindowCb $list $callback
	}]
    }
    set closeCmd [subst {
	$acceptCmd
	destroy $parent$widget
    }]
    $parent$widget.buttonRow.close.button configure -command $closeCmd
    APSDialogBoxAddButton .accept -parent $parent$widget -text Accept \
      -command $acceptCmd -contextHelp "Accept selections, but keep this window open."
}

proc APSScrollHelp {} {
    return "Usage: APSScroll widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tframe.canvas\n\tframe.yscroll\n\tframe.canvas.frame\n\nReturns \$parent\$widget.frame.canvas.frame into which you pack widgets\nto be scrolled. Then call \"APSScrollAdjust \$parent\$widget\" \nto configure scrolling.\n\n"
}

proc APSScroll {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "NoName"
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption name contextHelp}

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }

    APSFrame $widget -parent $parent -noPack $noPack -packOption $packOption \
      -name $name

    # Create a scrollable canvas
    canvas $parent$widget.frame.canvas -width 10 -height 10 \
      -yscrollcommand [list $parent$widget.frame.yscroll set]
    scrollbar $parent$widget.frame.yscroll -orient vertical \
      -command [list $parent$widget.frame.canvas yview]
    pack $parent$widget.frame.yscroll -side right -fill y
    pack $parent$widget.frame.canvas -side left -fill both -expand 1
    pack $parent$widget.frame -side top -fill both -expand 1
   
    set canvas $parent$widget.frame.canvas

    # Create one frame to hold everything
    # and position it on the canvas
    set f [frame $canvas.frame -bd 0]
    $canvas create window 0 0 -anchor nw -window $f
    return $f
}

proc APSScrollAdjustHelp {} {
    return "Usage: APSScrollAdjust widget \n\[-numVisible <string>\]\t\n\[-scrollIncrement <string>\]\t\n\nWidget given must be the base widget from APSScroll (\$parent\$widget).\nAdjusts the canvas and scrolling parameters to do something reasonable with\nthe widgets you have packed into the scrolled frame. In general, it is\nassumed you have packed homogenous widgets, in which case -numVisible\ndetermines how many you will see at a time. Scrolling is set up to jump in\nincrements of one widget.\n\n"
}

proc APSScrollAdjust {widget args} {
    global apsScrollResource
    set numVisible 4
    set scrollIncrement 0
    APSStrictParseArguments {numVisible scrollIncrement}

    set children [pack slaves $widget.frame.canvas.frame]
    set canvas $widget.frame.canvas

    # Determine width of widest widget and height of all widgets
    set maxWidth 0
    set maxHeight 0
    foreach child $children {
	set maxHeight [expr $maxHeight + [winfo height $child]]
	set width [winfo width $child]
	if {$width > $maxWidth} {
	    set maxWidth $width
	}
    }
    if {$scrollIncrement != 0} {
	set incr $scrollIncrement
    } else {
	set incr [winfo height [lindex $children 0]]
    }
    set height [expr $incr * $numVisible]
    $canvas config -scrollregion "0 0 $maxWidth $maxHeight"
    $canvas config -yscrollincrement $incr
    $canvas config -width $maxWidth -height $height
}

proc APSScrolledStatusHelp {} {
    return "Usage: APSScrolledStatus widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-textVariable <string>\]\t\n\[-width <string>\]\t\n\[-height <string>\]\t\n\[-name <string>\]\t\n\[-contextHelp <string>\]\t\n\nCreates\t\$parent\$widget.\n\tframe.text\n\tframe.scroll\n"
}
proc APSScrolledStatus {widget args} {
    global apsContextHelp apsEntryBackground
    global apsScrolledStatusVar
    set parent ""
    set noPack 0
    set packOption "-side top"
    set name "NoName"
    set textVariable ""
    set width 40
    set height 5
    set contextHelp ""
    APSStrictParseArguments {parent noPack packOption name textVariable \
			       width height contextHelp}
    
    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp    
    }
    APSFrame $widget -parent $parent -noPack $noPack -packOption $packOption \
      -name $name -label Status: \
      -contextHelp "Changes to application status are recorded here."

    text $parent$widget.frame.text -width $width -height $height \
      -bd 1 -relief raised -setgrid 1 -bg $apsEntryBackground \
      -yscrollcommand "$parent$widget.frame.scroll set"
    scrollbar $parent$widget.frame.scroll \
      -command "$parent$widget.frame.text yview"
    pack $parent$widget.frame.scroll -side right -fill y
    pack $parent$widget.frame.text -side left -fill both -expand 1

    global $textVariable
    $parent$widget.frame.text insert end [subst $$textVariable]\n

    set apsScrolledStatusVar($textVariable) $parent$widget
    trace variable $textVariable w APSScrolledStatusUpdate
}

# Variable trace callback procedure
proc APSScrolledStatusUpdate {varName index op} {
    global $varName
    global apsScrolledStatusVar

    set window $apsScrolledStatusVar($varName)

    switch $op {
	w {
	    if {[winfo exists $window]} {
		$window.frame.text insert end [subst $$varName]\n
		$window.frame.text yview moveto 1.0
	    } else {
		trace vdelete $varName w APSScrolledStatusUpdate
		unset apsScrolledStatusVar($varName)
	    }
	}
	u {
	    trace vdelete $varName w APSScrolledStatusUpdate
	    unset apsScrolledStatusVar($varName)
	}
    }
}


#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.7  1996/02/22 19:59:35  borland
# Removed debugging statement from APSCheckSDDSFile.
#
# Revision 1.6  1996/02/12  21:43:23  borland
# Change APSCheckSDDSFile to use sddscheck when file extension is gz, F, or
# Z.
#
# Revision 1.5  1996/02/12  21:36:47  borland
# Added close statement to APSCheckSDDSFile.
#
# Revision 1.4  1996/01/16  21:20:12  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSGetSDDSColumnHelp {} {
    return "Usage: APSGetSDDSColumn\n\[-fileName <string>\]\t\n\[-column <string>\]\t\n\[-page <string>\]\t\n\nExtracts column from sdds file and returns it as a tcl list."
}
proc APSGetSDDSColumn {args} {
    set fileName "data.sdds"
    set column ""
    set page 0
    APSStrictParseArguments {fileName column page} 

    set extraArgs -noquotes
    if {$page > 0} {
	set extraArgs "$extraArgs -page=$page"
    }
    set cmd "exec sdds2stream -column=$column $fileName $extraArgs"
    if {[catch $cmd dataString]} {
       puts stderr APSGetSDDSColumn:$dataString
       return ""
    } else {
       set dataList [split $dataString "\n"]
       return $dataList
    }
}

proc APSGetSDDSParameterHelp {} {
    return "Usage: APSGetSDDSParameter\n\[-fileName <string>\]\t\n\[-parameter <string>\]\t\n\[-page <string>\]\t\n\nExtracts parameter from sdds file and returns it."
}
proc APSGetSDDSParameter {args} {
    set fileName "data.sdds"
    set parameter ""
    set page 0
    APSStrictParseArguments {fileName parameter page} 

    set extraArgs -noquotes
    if {$page > 0} {
	set extraArgs "$extraArgs -page=$page"
    }
    set cmd "exec sdds2stream -parameter=$parameter $fileName $extraArgs"
    if {[catch $cmd dataString]} {
       puts stderr APSGetSDDSParameter:$dataString
       return ""
    } else {
       return $dataString
    }
}

proc APSGetSDDSNamesHelp {} {
    return "Usage: APSGetSDDSNames\n\[-fileName <string>\]\t\n\[-class <string>\] where <string> is column (default), parameter, or array.\t\n\nExtracts data names by class from an sdds file\nand returns it as a tcl list."
}
proc APSGetSDDSNames {args} {
    set fileName "data.sdds"
    set class column
    APSStrictParseArguments {fileName class}

    if {$class!="column" && $class!="parameter" &&
	$class!="array"} {
	return {}
    }
    set cmd "exec sddsquery -$class $fileName"
    if {[catch $cmd dataString]} {
	puts stderr APSGetSDDSNames:$dataString
	return ""
    } else {
	set dataList [split $dataString "\n"]
	return $dataList
    }
}

proc APSCheckSDDSFileHelp {} {
    return "Usage: APSCheckSDDSFile\n\[-fileName <string>\]\t\n\nVerify that given fileName is an SDDS1 file. \nReturns 1 if true, 0 otherwise."
}
proc APSCheckSDDSFile {args} {
    set fileName "data.sdds"
    APSStrictParseArguments {fileName}

    if {![file exists $fileName] || [file isdirectory $fileName]} {
        return 0
    }

    if {[string compare [file extension $fileName] ".gz"]==0 || \
          [string compare [file extension $fileName] ".F"]==0 || \
          [string compare [file extension $fileName] ".Z"]==0} {
        catch {sddscheck $fileName} result
        return [string compare $result ok]
    }

    if {[catch {open $fileName r} fd1]} {
	puts stderr APSCheckSDDSFile:$fd1
	return 0
    } else {
	gets $fd1 line
        close $fd1
	if {[string compare $line "SDDS1"]==0} { return 1 }
	return 0
    }
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.4  1996/01/16 21:20:18  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSSoundHelp {} {
    return "Usage: APSSound\n\[-type <string>\] where <string> is working, alert, or emergency\t\n\[-volume <string>\] from 1 to 100\t\n\[-iterations <string>\]\t\n\[-period <string>\] where <string> is in milliseconds or \"continuous\".\nPlays given sound type in background."
}

proc APSSound {args} {
    set type "working"
    set volume 50
    set iterations 1
    set period 2000
    
    APSStrictParseArguments {type volume iterations period}
    catch "after 0 APSPlaySound $type $volume $iterations $period"
}

proc APSPlaySound {type volume iteration period} {
    global apsSoundPath
    if {$period == "continuous"} {
	set bgFlag ""
    } else {
	set bgFlag "&"
    }
    switch $type {
	working {
	    set cmd \
	      "exec play -i -v $volume $apsSoundPath/working.au $bgFlag"
	    catch $cmd
	}
	alert  {
	    set cmd \
	      "exec play -i -v $volume $apsSoundPath/alert.au $bgFlag"
	    catch $cmd
	}
	emergency  {
	    set cmd \
	      "exec play -i -v $volume $apsSoundPath/emergency.au $bgFlag"
	    catch $cmd
	}
    }
    if {$iteration == 1} {
	return
    } elseif {$period == "continuous"} {
	APSPlaySound $type $volume [incr iteration -1] $period
    } else {
	catch "after $period APSPlaySound $type $volume [incr iteration -1] $period"
    }
    
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.12  1996/02/09 23:42:23  saunders
# Added apsMap global.
#
# Revision 1.11  1996/02/09  17:12:14  saunders
# Additions to support APSMenuFrame procedure.
#
# Revision 1.10  1996/01/16  21:20:21  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSStandardSetupHelp {} {
    return "Usage: APSStandardSetup \nInitializes a number of global variables and tcl/tk options.\n"
}

proc APSStandardSetup {} {
    global apsSetupInitialized
    global apsBackground apsForeground apsEntryBackground
    global apsMenuForeground apsMenuBackground
    global tcl_precision tk_strictMotif
    global apsContextHelp apsContextHelpEnable apsContextHelpExcludeList
    global apsWidgetCursorType
    global apsSoundPath
    global apsUniqueNumber
    global apsUniqueName
    # NOTE: remove this ASAP (from ~borland/tcl/rel for APSTmpString)
    global apsTmpStringCount
    global apsScrolledStatusVar
    global apsMenuFrame
    global apsMap

    if {[info exists apsSetupInitialized]} {
	return
    } else {
	set apsSetupInitialized 1
	set apsContextHelp(init) "Test Context Help Message"
	set apsContextHelpEnable 0
	set apsContextHelpExcludeList {}
	set apsWidgetCursorType(init) "Test Cursor Type"

	set apsSoundPath /usr/local/oag/apps/lib/sun4
	set apsScrolledStatusVar(init) "init"
	set apsMenuFrame(init) "init"
	set apsMap(init) "init"
	set apsUniqueNumber 0
	# NOTE: remove this ASAP (from ~borland/tcl/rel for APSTmpString)
	set apsTmpStringCount 0
	
	set apsBackground Gray85
	set apsForeground Black
	set apsEntryBackground White
	set apsMenuBackground \#597EE1
	set apsMenuForeground White

	# Do this only if started by interpreter with Tk
	if {[info command frame] != ""} {
#	    option add *font -adobe-courier-medium-r-normal-*-14-*-*-*-*-*-*-*
	    #  Nice looking, but not fixed width
	    #	option add *font -Adobe-Helvetica-Medium-R-Normal--*-120-*
	    option add *background $apsBackground
	    option add *foreground $apsForeground
	    option add *Entry.background $apsEntryBackground
	    set tk_strictMotif 1
	}
	    
	set tcl_precision 17
    }
}


#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.1  1996/01/16 21:38:29  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

proc APSInfoDialogHelp {} {
    return "Usage: APSInfoDialog widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-name <string\]\t\n\[-infoMessage <string>\]\t\n\[-contextHelp <string>\]\n\nCreates\t\$parent\$widget.\n\tmsg\n\tbuttonRow.ok.button\n\tbuttonRow.cancel.button"
}

proc APSInfoDialog {widget args} {
    global apsContextHelp apsInfoDialogChoice apsInfoDialogInput
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "UntitledInfo"
    set infoMessage "No Info"
    set contextHelp ""
    set width 20

    APSParseArguments {parent noPack packOption name infoMessage contextHelp width }

    if {$contextHelp != ""} {
	set apsContextHelp($parent$widget) $contextHelp
    }

    set windowExists [winfo exists $parent$widget]
    if {$parent == ""} {
	if {$windowExists} {
	    raise $widget
	} else {
	    toplevel $widget
	    wm title $widget $name
	}
    } else {
	frame $parent$widget
	if {!$noPack} {
	    eval pack $parent$widget $packOption
	}
    }

    if {$windowExists} {
        destroy $parent$widget
        } 
    message $parent$widget.msg -relief raised -justify center -bd 2 \
	  -width 10c -text "$infoMessage\n                            "
    pack $parent$widget.msg -side top -fill both -expand 1
	
    APSLabeledEntry .lentry -parent $parent$widget -label "Input: " \
        -textVariable apsInfoDialogInput -width $width \
        -contextHelp $contextHelp

    frame $parent$widget.buttonRow -relief raised -bd 2
    pack $parent$widget.buttonRow -side bottom -fill x
	
    APSButton .ok -parent $parent$widget.buttonRow -text OK \
	  -command "set apsInfoDialogChoice OK" -highlight 1
    APSButton .cancel -parent $parent$widget.buttonRow -text CANCEL \
	  -command "set apsInfoDialogChoice CANCEL" -highlight 0


    focus $parent$widget
    grab $parent$widget
    tkwait variable apsInfoDialogChoice
    destroy $parent$widget
    if [string compare $apsInfoDialogChoice "OK"]==0 { 
        return $apsInfoDialogInput
    } 
    return ""
}
#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.11  1996/02/26 16:29:26  borland
# Added routine to define apsDaysInMonths, with check of whether it exists
# already.
#
# Revision 1.10  1996/02/19  16:53:42  borland
# Added Y-M-D date format option to APSFormatDate.
#
# Revision 1.9  1996/02/10  00:59:10  borland
# Added -leadingZeros option to several routines.
#
# Revision 1.8  1996/02/05  17:32:50  borland
# Added APSOffsetDateInfo procedure.  Should be used in place of APSPreviousDaysDate
# and APSTodaysDateInfo.
#
# Revision 1.7  1996/02/04  22:13:20  borland
# Added APSFormatDate procedure.  Changed other routines to call it.
#
# Revision 1.6  1996/02/04  21:21:00  borland
# Added APSTodaysDateInfo procedure.  Like APSPreviousDaysDate, but gives
# information about today.
#
# Revision 1.5  1996/02/04  20:53:06  borland
# Added APSMakeFourDigitYear procedure; used by other routines to help deal
# with two-digit years and millenial boundary.
#
# Revision 1.4  1996/02/04  20:30:18  borland
# APSMonthDay now scans julianDay and year arguments to properly interpret
# numbers with leading zeros.
#
# Revision 1.3  1996/01/20  22:05:35  borland
# Removed debugging statement in APSDateBreakDown.
#
# Revision 1.2  1996/01/20  21:53:34  borland
# Modified APSDateConversion to support two-digit and four-digit year conventions
# plus Julian Day; use uplevel instead of upvar as the latter doesn't work if
# the variables don't exist ahead of time.
#
# Revision 1.1  1996/01/16  21:38:36  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

#set auto_path [linsert $auto_path 0 /home/helios/BORLAND/tcl/rel]
#set auto_path [linsert $auto_path 0  /usr/local/oag/apps/lib/sun4]

# APSDate:
# Passes a string to the UNIX date utility
#
proc APSDate {string} {
    catch {exec date $string} result
    return $result
}

# APSMakeFourDigitYear 
# Convert two-digit year to four-digit year, assuming year is
# between 1990 and 2089
#
proc APSMakeFourDigitYear {year} {
    scan $year "%ld" year
    if $year>100 {
        return $year
    }
    if $year<90 {
        incr year 2000
    } else {
        incr year 1900
    }
    return $year
}

# APSFormatDate 
# Put date into one of several formats 
#
proc APSFormatDate {args} {
    set year ""
    set month ""
    set day ""
    set julianDay ""
    set twoDigitYear 0
    set dateFormat ""
    set dateList {}
    set leadingZeros 1
    APSStrictParseArguments {year month day julianDay dateList twoDigitYear dateFormat leadingZeros}
    foreach var {year month day julianDay} {
        set value [subst \$$var]
        if [string length $value] {
            scan $value %ld $var
        }
    }
    if [llength $dateList] {
        # list: year julian-day month day 
        if [llength $dateList]!=4 {
            return 0
        }
        scan [lindex $dateList 0] %ld year
        scan [lindex $dateList 1] %ld julianDay
        scan [lindex $dateList 2] %ld month
        scan [lindex $dateList 3] %ld day
    } else {
        if ![string length $year] {
            return 0
        }
        if [string length $julianDay] {
            set mmdd [APSMonthDay -julianDay $julianDay -year $year]
            set month [lindex $mmdd 0]
            set day [lindex $mmdd 1]
        } else {
            set julianDay [APSJulianDay -day $day -month $month \
                             -year $year]
        }
    }

    set month [format %02ld $month]
    set day   [format %02ld $day]
    set julianDay [format %03ld $julianDay]

    if !$twoDigitYear {
        set year [APSMakeFourDigitYear $year]
    } else {
        scan $year %ld year
        set year [format %02ld [expr $year - int($year/100)*100]]
    }
    
    switch $dateFormat {
        "Y-J-MD" { 
            return ${year}-${julianDay}-${month}${day}
        }
        "YMD" { 
            return ${year}${month}${day}
        }
        "Y-M-D" {
            return ${year}-${month}-${day}
        }
        "MDY" {
            return ${month}${day}${year}
        }
        "list" {
            if !$leadingZeros {
                scan $month "%ld" month
                scan $day "%ld" day
                scan $julianDay "%ld" julianDay
            }
            return [concat $year $julianDay $month $day]
        }
        default {
            return ${year}${month}${day}
        }
    }
}


        
# APSDateBreakDown
# Puts the day, month, and year into variables
#
proc APSDateBreakDown {args} {
    set dayVariable ""
    set monthVariable ""
    set yearVariable ""
    set julianDayVariable ""
    set twoDigitYear 1
    set leadingZeros 1
    APSStrictParseArguments {julianDayVariable dayVariable monthVariable yearVariable twoDigitYear leadingZeros}
    catch {exec date +%d:%m:%y:%j} result
    set dateList [split $result :]
    if !$leadingZeros {
        scan [lindex $dateList 0] "%ld" day  
        scan [lindex $dateList 1] "%ld" month
        scan [lindex $dateList 2] "%ld" year 
        scan [lindex $dateList 3] "%ld" jday 
    } else {
        set day   [lindex $dateList 0]
        set month [lindex $dateList 1]
        set year  [lindex $dateList 2]
        set jday  [lindex $dateList 3]
    }

    if [string length $dayVariable] {
        uplevel "set $dayVariable $day"
    }
    if [string length $monthVariable] {
        uplevel "set $monthVariable $month"
    }
    if [string length $julianDayVariable] {
        uplevel "set $julianDayVariable $jday"
    }
    if [string length $yearVariable] {
        if !$twoDigitYear {
            set year [APSMakeFourDigitYear $year]
        }
        uplevel "set $yearVariable $year"
    }
}


# APSLeapYear:
# Returns 0/1 for nonleap/leap year
# Returns -1 on error
#
proc APSLeapYear {year} {
    if {$year<0} {
        return -1
    }
    set year [APSMakeFourDigitYear $year]
    return [expr $year%4==0 && $year%100!=0 || $year%400==0]
}

proc APSDefineDaysInMonths {} {
    # days in each month for leap years, nonleap years
    global apsDaysInMonths
    if {![info exists apsDaysInMonths]} {
        set apsDaysInMonths(0,1) 31 
        set apsDaysInMonths(0,2) 28 
        set apsDaysInMonths(0,3) 31 
        set apsDaysInMonths(0,4) 30 
        set apsDaysInMonths(0,5) 31 
        set apsDaysInMonths(0,6) 30 
        set apsDaysInMonths(0,7) 31 
        set apsDaysInMonths(0,8) 31 
        set apsDaysInMonths(0,9) 30 
        set apsDaysInMonths(0,10) 31 
        set apsDaysInMonths(0,11) 30 
        set apsDaysInMonths(0,12) 31
        set apsDaysInMonths(1,1) 31 
        set apsDaysInMonths(1,2) 29
        set apsDaysInMonths(1,3) 31 
        set apsDaysInMonths(1,4) 30 
        set apsDaysInMonths(1,5) 31 
        set apsDaysInMonths(1,6) 30 
        set apsDaysInMonths(1,7) 31 
        set apsDaysInMonths(1,8) 31 
        set apsDaysInMonths(1,9) 30 
        set apsDaysInMonths(1,10) 31 
        set apsDaysInMonths(1,11) 30 
        set apsDaysInMonths(1,12) 31
    }
}

# APSJulianDay:
# Computes the Julian day from the year,month, and day
#
proc APSJulianDay {args} {
    set day 0
    set month 0
    set year 0
    APSStrictParseArguments {day month year}
    if [llength $args] { return 0 }
    if {[string length $day]==0 || [string length $month]==0 || [string length $year]==0} {
        return 0
    }
    scan $day "%ld" day
    scan $month "%ld" month
    scan $year "%ld" year
    if {$day<1 || $day>31 || $month<1 || $month>12 || $year<0} {
        return 0
    }
    set leapYear [APSLeapYear $year]

    APSDefineDaysInMonths
    global apsDaysInMonths 
    set daysInMonth [subst \$apsDaysInMonths($leapYear,$month)]
    if {$day>$daysInMonth} {
        set day $daysInMonth
    }

    set jday 0
    incr jday $day
    for {set i 1} {$i<$month} {incr i} {
        incr jday [subst \$apsDaysInMonths($leapYear,$i)]
    }
    return $jday
}

# APSMonthDay
# Computes day of month and month from julian day and year 
#

proc APSMonthDay {args} {
    set julianDay 0
    set year 0
    APSStrictParseArguments {julianDay year}
    if {[llength $julianDay]==0 || [llength $year]==0} {
        return 0
    }
    scan $julianDay "%ld" julianDay
    scan $year "%ld" year
    if {$julianDay<0 || $julianDay>366 || $year<0} {
        return 0
    }
    set leapYear [APSLeapYear  $year]
    if {$leapYear==0 && $julianDay>=365} {
        return {12 31}
    }
    set sum 0
    APSDefineDaysInMonths
    global apsDaysInMonths 
    for {set i 1} {$i<=12} {incr i} {
        set days [subst \$apsDaysInMonths($leapYear,$i)]
        if {[expr $sum+$days]<$julianDay} {
            incr sum $days
        } else {
            break
        }
    }
    set dayOfMonth [expr $julianDay-$sum]
    return [list $i $dayOfMonth]
}


proc testDates {} {
    set errors 0
    for {set year 1990} {$year<2010} {incr year} {
        if {[APSLeapYear $year]==1} {
            set dlimit 367
        } else {
            set dlimit 366
        }
        for {set jd 1} {$jd<$dlimit} {incr jd} {
            set md [APSMonthDay -julianDay $jd -year $year]
            set jd1 [APSJulianDay -day [lindex $md 1] -month [lindex $md 0] -year $year]
            if {$jd!=$jd1} {
                puts "$jd $jd1 $year [lindex $md 0] [lindex $md 1]"
                incr errors
            }
        }
        puts "Done with $year"
    }
    puts "$errors errors"
}

# APSPreviousDaysDate
# Returns the date of the day before a given date
#
proc APSPreviousDaysDate {args} {
    set julianDay ""
    set year ""
    set month ""
    set day ""
    set twoDigitYear 0
    set dateFormat list
    APSStrictParseArguments {julianDay year month day dateFormat twoDigitYear}
    if ![string length $year] {
        puts stderr "Empty year string (APSPreviousDaysDate)"
        if [string compare $dateFormat list]==0 {
            return {}
        }
        return 0
    }
    set year [APSMakeFourDigitYear $year]
    if [string length $julianDay]==0 {
        set julianDay [APSJulianDay -day $day -month $month -year $year]
        if !$julianDay {
            puts stderr "Bad Julian day (APSPreviousDaysDate)"
            if [string compare $dateFormat list]==0 {
                return {}
            }
            return 0
        }
    }

    scan $julianDay %ld julianDay
    if {[set julianDay [expr $julianDay-1]]<1} {
        incr year -1
        if [APSLeapYear $year] {
            set julianDay 366
        } else {
            set julianDay 365
        }
    }
    set julianDay [format "%03ld" $julianDay]
    return [APSFormatDate -year $year -julianDay $julianDay \
              -dateFormat $dateFormat]
}

# APSTodaysDateInfo
# Returns date information for today
#
proc APSTodaysDateInfo {args} {
    set twoDigitYear 0
    set dateFormat list
    APSStrictParseArguments {dateFormat twoDigitYear}
    set year 0
    set julianDay 0
    APSDateBreakDown \
      -yearVariable year -julianDayVariable julianDay
    return [APSFormatDate -year $year -julianDay $julianDay \
              -dateFormat $dateFormat]
}

#
# APSOffsetDateInfo
# Returns date info for a day offset by N days from given day
#
proc APSOffsetDateInfo {args} {
    set julianDay ""
    set year ""
    set month ""
    set day ""
    set dateList {}
    set twoDigitYear 0
    set dateFormat list
    set offset 0
    set today 0
    set leadingZeros 1
    APSStrictParseArguments {julianDay year month day dateFormat twoDigitYear dateList offset today leadingZeros}
    if $today {
        APSDateBreakDown -yearVariable year -julianDayVariable julianDay -twoDigitYear 1
    } else {
        set dateListLen [llength $dateList]
        if $dateListLen {
            if {$dateListLen!=2 && $dateListLen!=4} {
                if [string compare $dateFormat list]==0 {
                    return {}
                }
                return 0
            }
            set year [lindex $dateList 0]
            set julianDay [lindex $dateList 1]
            if $dateListLen==4 {
                set month [lindex $dateList 2]
                set day [lindex $dateList 3]
            }
        }
    }

    if ![string length $year] {
        puts stderr "Empty year string (APSOffsetDateInfo)"
        if [string compare $dateFormat list]==0 {
            return {}
        }
        return 0
    }
    set year [APSMakeFourDigitYear $year]
    if [string length $julianDay]==0 {
        set julianDay [APSJulianDay -day $day -month $month -year $year]
        if !$julianDay {
            puts stderr "Bad month/day (APSOffsetDateInfo)"
            if [string compare $dateFormat list]==0 {
                return {}
            }
            return 0
        }
    }

    scan $julianDay %ld julianDay
    if $offset<0 {
        while {$offset} {
            set newJDay [expr $offset+$julianDay]
            if $newJDay<=0 {
                incr year -1
                set daysInYear 365
                if [APSLeapYear $year] {
                    set daysInYear 366
                }
                incr offset $julianDay
                set julianDay $daysInYear
            } else {
                set julianDay $newJDay
                set offset 0
            }
        }
    } else {
        set daysInYear 365
        if [APSLeapYear $year] {
            set daysInYear 366
        }
        while {$offset} {
            set newJDay [expr $offset+$julianDay]
            if $newJDay>$daysInYear {
                incr year 1
                incr offset [expr -($daysInYear-$julianDay)-1]
                set julianDay 1
                set daysInYear 365
                if [APSLeapYear $year] {
                    set daysInYear 366
                }
            } else {
                set julianDay $newJDay
                set offset 0
            }
        }
    }
        
    return [APSFormatDate -year $year -julianDay $julianDay \
              -dateFormat $dateFormat -leadingZeros $leadingZeros ]
}

proc testDateOffset {} {
    for {set year 1990} {$year<=2010} {incr year} {
        set daysInYear 365
        if [APSLeapYear $year] {
            set daysInYear 366 
        }
        for {set jday 1} {$jday<=$daysInYear} {incr jday} {
            puts "Working on $jday of $year"
            set refDate [APSFormatDate -dateFormat Y-J-MD -year $year -julianDay $jday]
            for {set offset -1000} {$offset<=1000} {incr offset 500} {
                set offDateList [APSOffsetDateInfo -dateFormat list -year $year -julianDay $jday -offset $offset]
                set compDate [APSOffsetDateInfo -dateFormat Y-J-MD -dateList $offDateList -offset [expr -$offset]]
                if [string compare $refDate $compDate] {
                    puts "Error: $compDate (offset $offset) does not match $refDate"
                }
            }
        }
    }
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.2  1996/02/12 21:33:31  borland
# Added first version of APSMakeTimeDateEntryWidget.
#
# Revision 1.1  1996/01/16  21:38:38  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

proc APSMakeDateEntryWidget {widget args} {
    set parent ""
    set dayVariable ""
    set yearVariable ""
    set monthVariable ""
    set label "Date (year, month, day): "
    APSParseArguments {parent dayVariable yearVariable monthVariable label}
    global $monthVariable $yearVariable $dayVariable
    set dateList [split [exec date +%y:%m:%d] :]
    set $yearVariable  [lindex $dateList 0]
    set $monthVariable [lindex $dateList 1]
    set $dayVariable   [lindex $dateList 2]
    APSLabeledEntryFrame $widget -parent $parent \
      -label $label \
      -variableList "$yearVariable $monthVariable $dayVariable" \
      -width 4 -orientation horizontal \
      -contextHelp "Enter the year, month, and day in the three boxes provided."
}

proc APSMakeDateTimeEntryWidget {widget args} {
    set parent ""
    set dayVariable ""
    set yearVariable ""
    set monthVariable ""
    set hourVariable ""
    set label "Date and time (year, month, day, hour): "
    APSParseArguments {parent dayVariable yearVariable monthVariable hourVariable label}
    global $monthVariable $yearVariable $dayVariable $hourVariable
    set dateList [split [exec date +%y:%m:%d:%H:%M] :]
    scan [lindex $dateList 0] "%ld" $yearVariable 
    scan [lindex $dateList 1] "%ld" $monthVariable 
    scan [lindex $dateList 2] "%ld" $dayVariable 
    scan [lindex $dateList 3] "%ld" hour
    scan [lindex $dateList 4] "%ld" minute
    set $hourVariable  [expr $hour+$minute/60.0]
    APSLabeledEntryFrame $widget -parent $parent \
      -label $label \
      -variableList "$yearVariable $monthVariable $dayVariable $hourVariable" \
      -width 4 -orientation horizontal \
      -contextHelp "Enter the year, month, day, and hour in the four boxes provided.  The hour is a floating point number from 0 to 24, so 11:30am would be entered as 11.5 ."
}

proc APSMakeYearMonthEntryWidget {widget args} {
    set parent ""
    set yearVariable ""
    set monthVariable ""
    set label "Date (year, month): "
    APSParseArguments {parent  yearVariable monthVariable label}
    global $monthVariable $yearVariable 
    set dateList [split [exec date +%y:%m] :]
    set $yearVariable  [lindex $dateList 0]
    set $monthVariable [lindex $dateList 1]
    APSLabeledEntryFrame $widget -parent $parent \
      -label $label \
      -variableList "$yearVariable $monthVariable" \
      -width 4 -orientation horizontal \
      -contextHelp "Enter the year and month in the two boxes provided."
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.2  1996/02/10 00:57:54  borland
# Added -noOk option.  Now uses APSStrictParseArguments.
#
# Revision 1.1  1996/01/16  21:38:31  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#


set apsCheckButtonsID 0
proc APSSDDSCheckButtons {widget args} {
    global apsCheckButtonsID
    incr apsCheckButtonsID
    global apsCheckButtonstkWaitVar$apsCheckButtonsID


    set callback ""
    set modal 0
    set page 0
    set arrayName ""
    set column ""
    set fileName ""
    set parent ""
    set name "Check Button Dialog"
    set limit 0
    set widthLimit 0
    set duplicate 0
    set noOk 0
    APSStrictParseArguments {callback modal page arrayName column fileName parent name \
                               limit widthLimit duplicate noOk}
    if {$limit==0 && $widthLimit==0} {
        set widthLimit 100
    }

    if {[string length $fileName]==0 || ![file exists $fileName] ||
        [string length $column]==0} { 
        return 0 
    }
    set data [APSGetSDDSColumn -fileName $fileName -column $column -page $page]
    if [llength $data]==0 {
        return 0
    } elseif {$duplicate == 0} {
        set dataindex 0
        set data [lsort $data]
        set nextdata [lindex $data [expr $dataindex + 1]]
        set newdatalist ""

        foreach elem $data {
                if {($nextdata != $elem)} {
                        lappend newdatalist [lindex $data $dataindex]
                }
                incr dataindex
                set nextdata [lindex $data [expr $dataindex + 1]]
        }
        unset data
        set data $newdatalist
    }
    global $arrayName

    APSDialogBox $widget -parent $parent -name $name 
    set w $parent$widget

    APSDialogBoxAddButton .setall -parent $w -text "SET ALL" \
      -command "APSSetAllArrayEntries $arrayName 1" \
      -contextHelp "Set all checkbuttons."
    APSDialogBoxAddButton .clear -parent $w -text "CLEAR" \
      -command "APSSetAllArrayEntries $arrayName 0" \
      -contextHelp "Clear all checkbuttons."

    set i 0
    set row 0
    set width 0
    set first 1
    foreach elem $data {
        set length [string length $elem]
        set width [expr $width+$length]
        if {$first || ($limit!=0 && $i%$limit==0) || ($widthLimit!=0 && $width>$widthLimit)} {
            set first 0
            incr row
            frame $w.userFrame.row$row -bd 2
            pack $w.userFrame.row$row -side top
            set width $length
        }
        set vname [join [concat $arrayName ( $elem )] ""]
        checkbutton $w.userFrame.row$row.cb$i -variable $vname -text $elem
        set $vname 0
        pack $w.userFrame.row$row.cb$i -side left
        incr i
    }
    if [string length $callback]==0 {
        $w.buttonRow.ok.button configure \
          -command "destroy $w; \
                        set apsCheckButtonstkWaitVar$apsCheckButtonsID OK"
        $w.buttonRow.cancel.button configure \
          -command "APSSetAllArrayEntries $arrayName 0; \
                        destroy $w; \
                        set apsCheckButtonstkWaitVar$apsCheckButtonsID CANCEL"  
    } else {
        $w.buttonRow.ok.button configure \
          -command "destroy $w; \
                        $callback $arrayName OK; \
                        set apsCheckButtonstkWaitVar$apsCheckButtonsID OK" 
        $w.buttonRow.cancel.button configure \
          -command "APSSetAllArrayEntries $arrayName 0; \
                        destroy $w; \
                        $callback $arrayName CANCEL; \
                        set apsCheckButtonstkWaitVar$apsCheckButtonsID CANCEL"  
    }
    APSEnableButton $w.buttonRow.ok.button 
    if $noOk {
        APSDisableButton $w.buttonRow.ok.button 
        APSDisableButton $w.buttonRow.cancel.button 
    }


    if {$modal} {
        set oldFocus [focus]
        grab set $w
        focus $w
        update
        set apsCheckButtonstkWaitVar$apsCheckButtonsID ""
        tkwait variable apsCheckButtonstkWaitVar$apsCheckButtonsID
        focus $oldFocus
        update
        if {[subst \$apsCheckButtonstkWaitVar$apsCheckButtonsID] == "OK"} { return 1 }
        return 0
    }
    return 1
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.1  1996/01/16 21:38:34  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

set apsRadioButtonsID 0
proc APSSDDSRadioButtons {widget args} {
    global apsRadioButtonsID
    incr apsRadioButtonsID
    global apsRadioButtonstkWaitVar$apsRadioButtonsID


    set modal 0
    set page 0
    set variable ""
    set column ""
    set fileName ""
    set parent ""
    set name "Radio Button Dialog"
    set limit 0
    set widthLimit 0
    set callback ""
    set duplicate 0
    APSParseArguments {callback modal page variable column fileName parent name limit duplicate}
    if {$limit==0 && $widthLimit==0} {
        set widthLimit 100
    }

    if {[string length $fileName]==0 || ![file exists $fileName] ||
        [string length $column]==0} { 
        return 0 
    }
    set data [APSGetSDDSColumn -fileName $fileName -column $column -page $page]
    if [llength $data]==0 {
        return 0
    } elseif {$duplicate == 0} {
        set dataindex 0
        set data [lsort $data]
        set nextdata [lindex $data [expr $dataindex + 1]]
        set newdatalist ""

        foreach elem $data {
                if {($nextdata != $elem)} {
                        lappend newdatalist [lindex $data $dataindex]
                }
                incr dataindex
                set nextdata [lindex $data [expr $dataindex + 1]]
        }
        unset data
        set data $newdatalist
    }
    global $variable

    APSDialogBox $widget -parent $parent -name $name 
    set w $parent$widget

    set i 0
    set row 0
    set width 0
    set first 1
    foreach elem $data {
        set length [string length $elem]
        set width [expr $width+$length]
        if {$first || ($limit!=0 && $i%$limit==0) || ($widthLimit!=0 && $width>$widthLimit)} {
            set first 0
            incr row
            frame $w.userFrame.row$row -bd 2
            pack $w.userFrame.row$row -side top
            set width $length
        }
        radiobutton $w.userFrame.row$row.cb$i -variable $variable -text $elem \
          -value $elem
        pack $w.userFrame.row$row.cb$i -side left
        incr i
    }
    if [string length $callback]==0 {
        $w.buttonRow.ok.button configure \
          -command "destroy $w; \
                        set apsRadioButtonstkWaitVar$apsRadioButtonsID OK"
        $w.buttonRow.cancel.button configure \
          -command "destroy $w; \
                        set apsRadioButtonstkWaitVar$apsRadioButtonsID CANCEL"  
    } else {
        $w.buttonRow.ok.button configure \
          -command "destroy $w; \
                        $callback $variable OK; \
                        set apsRadioButtonstkWaitVar$apsRadioButtonsID OK"
        $w.buttonRow.cancel.button configure \
          -command "destroy $w; \
                        $callback $variable CANCEL; \
                        set apsRadioButtonstkWaitVar$apsRadioButtonsID CANCEL"  
    }

    APSEnableButton $w.buttonRow.ok.button 

    if {$modal} {
        set oldFocus [focus]
        grab set $w
        focus $w
        update
        set apsRadioButtonstkWaitVar$apsRadioButtonsID ""
        tkwait variable apsRadioButtonstkWaitVar$apsRadioButtonsID
        focus $oldFocus
        update
        if {[subst \$apsRadioButtonstkWaitVar$apsRadioButtonsID] == "OK"} { return 1 }
        return 0
    }
    return 1
}


#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.1  1996/01/16 21:38:41  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

set ButtonBoxFont -adobe-courier-bold-r-*-*-*-120-75-75-*-*-*-*

#
# find the length of the longest string in a list
#
proc APSMaxStringLength {stringList} {
    set max 0
    foreach i $stringList {
        set len [string length $i]
        if {$len > $max} { set max $len}
    }
    return $max
}

proc MakeFrameOrWindow {widget args} {
    set toplevel 1
    set title ""
    APSParseArguments {toplevel title}
    if {$toplevel} {
        # create a top level window $widget
        if {[winfo exists $widget]} {destroy $widget}
        toplevel $widget
        raise $widget
        wm title $widget $title
    } else {
        # create a frame with the given name
        if {[winfo exists $widget]} {destroy $widget}
        frame $widget -relief raised -bd 4 
        pack $widget -side bottom
    }
    return $widget
}

proc RunSDDSListboxDataTaker {DataTaker widget LBID count x y} {
    global apsColData$LBID
    set selection [$widget nearest $y]
    set data {}
    for {set idata 0} {$idata<$count} {incr idata} {
        set varname apsColData${LBID}($idata)
        set elem [eval lindex \$$varname $selection]
        lappend data $elem
    }
    eval $DataTaker $data
}

proc APSRemoveColData {count ID } {
    global apsColData$ID
    for {set i 0} {$i<$count} {incr i} {
        unset apsColData${ID}($i)
    }
}

proc APSDefaultLabelMaker {data width} {
    set count [llength $data]
    set result {}
    for {set i 0} {$i<$count} {incr i} {
        set d0 [lindex $data  $i]
        set w0 [lindex $width $i]
        set sform [format "%%%lds" $w0]
        lappend result [format $sform $d0]
    }
    return [join $result "  "]
}

#
# Brings up listbox containing data from an SDDS file
# usage: APSMakeSDDSListbox <filename> <widget> [-toplevel 0 | 1]
#               [-title <string>] [-page <number>] [-labelmaker <procName>]
#               [-callback <procName>] <column-name> ...
#
set ColDataID 0
proc APSMakeSDDSListbox {filename widget args} {
    global ColDataID sddsListboxVar
    set count 0
    incr ColDataID
    global apsColData$ColDataID

    set toplevel 1
    set title "Listbox for SDDS file $filename"
    set page 0
    set labelmaker APSDefaultLabelMaker
    set callback ""
    APSParseArguments {toplevel title page labelmaker callback}
    APSClearOptions
    if {[llength $args]==0} {
        APSErrorPopUp "No columns named for SDDS listbox for file $filename"
        return {}
    }

    set LabelMaker $labelmaker
    set DataTaker $callback

    set maxlen {}
    foreach name $args {
        if {[string length $name]==0} {
            break;
        }
        set apsColData${ColDataID}($count) [APSGetSDDSColumn -fileName $filename -column $name -page $page]
        set width [APSMaxStringLength [eval lrange \$apsColData${ColDataID}($count) 0 end]]
        lappend maxlen $width
        incr count
    }

    set w [MakeFrameOrWindow $widget -toplevel $toplevel -title $title]

    set pane 0
    frame $w.list
    frame $w.ops
    pack $w.list $w.ops -side top
    
    listbox $w.list.lbox -font "-adobe-courier-bold-r-normal-*-*-120-75-75-*-*-*-*" \
      -yscrollcommand "$w.list.scroll set"
    scrollbar $w.list.scroll -command "$w.list.lbox yview"
    set i 0
    set maxwidth 0
    set varname apsColData${ColDataID}(0)
    foreach col [eval lrange \$$varname 0 end] {
        set data {}
        for {set idata 0} {$idata<$count} {incr idata} {
            set varname apsColData${ColDataID}($idata)
            set elem [eval lindex \$$varname $i]
            lappend data $elem
        }
        set label [$LabelMaker $data $maxlen]
        set width [string length $label]
        if {$width>$maxwidth} { 
            set maxwidth $width
            $w.list.lbox configure -width [expr $maxwidth+2]
        }
        $w.list.lbox insert end $label
        incr i
    }
    pack $w.list.lbox $w.list.scroll -side left -fill y
    if {$DataTaker != ""} {
        bind $w.list.lbox <Double-ButtonPress-1> \
          "RunSDDSListboxDataTaker $DataTaker $w.list.lbox $ColDataID $count %x %y"
        APSButton .cancel -parent $w.ops -text CANCEL -command "destroy $w; APSRemoveColData $count $ColDataID" \
          -contextHelp "Remove listbox without choosing anything."

    } else {
        APSButton .ok -parent $w.ops -text OK -command "set sddsListboxVar OK" \
          -contextHelp "Remove listbox and return highlighted entry as choice."
        APSButton .cancel -parent $w.ops -text CANCEL -command "set sddsListboxVar CANCEL" \
          -contextHelp "Remove listbox without choosing anything."
    }

    if {$DataTaker == ""} {
        set oldFocus [focus]
        grab set $w
        focus $w
        update
        
        set sddsListboxVar ""
        tkwait variable sddsListboxVar
        set selection [$w.list.lbox curselection]
        destroy $w
        focus $oldFocus
        update
        
        if {$selection == ""} {
            return {}
        }
        
        if {$sddsListboxVar == "OK"} {
            set data {}
            for {set idata 0} {$idata<$count} {incr idata} {
                set varname apsColData${ColDataID}($idata)
                set elem [eval lindex \$$varname $selection]
                unset $varname
                lappend data $elem
            }
            return $data
        }
        return {}
    }
    return {}
}

#
# Used by MakeSDDSButtons... to advance a page
#
proc GoToNextButtonPage {w pane} {
    pack forget $w.p${pane}
    incr pane
    pack $w.p$pane -side top -fill x
    update
}

#
# Used by MakeSDDSButtons... to back up a page
#
proc GoToLastButtonPage {w pane} {
    pack forget $w.p${pane}
    incr pane -1
    pack $w.p$pane -side top -fill x
    update
}

#
# Brings up buttons labeled with data from an SDDS file
# usage: APSMakeSDDSButtons <filename> <widget> -callback <procName>
#       [-toplevel 0 | 1] [-title <string>] [-limit <number>]
#       [page <number>] [-labelmaker <procName>] 
#       <columnName> ...
#
proc APSMakeSDDSButtons {filename widget args} {
    global ButtonBoxFont

    set callback ""
    set toplevel 1
    set title "Buttons for SDDS file $filename"
    set limit 10
    set page 0
    set labelmaker APSDefaultLabelMaker
    APSParseArguments {callback toplevel title limit page labelmaker}
    APSClearOptions
    if {[llength $args]==0} {
        APSErrorPopUp "No columns named for SDDS button list for file $filename"
        return {}
    }
    if {[string length $callback]==0} {
        APSErrorPopUp "No callback named for SDDS button list for file $filename"
        return {}
    }

    set count 0
    set maxwidth 0
    set maxlen {}
    foreach name $args {
        if {[string length $name]==0} {
            break;
        }
        set colData($count) [APSGetSDDSColumn -fileName $filename -column $name -page $page]
        set width [APSMaxStringLength $colData($count)]
        lappend maxlen $width
        incr count
    }
    set i 0
    foreach col0 $colData(0) {
        set data {}
        for {set idata 0} {$idata<$count} {incr idata} {
            set elem [lindex $colData($idata) $i]
            lappend data $elem
        }
        set label [$labelmaker $data $maxlen]
        set length [string length $label]
        if {$length>$maxwidth} { set maxwidth $length}
        incr i
    }
    set width $maxwidth

    set w [MakeFrameOrWindow $widget -title $title -toplevel $toplevel]

    set pane 0
    frame $w.p${pane} 
    frame $w.p${pane}.np -bd 4 
    pack $w.p${pane}.np -side top -fill x

    set i 0
    set madeNP -1
    foreach col0 $colData(0) {
        if {$i!=0} {
            if {$i%$limit==0} {
                APSButton .next -parent $w.p${pane}.np -text "NEXT PAGE" \
                  -command "{GoToNextButtonPage} $w ${pane}" \
                  -contextHelp "Go to the next page of buttons."
                APSButton .ok -parent $w.p${pane}.np -text "OK" \
                  -command "destroy $w" \
                  -contextHelp "Destroy button pages and continue."
                if {$pane != 0} {
                    APSButton .previous -parent $w.p${pane}.np -text "PREVIOUS PAGE" \
                      -command "GoToLastButtonPage $w ${pane}" \
                      -contextHelp "Go to the previous page of buttons."
                }
                set madeNP $pane
                incr pane
                frame $w.p${pane} 
                frame $w.p${pane}.np -bd 4 
                pack $w.p${pane}.np -side top -fill x
            }
        }   
        set data {}
        for {set idata 0} {$idata<$count} {incr idata} {
            set elem [lindex $colData($idata) $i]
            lappend data $elem
        }
        APSButton .b$i -parent $w.p${pane} \
          -text  [$labelmaker $data $maxlen ] \
          -command "$callback $data" \
          -contextHelp "Invoke processing of with the data indicated on the label."    \
          -noPack 1
        $w.p${pane}.b$i.button configure -font $ButtonBoxFont 
        pack $w.p${pane}.b$i -side top -fill x
        incr i
    }
    if {$madeNP != $pane} {
        if {$pane != 0} {
            APSButton .previous -parent $w.p${pane}.np -text "PREVIOUS PAGE" \
              -command "pack forget $w.p${pane}; pack $w.p[expr $pane-1]" \
              -contextHelp "Go to the previous page of buttons."
            APSButton .ok -parent $w.p${pane}.np -text "OK" \
              -command "destroy $w" \
              -contextHelp "Destroy button pages and continue."
        }
    }                
    pack $w.p0 -side top -fill x
}

#
# Make check buttons for a column of SDDS data
# usage: APSMakeSDDSCheckButtons <filename> <widget> <columnName> <arrayName> 
#       [-toplevel {0 | 1}] [-title <title>] [-page <page>] [-limit <number>] 
#       [-callback <procName>]
#

proc APSMakeSDDSCheckButtons {filename widget column arrayName args} {
    global $arrayName sddsCheckbuttonVar

    set toplevel 1
    set title "Check buttons for column $column of file $filename"
    set page 0
    set limit 8
    set callback ""
    set nograb 0
    APSParseArguments {toplevel title page limit callback nograb}

    set data [APSGetSDDSColumn -fileName $filename -column $column -page $page]
    
    set w [MakeFrameOrWindow $widget -title $title -toplevel $toplevel]

    frame $w.op 
    pack $w.op -side top
    if {[string length $callback]==0} {
        APSButton .ok -parent $w.op -text OK -command "set sddsCheckbuttonVar OK" \
          -contextHelp "Return with selected checkbuttons as choices."
        APSButton .cancel -parent $w.op -text CANCEL -command "set sddsCheckbuttonVar CANCEL" \
          -contextHelp "Return without making a choice."
        APSButton .clear -parent $w.op -text CLEAR -command "APSSetAllArrayEntries $arrayName 0" \
          -contextHelp "Clear all selected checkbuttons."
        APSButton .set -parent $w.op   -text "SET ALL" -command "APSSetAllArrayEntries $arrayName 1" \
          -contextHelp "Choose all checkbuttons."
    } else {
        APSButton .ok -parent $w.op -text OK -command "destroy $w" \
          -contextHelp "Return with selected checkbuttons as choices."
        APSButton .clear -parent $w.op -text CLEAR -command "APSSetAllArrayEntries $arrayName 0; $callback" \
          -contextHelp "Clear all selected checkbuttons."
        APSButton .set -parent $w.op   -text "SET ALL" -command "APSSetAllArrayEntries $arrayName 1; $callback" \
          -contextHelp "Choose all checkbuttons."
    }

    set i 0
    set row 0
    foreach elem $data {
        if {$i%$limit==0} {
            incr row
            frame $w.row$row -bd 2
            pack $w.row$row -side top
        }
        set vname [join [concat $arrayName ( $elem )] ""]
        checkbutton $w.row$row.cb$i -variable $vname -text $elem
        if {[string length $callback]} {
            bind $w.row$row.cb$i <ButtonPress-1> "$callback"
            bindtags $w.row$row.cb$i "Checkbutton $w.row$row.cb$i $w all"
        }
        set $vname 0
        pack $w.row$row.cb$i -side left
        incr i
    }
    if {[string length $callback]==0} {
        if {!$nograb} {
            set oldFocus [focus]
            grab set $w
            focus $w
            update
        }

        set sddsCheckbuttonVar ""
        tkwait variable sddsCheckbuttonVar
        destroy $w
        if {!$nograb} {
            focus $oldFocus
            update
        }
        if {$sddsCheckbuttonVar == "OK"} { return 1 }
        return 0
    }
}









#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.3  1996/03/18 17:51:24  borland
# Added chmod to APSUpdateUseLog to guarantee that use logs are readable by
# everyone.
#
# Revision 1.2  1996/02/01  22:08:09  borland
# Changed gunzip command in UnpackFile to gzip command with "no-name" option.
#
# Revision 1.1  1996/01/16  21:38:44  saunders
# Unofficial routines from ~borland/tcl/rel added. These should
# ultimately be removed from use or modified to comply with
# current conventions.
#
#

# set preferredFont -Adobe-Helvetica-Bold-R-Normal--*-120-*
# option add *font $preferredFont

proc APSStatusLine {widget args} {
    set variable ""
    set parent ""
    set width 60
    APSParseArguments {variable parent width}
    if {$variable==""} { return 0 }
    global $variable
    APSLabeledOutput $widget -parent $parent -label "Status: " -width $width -textVariable $variable \
      -contextHelp "Status display line."
}

proc APSSetAllArrayEntries {arrayName value} {
    upvar $arrayName dataArray
    foreach elem [array names dataArray] {
        set dataArray($elem) $value
    }
}

set apsTmpStringCount 0
proc APSTmpString { } {
    global env apsTmpStringCount
    set string [exec date +%y%m%d-%H%M%S]-[pid]
    set string ${string}$env(USER)$apsTmpStringCount
    incr apsTmpStringCount
    return $string
}

proc APSYesNoPopUp {ques} {
    global apsYesNoPopUpAnswer
    set w .yesno
    toplevel $w
    wm title $w ""
    frame $w.message -relief raised -bd 2
    pack $w.message -side top
    message $w.message.text -text "$ques" -width 50m
    pack $w.message.text
    frame $w.buttons -bd 2
    button $w.buttons.yes -text YES -command "set apsYesNoPopUpAnswer 1" -relief raised 
    button $w.buttons.no -text NO -command "set apsYesNoPopUpAnswer 0" -relief raised 
    pack $w.buttons -side top
    pack $w.buttons.yes $w.buttons.no -side left 
    
    set oldFocus [focus]
    grab set $w
    focus $w
    
    tkwait variable apsYesNoPopUpAnswer 
    destroy $w
    update
    focus $oldFocus
    return $apsYesNoPopUpAnswer
}

proc APSErrorPopUp {message} {
    global apsErrorPopUpAnswer
    set w .error
    toplevel $w
    wm title $w ""
    frame $w.message -relief raised -bd 2
    pack $w.message -side top
    message $w.message.text -text "Error: $message" -width 50m
    pack $w.message.text
    frame $w.buttons -bd 2
    button $w.buttons.ok -text OK -command "set apsErrorPopUpAnswer 1" -relief raised 
    pack $w.buttons -side top
    pack $w.buttons.ok -side left 
    
    set oldFocus [focus]
    tkwait visibility $w
    grab set $w
    focus $w
#    bind $w <Visibility> "DCTraise $w"
    
    tkwait variable apsErrorPopUpAnswer 
    destroy $w
    update
    focus $oldFocus
}

proc APSMakeExitButton {widget} {
    frame $widget -bd 2
    pack $widget -side top 
    button $widget.exit -text EXIT -command exit -relief raised 
    pack $widget.exit -side left 
}

proc APSMakeLabeledEntries {widget labelText args} {
    set ewidth 15
    set lanchor w
    set eanchor e
    set eside left
    set estate normal
    set erelief sunken
    set fside top
    set lwidth [string length $labelText]
    APSParseArguments {lwidth ewidth lanchor eside estate erelief fside}
    switch $lanchor {
        e       {set eanchor w ; set lside right}
        w       {set eanchor e ; set lside left}
        n       {set eanchor s ; set lside top}
        s       {set eanchor n ; set lside bottom}
    }                
    frame $widget  
    pack $widget -side $fside -fill x 

    label $widget.label -text "$labelText"   -width $lwidth
    pack $widget.label -anchor $lanchor -side $lside  -fill x

    frame $widget.entries
    pack $widget.entries -anchor $eanchor -side $lside -fill x
    set i 0
    foreach arg $args {
        entry $widget.entries.entry$i -textvariable $arg -width $ewidth \
          -relief $erelief -bg white -state $estate
        pack $widget.entries.entry$i -side $eside  -padx 2
        incr i
    }
}

proc APSMakeLabeledOutputs {widget labelText args} {
    eval APSMakeLabeledEntries $widget \"$labelText\" -estate disabled -erelief ridge $args 
}


proc APSUpdateUseLog {filename mode} {
    global env
    set tmpfile /tmp/[APSTmpString].$env(USER)
    set tmpexten [APSTmpString]
    set fout [open $tmpfile w]
    puts $fout {SDDS1}
    puts $fout {&column name=YearUsed type=long &end} 
    puts $fout {&column name=MonthUsed type=long &end} 
    puts $fout {&column name=DayUsed type=long &end} 
    puts $fout {&column name=HourUsed type=float &end} 
    puts $fout {&column name=JulianDayUsed type=long &end} 
    puts $fout {&column name=TimeStamp type=string format_string=%28s &end} 
    puts $fout {&column name=Mode type=string format_string=%12s &end}  
    puts $fout {&column name=User type=string format_string=%10s &end} 
    puts $fout {&data mode=ascii no_row_counts=1 &end} 
    set timeString [exec date +%y:%m:%d:%H:%M:%S:%j]
    set TimeStamp [exec date]
    scan $timeString "%d:%d:%d:%d:%d:%d:%d" YearUsed MonthUsed DayUsed Hour Min Sec JDayUsed
    if {$YearUsed<1995} {
        if {$YearUsed>94} {
            set YearUsed [expr $YearUsed+1900]
        } else {
            set YearUsed [expr $YearUsed+2000]
        }
    }
    set HourUsed [expr $Hour+($Min+$Sec/60.0)/60.0]
    puts $fout "$YearUsed $MonthUsed $DayUsed $HourUsed $JDayUsed \"$TimeStamp\" $mode $env(USER)" 
    close $fout
    if {[file exists $filename]} {
        exec sddscombine $tmpfile $filename -merge -overwrite $filename.$tmpexten
        set junk $filename.[APSTmpString]
        exec mv $filename $junk
        exec mv $filename.$tmpexten $filename
        exec rm -f $junk
        catch {exec chmod a+r $filename}
    } else {
        exec mv $tmpfile $filename
        catch {exec chmod a+r $filename}
    }
}

#
# make a string save to use in an commandline qualifier
#

proc APSMakeSafeQualifierString {comment} {
    regsub -all "," $comment "\\\," comment1
    return $comment1
}


set APSErrorText "No error."
proc APSErrorReturn {code text} {
    global APSErrorText
    set APSErrorText $text
    return $code
}

proc APSGetErrorText {} {
    global APSErrorText
    return $APSErrorText
}

proc APSTrimLeadingChars {args} {
    set character ""
    set text ""
    APSParseArguments {character text}
    if [string length $character]==0 { return $text }
    if [string length $text]==0 { return $text }
    regsub -all "^(\\$character)+" $text "" newtext
    return $newtext
}


proc APSUnpackFile {args} {
    set fileName ""
    set unpackMode ""
    set unpackedFile ""
    APSParseArguments {fileName unpackMode unpackedFile}
    if {[string length $fileName]==0 || ![file exists $fileName]} {
        return 0
    }
    
    set unpackCommand(freeze) "freeze -dc "
    set unpackCommand(compress) "uncompress -c "
    set unpackCommand(gzip) "gzip -dcn "
    set packSuffix(freeze) ".F"
    set packSuffix(compress) ".Z"
    set packSuffix(gzip) ".gz"

    switch $unpackMode {
        none   {
            return -1 
        }
        auto   { 
            set suffix [file extension $fileName]
            if [string length $suffix]==0 {
                return 0
            }
            set pairs [array get packSuffix]
            set index [lsearch -exact $pairs $suffix]
            if $index==-1 {
                # assume no unpacking is needed
                return -1
            }
            incr index -1
            set command $unpackCommand([lindex $pairs $index])
        }
        freeze -
        compress -
        gzip {
            set command $unpackCommand($unpackMode)
        }
        default {
            return 0
        }
    }
    catch {eval exec $command $fileName > $unpackedFile} message
    if [string length $message] { 
        APSAlertBox .[APSTmpString] -errorMessage "Unable to unpack $fileName:\n$message"
        return 0
    }
    return 1
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.3  1996/03/17 01:32:43  borland
# Eliminated some blank lines that cppatch complained about.
#
# Revision 1.2  1996/03/17  01:31:42  borland
# Added sorting and tolerance overrides.
#
# Revision 1.1  1996/03/17  00:28:19  borland
# First version.
#
#

set requestDir /home/helios/oagData/SCR/requestFiles
set snapDir /home/helios/oagData/SCR/snapshots

proc APSSaveMachine {args} {
    global requestDir snapDir
    set machine ""
    set description ""
    set snapshot ""
    if [APSStrictParseArguments {machine description snapshot}] {
        return -code error {}
    }

    if ![file exists $requestDir/$machine.req] {
        return -code error -errorinfo "Bad machine name $machine" {}
    }

    set oldDir [pwd]
    cd $snapDir
    set dt [exec date +%y%m%d-%H%M%S]
    while {[string length $snapshot]==0} {
        set snapshot ${machine}${dt}
        if [file exists $snapshot] {
            set snapshot ""
            set dt [exec date +%y%m%d-%H%M%S]
        }
    }

    if [string length $description] {
        set description [APSMakeSafeQualifierString $description]
    }

    if [catch {exec burtrb -f ${requestDir}/${machine}.req -sdds -l /tmp/burtlog$dt \
                 | sddsconvert -binary -pipe \
                 | sddsxref -pipe ${requestDir}/${machine}.req -take=* \
                 | sddsprocess -pipe=in $snapshot \
                 "-print=parameter,SnapshotDescription,$description" \
                 "-print=parameter,RequestFile,${requestDir}/${machine}.req" \
                 "-print=parameter,SnapshotFilename,${snapshot}"} result] {
        if ![file exists $snapshot] {
            cd $oldDir
            return -code error -errorinfo "$result"  {}
        }
    }

    set filename [file tail $snapshot]
    set pathname [file dirname $snapshot]

    exec gzip $snapshot
    exec chmod -w [glob ${snapshot}*]
    set snapshot $snapshot.gz

    exec sddscollapse $snapshot $pathname/c$filename &
    exec sddssort $snapshot -column=Category -unique $pathname/categ$filename  &
    exec sddssort $snapshot -column=Beamline -unique $pathname/beam$filename  &

    cd $oldDir
    if [file size /tmp/burtlog$dt] {
        return -code ok [list $snapshot /tmp/burtlog$dt]
    }
    return -code ok $snapshot
}

proc APSCompareMachine {args} {
    global snapDir requestDir
    set machine ""
    set snapshot1 ""
    set snapshot2 ""
    set graphics 0
    set percentTolerance 0
    set absoluteTolerance 0
    set sortChoice none

    if [APSStrictParseArguments {machine snapshot1 snapshot2 graphics percentTolerance absoluteTolerance \
                               sortChoice}]==-1 {
        return -code error -errorinfo "Bad arguments to APSCompareMachine" {}
    }
    if $absoluteTolerance>0 {
        set percentTolerance 0
    }

    if ![string length $snapshot2] {
        if ![file exists $requestDir/$machine.req] {
            return -code error -errorinfo "Bad machine name $machine" {}
        }
    }

    set oldDir [pwd]
    cd $snapDir

    set tmproot /tmp/[APSTmpString]

    if ![file exists $snapshot1] {
        set snapshot1 = ${snapshot1}.gz
    } 
    if ![file exists $snapshot1] {
        cd $oldDir
	return -code error -errorinfo "$snapshot1 does not exist" {}
    }
    set snap1Label "$snapshot1"

    if [string length $snapshot2]==0 {
	set snapshot2 ${tmproot}
	if [catch {APSSaveMachine -machine $machine \
                     -description "APSCompareMachine temporary snapshot"  \
                     -snapshot $snapshot2} result] {
            cd $oldDir
            return -code error -errorinfo "Unable to make temporary snapshot" {}
        }
        set snapshot2 [lindex $result 0]
	set snap2Label "present configuration"
    } else {
        set snap2Label $snapshot2
    }

    if ![file exists $snapshot2] {
        set snapshot2 ${snapshot2}.gz
    }
    if ![file exists $snapshot2] {
        cd $oldDir
        return -code error -errorinfo "$snapshot2 does not exist" {}
    }
	
    set rintFile1 ${tmproot}-1
    set sdiffFile ${tmproot}-2
    set printFile2 ${tmproot}-3
    set diffFile ${tmproot}-4
    set bothSnaps ${tmproot}-5
    set enumDiff ${tmproot}-6

    if $absoluteTolerance>0 {
        set redefToleranceArg "-redefine=column,Tolerance,$absoluteTolerance" 
        set toleranceLabel "using absolute tolerance of $absoluteTolerance"
    } elseif $percentTolerance>0 {
        set redefToleranceArg "-redefine=column,Tolerance,Reference [expr $percentTolerance/100.0] * abs"
        set toleranceLabel "using percent tolerance of $percentTolerance"
    } else {
        set redefToleranceArg "-filter=col,Tolerance,0,1e300"
        set toleranceLabel "using default tolerances"
    }

    exec sddsprocess $snapshot2 -pipe=out "-match=col,ValueString=,!" -nowarn \
      | sddsconvert -pipe -rename=col,ValueString=NewValueString -binary -nowarn \
      | sddsxref $snapshot1 -pipe -match=ControlName -take=* -nowarn \
      | tee $bothSnaps \
      | sddsprocess -pipe -nowarnings -match=col,IsNumerical=y -nowarn \
      "-match=col,ValueString=,!" \
      "-scan=col,Value,NewValueString,%lf" \
      "-scan=col,Reference,ValueString,%lf" \
      $redefToleranceArg \
      "-define=column,Difference,Value Reference -" \
      | tee $diffFile \
      | sddsprocess -pipe=in $sdiffFile -nowarn \
      "-define=col,AbsDifference,Difference abs,format_string=%12.6f" \
      "-define=col,ExcessDifference,AbsDifference Tolerance -" \
      "-filter=col,ExcessDifference,-1e300,0,!"

    exec sddsprocess $bothSnaps $enumDiff "-match=col,IsNumerical=y,!" \
      "-match=col,ValueString=@NewValueString,!" -nowarning

    catch {exec sdds2stream $sdiffFile -rows} rows
    exec echo $rows with significant difference  > $printFile1
    switch -- $sortChoice {
        byValue {
            set sortArg -column=Reference,incr
        }
        byError {
            set sortArg -column=AbsDifference,decr
        }
        byName {
            set sortArg -column=ControlName
        }
        default {
            set sortArg ""
        }
    }
    if [string length $sortArg] {
        set newDiff ${tmproot}-7
        if ![catch {exec sddssort $sdiffFile $newDiff $sortArg} result] {
            exec mv $newDiff $sdiffFile
        }
    }
    exec sddsprintout -column=ControlName,format=%-30s \
      "-title=Numerical differences between $snap1Label and $snap2Label $toleranceLabel"  \
      "-col=(Reference,AbsDifference,Tolerance),format=%13.5g"  \
      $sdiffFile >> $printFile1

    catch {exec sdds2stream $enumDiff -rows} rows
    exec echo $rows with enumerated-value difference > $printFile2
    if [string compare $sortChoice byName]==0 {
        set newDiff ${tmproot}-8
        if ![catch {exec sddssort $enumDiff $newDiff $sortArg} result] {
            exec mv $newDiff $enumDiff
        }
    }
    exec sddsprintout $enumDiff -column=ControlName,format=%-30s \
      "-title=Enumerated differences between $snap1Label and $snap2Label"  \
      "-column=(ValueString,NewValueString),format=%22s" \
        >> $printFile2 

    if $graphics {
        APSFileDisplayWindow [APSUniqueName .] \
          -fileName $printFile1 -deleteOnClose 1 -width 100 \
          -comment "Numerical-value snapshot differences"
        APSFileDisplayWindow [APSUniqueName .] \
          -fileName $printFile2 -deleteOnClose 1 -width 100 \
          -comment "Enumerated-value snapshot differences"
        switch $machine {
            PLET {
                exec plotPLETcomparison $diffFile $enumDiff $tmproot \
                  $snap1Label $snap2Label 1 & 
            }
            BTS {
                exec plotBTScomparison $diffFile $enumDiff $tmproot \
                  $snap1Label $snap2Label 1 & 
            }
            default {
                exec rm $diffFile $enumDiff
            }
        }
        exec rm $sdiffFile $bothSnaps
        cd $oldDir
        return {}
    } else {
        exec rm $bothSnaps $diffFile
        cd $oldDir
        return [list $sdiffFile $enumDiff $printFile1 $printFile2]
    }
}

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.4  1996/03/10 23:50:04  borland
# Fixed use of contextHelp option.
#
# Revision 1.3  1996/03/10  23:47:31  borland
# Fixed destroy command for DONE button.
#
# Revision 1.2  1996/03/10  22:15:40  borland
# First name clash with help routine.
#
# Revision 1.1  1996/03/10  22:13:33  borland
# First version.
#
#


#
# find the length of the longest string in a list
#
proc APSMaxStringLength {stringList} {
    set max 0
    foreach i $stringList {
        set len [string length $i]
        if {$len > $max} { set max $len}
    }
    return $max
}

proc APSMaxStringLengthHelp {} {
    return "Usage: APSMaxStringLength <list>"
}


proc APSSDDSListboxDefaultLabelMaker {args} {
    set dataList {}
    set widthList {}
    APSStrictParseArguments {dataList widthList}
    set count [llength $dataList]
    set result {}
    for {set i 0} {$i<$count} {incr i} {
        set d0 [lindex $dataList  $i]
        set w0 [lindex $widthList $i]
        set sform [format "%%%lds" $w0]
        lappend result [format $sform $d0]
    }
    return [join $result "  "]
}

proc APSSDDSListboxHelp {} {
    return "Usage: APSSDDSListbox <widget> -fileName <fileName> \[-parent <parent>\] \
\[-page <page>\] \[-page <page>\] \[-labelMaker <procName>\] \[-callback <procName>\] \
\[-columnList <list>\]"
}

#
# Brings up listbox containing data from an SDDS file
# usage: APSSDDSListbox <widget> -fileName <filename>  -parent <parent>
#               [-page <number>] [-labelMaker <procName>]
#               [-callback <procName>] 
#               [-columnList <list>]
#
proc APSSDDSListbox {widget args} {
    set fileName ""
    set parent ""
    set page 0
    set labelMaker ""
    set callback ""
    set columnList {}
    set arrayName ""
    set contextHelp "No context help available."

    APSStrictParseArguments {fileName parent page labelMaker callback columnList contextHelp arrayName}

    set LabelMaker $labelMaker
    global $arrayName

    set maxlen {}
    foreach name $columnList {
        if {[string length $name]==0} {
            break;
        }
        set ${arrayName}($name) [APSGetSDDSColumn -fileName $fileName -column $name -page $page]
        set width [APSMaxStringLength [subst \$${arrayName}($name)]]
        lappend maxlen $width
    }

    APSFrame $widget -parent $parent -label "" \
      -contextHelp "$contextHelp"

    set w $parent$widget.frame

    frame $w.list
    frame $w.ops
    pack $w.list $w.ops -side top
    
    listbox $w.list.lbox -font "-adobe-courier-bold-r-normal-*-*-120-75-75-*-*-*-*" \
      -yscrollcommand "$w.list.scroll set"
    scrollbar $w.list.scroll -command "$w.list.lbox yview"
    set i 0
    set maxwidth 0
    set itemsInList [llength [subst \$${arrayName}([lindex $columnList 0])]]
    for {set i 0} {$i<$itemsInList} {incr i} {
        set data {}
        foreach name $columnList {
            lappend data [lindex [subst \$${arrayName}($name)] $i]
        }
        set label [$LabelMaker -dataList $data -widthList $maxlen]
        set width [string length $label]
        if {$width>$maxwidth} { 
            set maxwidth $width
            $w.list.lbox configure -width [expr $maxwidth+2]
        }
        $w.list.lbox insert end $label
    }
    pack $w.list.lbox $w.list.scroll -side left -fill y

    bind $w.list.lbox <Double-ButtonPress-1> \
      "APSSDDSListboxRunCallback $w.list.lbox -callback $callback -y %y"

    APSButton .done -parent $w.ops -text DONE \
      -command "destroy $parent$widget" \
      -contextHelp "Remove listbox."

} 

proc APSSDDSListboxRunCallback {widget args} {
    set callback ""
    set y 0
    APSStrictParseArguments {callback y}
    eval $callback [$widget nearest $y]
}
#!/bin/sh  
# \
exec wish "$0" "$@"

#
# APSlib.tcl,v
# Revision 1.4  2002/08/02 15:38:36  jba
# Added license information.
#
# Revision 1.3  1997/02/06 22:33:35  jba
# Changes from John A.
#
# Revision 1.3  1996/03/22 04:18:31  borland
# Added routine to set PM charge enables.
#
# Revision 1.2  1996/03/18  23:17:38  borland
# Added exec command before cavputs.
#
# Revision 1.1  1996/03/18  23:13:48  borland
# First version.
#
#

#set auto_path [linsert $auto_path 0  /usr/local/oag/apps/lib/$env(HOST_ARCH)]
#set auto_path [linsert $auto_path 0 /usr/local/oag/lib_patch/$env(HOST_ARCH)]

proc TogglePulsedMagnetEnables {args} {
    set location All
    APSStrictParseArguments {location}
    set PARList "It:Ddg2chan4.GATE=<value>,It:ER1:er.OTL2=<value>,It:ParInjChargeDDG.GATE=<value>,It:Par:PSPchargeBO.VAL=<value>"
    set BoosterInjList "It:Ddg3chan2.GATE=<value>,It:Ddg2chan6.GATE=<value>"
    set BoosterExtList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>,Mt:Ddg1chan0.GATE=<value>"
    switch $location {
        PAR {
            set pvList $PARList
        }
        BoosterInj {
            set pvList $BoosterInjList
        }
        PARtoBoosterInj {
            set pvList $PARList,$BoosterInjList
        }
        BoosterExt {
            set pvList $BoosterExtList
        }
        PARtoBoosterExt -
        All {
            set pvList $PARList,$BoosterInjList,$BoosterExtList
        }
        default {
            return -code error -errorinfo "Invalid location: $location"
        }
    }
    
    regsub -all "<value>" $pvList 0 pvList0
    regsub -all "<value>" $pvList 1 pvList1
    if [catch {exec cavput -list=$pvList0} result] {
        return -code error -errorinfo "Error trying to turn off timing enables."
    }
    after 1000
    if [catch {exec cavput -list=$pvList1} result] {
        return -code error -errorinfo "Error trying to turn on timing enables."
    }
    return -code ok
}

proc SetPulsedMagnetEnables {args} {
    set location All
    set state 0
    APSStrictParseArguments {location state}
    set PARList "It:Ddg2chan4.GATE=<value>,It:ER1:er.OTL2=<value>,It:ParInjChargeDDG.GATE=<value>,It:Par:PSPchargeBO.VAL=<value>"
    set BoosterInjList "It:Ddg3chan2.GATE=<value>,It:Ddg2chan6.GATE=<value>"
    set BoosterExtList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>,Mt:Ddg1chan0.GATE=<value>"
    switch $location {
        PAR {
            set pvList $PARList
        }
        BoosterInj {
            set pvList $BoosterInjList
        }
        PARtoBoosterInj {
            set pvList $PARList,$BoosterInjList
        }
        BoosterExt {
            set pvList $BoosterExtList
        }
        PARtoBoosterExt -
        All {
            set pvList $PARList,$BoosterInjList,$BoosterExtList
        }
        default {
            return -code error -errorinfo "Invalid location: $location"
        }
    }
    
    regsub -all "<value>" $pvList $state pvList0
    if [catch {exec cavput -list=$pvList0} result] {
        return -code error -errorinfo "Error trying to change timing enables."
    }
    return -code ok
}

