#*************************************************************************
# Copyright (c) 2002 The University of Chicago, as Operator of Argonne
# National Laboratory.
# Copyright (c) 2002 The Regents of the University of California, as
# Operator of Los Alamos National Laboratory.
# This file is distributed subject to a Software License Agreement found
# in the file LICENSE that is included with this distribution. 
#*************************************************************************
#------------------------------------------
# dct.tcl    5/1/97 (was dct2.tcl B4) 
#------------------------------------------
#------------------------------------------

#------------------------------------------
# File Menu action callbacks
#------------------------------------------

proc DCTChangedDialog {action} {
     global GLOBALdatabaseChanged
     if { $GLOBALdatabaseChanged } {
          APSDialogBoxR .changedWarning -name "Warning: Possible Data Loss" \
          -contextHelp "Database changed warning."
          grab .changedWarning
          focus .changedWarning
          set w .changedWarning.buttonRow.ok.button
          set parent .changedWarning.userFrame
          label $parent.changedLabel -text \
	  "The requested operation may cause the loss of your
       most recent .db file changes. To save your newest
       data,  press  Cancel,  press (Save field values), 
       and then	choose  SAVE  from the File Menu.

      If the loss of this new .db file data does not matter,
       then just press OK."
          pack $parent.changedLabel
          $w configure -state normal -command "destroy .changedWarning;$action"
        #  set GLOBALdatabaseChanged 0
     } else {
         $action
     }
}

proc DCTFilePrintAction {} {
    tk_dialog .printSelection {Print Selection} {This is a dummy print selection dialog box, which is used because print isn't implemented yet.} {} 0 OK
     puts stderr {dummy file name}
}

#------------------------------------------
# Edit Menu action callbacks
#------------------------------------------

proc DCTEditCreateAction { widget } {
    set windowExists [winfo exists $widget]
    if {$windowExists} {
        raise $widget
        focus $widget
        return
    }
    DCTEditCreateRecord $widget
}

proc DCTEditNameAction { widget action} {
    set windowExists [winfo exists $widget]
    if {$windowExists} {
        raise $widget
        focus $widget
        return
    }
    APSDialogBox $widget -name "$action Record" \
    -contextHelp "$action a database record."
    set w $widget.buttonRow.ok.button
    if {"$action"=="Rename"} {
         $w configure -text "Rename" -state normal
         $w configure -command "DCTRenameRecordOK"
    } else {
         $w configure -text "Copy" -state normal
         $w configure -command "DCTCopyRecordOK"
    }
    $widget.buttonRow.cancel.button configure -text "Dismiss"

    global GLOBALselectedRecordName
    global newRenameRecordName
    global newCopyRecordName
    set parent $widget.userFrame
    label $parent.renameOldNameLabel -text "From Record Name:" \
        -anchor w
    label $parent.renameOldNameData -textvariable GLOBALselectedRecordName \
        -width 28 -relief sunken  
    APSLabeledEntry .labelEntry -parent $parent -width 50\
        -label "To  Record  Name:" -packOption "-side left -fill x -expand true"
    if {"$action"=="Rename"} {
         $parent.labelEntry.entry configure -textvariable newRenameRecordName
    } else {
         $parent.labelEntry.entry configure -textvariable newCopyRecordName
    }
    pack  $parent.renameOldNameLabel -side left
    pack  $parent.renameOldNameData  -side left  -fill x -expand true
    pack  $parent.labelEntry  -side bottom   
    pack  $parent.labelEntry.entry  -side left   -fill x -expand true
}


proc DCTEditDeleteAction { widget } {
    set windowExists [winfo exists $widget]
    if {$windowExists} {
        raise $widget
        focus $widget
        return
    }
    APSDialogBox $widget -name "Delete Record" \
         -contextHelp "Delete selected record" -width 50
    $widget.buttonRow.ok.button configure -text "Delete" -state normal \
         -command "DCTDeleteRecordOK $widget"
    $widget.buttonRow.cancel.button configure -text "Cancel"

    set parent $widget.userFrame
    global GLOBALselectedRecordName
    label $parent.deleteNameConfirmText \
        -text "Delete record?"
    label $parent.deleteNameConfirm \
	-textvariable GLOBALselectedRecordName
    pack $parent.deleteNameConfirmText $parent.deleteNameConfirm -side left
}

proc DCTEditPrefixAction { widget } {
    set windowExists [winfo exists $widget]
    if {$windowExists} {
        raise $widget
        focus $widget
        return
    }
    APSDialogBox $widget -name "Prefix" \
         -contextHelp "Record name prefix"
    $widget.buttonRow.ok.button configure -text "OK" -state normal \
         -command "DCTPrefixEntryOK $widget"
    $widget.buttonRow.cancel.button configure -text "Cancel"

    global GLOBALrecordNamePrefix
    set parent $widget.userFrame
    APSLabeledEntry .prefix -parent $parent -width 28\
        -label "Prefix:" -packOption "-side top -fill x"
    $parent.prefix.entry configure -textvariable GLOBALrecordNamePrefix
}

proc DCTEditSortAction {} {
    tk_dialog .selection {Sort Selection} {Not implemented yet.} {} 0 OK
}


#------------------------------------------
# View Menu action callbacks
#------------------------------------------

proc DCTViewAction {} {
    tk_dialog .viewSelection {View Selection} {This is a dummy report, which is used because the reports are not created yet.} {} 0 OK
    puts stderr {dummy view name}
}


#------------------------------------------
# File OK callbacks
#------------------------------------------

proc DCTTotalRecords {} {
     global GLOBALtotalRecords
     set GLOBALtotalRecords 0
     set numRecdes "[DCTdbGetNRecordTypes]"
     set quit [expr $numRecdes -1]
     if { "$numRecdes" == "0" } {
          return
     }
     set status [DCTdbFirstRecordType]
     for {set j 0} {$j < $numRecdes} {incr j} {
          set numRecords "[DCTdbGetNRecords]"
          set GLOBALtotalRecords [expr $GLOBALtotalRecords + $numRecords]
	  if {$j < $quit}  {
              set status [DCTdbNextRecordType]
          }
     }
}

proc DCTClearOK {} {
     DCTdbFreeEntry;
     DCTdbFreeBase;
     DCTdbAllocBase;
     DCTdbAllocEntry;
     global GLOBALdatabaseChanged
     global GLOBALfilename
     global GLOBALtotalRecords
     set GLOBALfilename "no filename"
     set GLOBALtotalRecords 0
     set GLOBALdatabaseChanged 0
     DCTSetScrolledRecordList
}

proc DCTNewOK {} {
     global GLOBALdatabaseChanged

     set numRecdes "[DCTdbGetNRecordTypes]"
     if { "$numRecdes" == "0" } {
	  set  GLOBALdatabaseChanged 0
          return
     }
#  puts "in DCTNewOK : numRecdes=$numRecdes"
     set status [DCTdbFirstRecordType]
     set  quit [expr $numRecdes -1 ]
     for {set j 0} {$j < $numRecdes} {incr j} {
          set numRecords "[DCTdbGetNRecords]"
          if { "$numRecords" == "0" } {
               set status [DCTdbNextRecordType]
	       continue   
          }
# puts "j=$j"
# puts "tcl ... numRecords to be deleted=$numRecords"
          for {set i 0} {$i < $numRecords} {incr i} {
	       set status [DCTdbFirstRecord]
               set status "[DCTdbDeleteRecord]"
          }
	  if {$j < $quit} {
              set status [DCTdbNextRecordType]
          }	      
     }
     set status [DCTdbFirstRecordType]
     global GLOBALdatabaseChanged
     set GLOBALdatabaseChanged 0
     global GLOBALfilename
     global GLOBALtotalRecords
     set GLOBALfilename "no filename"
     set GLOBALtotalRecords 0
     DCTSetScrolledRecordList
# puts "NewOK finished"
}

# this is a test DCTNewOK using C pgm  DCTdbDelAllRecIns
proc DCTNewOK2 {} {
     set numRecdes "[DCTdbGetNRecordTypes]"
     if { "$numRecdes" == "0" } {
          return
     }
# puts "numRecdes=$numRecdes"

     set status [DCTdbDelAllRecIns]

     set status [DCTdbFirstRecordType]
     global GLOBALdatabaseChanged
     set GLOBALdatabaseChanged 0
     DCTSetScrolledRecordList
# puts "NewOK finished"
}


proc DCTOpenOK {} {
# puts "*** DCTOpenOK entered ... request OPEN of a file ***"
# puts "*** call DCTNewOK ***"
     DCTNewOK
# puts "*** call DCTAppendOK ***"	

     DCTAppendOK
}

proc DCTAppendOK {} {
     global apsFileSelect
     global GLOBALfilename
     global GLOBALtotalRecords

     # puts "*** DCTAppendOK entered ***"
     set apsFileSelect(done) 1
     set windowExists [winfo exists .openSelection]
     if {$windowExists} {
          raise .openSelection
          focus .openSelection
          return
     }
     set filename [APSFileSelectDialog .openSelection -title "Append file"]
     if { "$filename" != "" } {
          set dirname [file dirname $filename]
          set basename [file tail $filename]
	  set numRecords $GLOBALtotalRecords
          set status [DCTdbReadDatabase $basename $dirname]
          DCTTotalRecords 
# puts "*****   GLOBALtotalRecords=$GLOBALtotalRecords"
	  if { $numRecords != $GLOBALtotalRecords } {
               set GLOBALfilename  $filename
	  }
	  # puts "*** Finished DCTdbReadDatabase from DCTAppendOK"
          if {"$status" == "0"} {
               global GLOBALdatabaseChanged
 # xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx was set to 1 before
               set GLOBALdatabaseChanged 0

               DCTSetScrolledRecordList
         } else {
       #       APSErrorPopUp "Cannot open/read file ($filename)."
               tk_dialog .nodbd {Error on open/read} {Cannot open/read 
                 the requested file } {} 0 OK
         }
     }
}

proc DCTSaveRecords {} {
# global GLOBALsavetype

# puts "in DCTSaveRecords: GLOBALsavetype=$GLOBALsavetype"
    set widget .saveDialog
    APSDialogBox $widget -name "Save Records" 
    grab set $widget
    focus $widget

    set w $widget.buttonRow.ok.button
    $w configure -text "Ok" -state normal \
          -command "DCTSaveRecordsOK $widget"

    $widget.buttonRow.cancel.button configure -text "Cancel"

    global GLOBALsavefile
    APSLabeledEntry .savefilename -parent .saveDialog.userFrame -label "Save Filename:" \
	  -textVariable GLOBALsavefile -width 60
}

proc DCTSaveRecordsOK {widget} {
    global GLOBALsavefile
     if { [file exists $GLOBALsavefile] } {
          APSDialogBox .existsWarning -name "File Exists Warning" \
          -contextHelp "File Exists warning."
          focus .existsWarning
          grab set .existsWarning

          set w .existsWarning.buttonRow.ok.button
          set parent .existsWarning.userFrame
          label $parent.existsLabel -text "File already exists. Overwrite?"
          pack $parent.existsLabel
          $w configure -state normal -command \
	  " destroy $widget; destroy .existsWarning;DCTSaveRecordsOKOK"

          set w .existsWarning.buttonRow.cancel.button
          $w configure -state normal -command \
	  "  destroy .existsWarning;DCTSaveRecordsOKOK"

     } else {
	 destroy $widget
         DCTSaveRecordsOKOK
     }
}

proc DCTSaveRecordsOKOK {} {
    global GLOBALsavefile
    global GLOBALsavetype

#  puts "in DCTSaveRecordsOKOK: GLOBALsavefile=$GLOBALsavefile"
#  puts "in DCTSaveRecordsOKOK: GLOBALsavetype=$GLOBALsavetype"

    set status [DCTdbWriteRecord  $GLOBALsavefile  $GLOBALsavetype ]
    if {"$status" != "0"} {
   #      APSErrorPopUp "Cannot write file ($GLOBALsavefile)."
             set errmsg [concat "Cannot write file " $GLOBALsavefile]	
	     tk_dialog .norec {Error...can't write} $errmsg {} 0 OK
    } else {
               global GLOBALdatabaseChanged
               set GLOBALdatabaseChanged 0 
    }
}

proc DCTSaveDefinitions {} {
    set widget .saveDialog
    APSDialogBox $widget -name "Save Definitionss " 
    grab set $widget
    focus $widget
    set w $widget.buttonRow.ok.button
    $w configure -text "Ok" -state normal \
          -command "DCTSaveDefinitionsOK; destroy $widget"
    $widget.buttonRow.cancel.button configure -text "Cancel"
    global GLOBALsavefile
    APSLabeledEntry .savefilename -parent .saveDialog.userFrame -label "Save Filename:" \
	  -textVariable GLOBALsavefile -width 60
}

proc DCTSaveDefinitionsOK {} {
    global GLOBALsavefile
    set status [DCTdbWriteDefinitions $GLOBALsavefile]
    if {"$status" != "0"} {
         APSErrorPopUp "Error writing definitions and records to file ($GLOBALsavefile)."
    }
}

#------------------------------------------
# Edit OK callbacks
#------------------------------------------

proc DCTCopyRecordOK {} {
     global GLOBALselectedRecordName
     global newCopyRecordName
     if {"$newCopyRecordName" == ""} {
           tk_dialog .norec2 {No record name} {No record name given ... you must
             specify a "to record" name before pressing the Copy button} {} 0 OK
	   return
     }
# puts " GLOBALselectedRecordName= $GLOBALselectedRecordName"
# puts " newCopyRecordName= $newCopyRecordName"
     set status "[DCTdbFindRecord $GLOBALselectedRecordName]"
     if {"$status" == "0"} {
# puts " find OK"
          set status "[DCTdbCopyRecord $newCopyRecordName]"
# puts " status=$status"
     }
     if {"$status" == "0"} {
          set status "[DCTdbFindRecord $newCopyRecordName]"
          set type [DCTdbGetRecordTypeName]
          set recordListString [format "%-28s %-25s" $newCopyRecordName $type]
          set w .userFrame.scrolledRecordList.listbox
          set index [$w curselection]
          $w selection clear $index
          $w insert end $recordListString
  
  #   puts " w=$w   name=$newCopyRecordName"
  #	  DCTinsertAlfa $newCopyRecordName  $w 

          $w selection set end
          DCTSetRecordSelection
          $w yview end
          global GLOBALdatabaseChanged
          set GLOBALdatabaseChanged 1
     } else {
             set errmsg [concat "Duplicate record name: " $GLOBALselectedRecordName]	
	     tk_dialog .nocopy {Error...can't copy} $errmsg {} 0 OK
     }
}

proc DCTRenameRecordOK {} {
     global GLOBALselectedRecordName
     global newRenameRecordName
     if {"$newRenameRecordName" == ""} {
           tk_dialog .norec3 {No record name} {No record name given ... you must
             specify a "to record" name before pressing the Rename button} {} 0 OK
	   return
     }
     set status "[DCTdbFindRecord $GLOBALselectedRecordName]"
     if {"$status" == "0"} {
          set status "[DCTdbRenameRecord $newRenameRecordName]"
     }
     if {"$status" == "0"} {
          set status "[DCTdbFindRecord $newRenameRecordName]"
          set type [DCTdbGetRecordTypeName]
          set recordListString [format "%-28s %-25s" $newRenameRecordName $type]
          set w .userFrame.scrolledRecordList.listbox
          set index [$w curselection]
          $w delete $index
          $w insert $index $recordListString

#     puts " w=$w   name=$newRenameRecordName"
#	  DCTinsertAlfa $newRenameRecordName  $w 

          $w selection set $index
          DCTSetRecordSelection
          set newRenameRecordName ""
          global GLOBALdatabaseChanged
          set GLOBALdatabaseChanged 1
     } else {
             set errmsg [concat "Duplicate record name" $GLOBALselectedRecordName]	
	     tk_dialog .noren {Error...rename record } $errmsg {} 0 OK
     }
}

proc DCTDeleteRecordOK {widget} {
     global GLOBALselectedRecordName
     set status "[DCTdbFindRecord $GLOBALselectedRecordName]"
     set status "[DCTdbDeleteRecord $GLOBALselectedRecordName]"
     if {"$status" == "0"} {
          set w .userFrame.scrolledRecordList.listbox
          set index [$w curselection]
          $w selection clear $index
          $w delete $index
          $w selection set $index
          DCTSetRecordSelection
          global GLOBALdatabaseChanged
          set GLOBALdatabaseChanged 1
     } else {
  #      APSErrorPopUp "Cannot delete record ($name)."
             set errmsg [concat "Cannot delete record " $GLOBALselectedRecordName]	
	     tk_dialog .nodel {Error...record name} $errmsg {} 0 OK
     }
     #destroy $widget
}

proc DCTFieldSelectCallback {} {
}

proc DCTSetScrolledFieldList {} {
     global GLOBALrelFieldName
     global GLOBALrelFieldIndex
     global GLOBALfieldListFrame
     global GLOBALverifyError
     global GLOBALfieldIndex
     global  DCT_MENUFORM
     global  DCT_INLINK
     global  DCT_OUTLINK
     global  DCT_FWDLINK

     set  GLOBALverifyError  0
     set  GLOBALfieldIndex 999

     set  GLOBALrelFieldName "none" 
     set w $GLOBALfieldListFrame
     set childlist [winfo children $w]
     foreach x $childlist {
        destroy $x
     }
     global GLOBALselectedRecordName
     if { "$GLOBALselectedRecordName" == "" } {
          DCTFieldSelectCallback 
          return 
     }

     set numFields "[DCTdbGetNFields]"
# puts "for record= $GLOBALselectedRecordName no.of fields=$numFields"

     if { "$numFields" == "0" } {
          return 
     }

     set quit  [expr $numFields - 1]

     set status [DCTdbFirstField]

# Start of loop on all fields of the given record .................................
     for {set i 0} {$i<$numFields} {incr i} {
          if {"$status" != "0" } break

          set prompt [DCTdbGetPrompt]

          set fieldName [DCTdbGetFieldName]

          set fieldType [DCTdbGetFieldType]
            global GLOBALlinkField$i
	    set linkfield  0
	    if {$fieldType == $DCT_INLINK}   {set linkfield 99}
	    if {$fieldType == $DCT_OUTLINK}  {set linkfield 99}
	    if {$fieldType == $DCT_FWDLINK}  {set linkfield 99}
          set GLOBALlinkField$i  $linkfield

          set fieldRange [DCTdbGetRange]

	  set fieldValue [DCTdbGetString]

# puts "for i=$i fieldValue=$fieldValue fieldType=$fieldType fieldName=$fieldName\n"

          global GLOBALlinkType$i
     
          if {$linkfield == 99} {
	      set  linkval  [DCTdbGetLinkType]
             # puts "in SetScrolledFL: i=$i fieldName=$fieldName  linktype=$linkval\n"

	      set  GLOBALlinkType$i  $linkval
	  }

          frame $w.item$i

          set fieldString ""
          set fieldString [format "%-22s %-5s  %-8s " $prompt $fieldName $fieldRange ]
          label $w.item$i.prompt -text $fieldString 
          pack $w.item$i.prompt -side left  -fill x

	  global GLOBALnoDevice$i
          global GLOBALfieldValue$i
	  global GLOBALfieldValueGood$i
          global GLOBALfieldValueCancel$i
          global GLOBALfieldName$i
	  global GLOBALuserValue$i
	      set GLOBALfieldName$i  $fieldName
	      set GLOBALfieldValueCancel$i $fieldValue
	      set GLOBALfieldValue$i $fieldValue
	      set GLOBALfieldValueGood$i $fieldValue
	      set GLOBALnoDevice$i 0

# For DTYP, save global value for related field name
          if {$fieldType == $DCT_MENUFORM} {
              set  GLOBALrelFieldName [DCTdbGetRelatedField]
              set  GLOBALuserValue$i  $fieldValue
#	      set  GLOBALfieldType$i  4
	      set  GLOBALfieldType$i $DCT_MENUFORM
# puts "... tcl.DCTSetScrolledFieldList: related field =$GLOBALrelFieldName"
	  }

# when current($i) field name = related field name, then save related field index
	  if {$fieldName == $GLOBALrelFieldName} {
	      set  GLOBALrelFieldIndex  $i
	      set  linkval [DCTdbGetLinkType]
	      set  GLOBALlinkType$i $linkval 
          }

# ... call to function which will put  fieldValue  to screen 

          DCTFieldListEntry $w.item$i $fieldType $i

          if {$i < $quit}  { set status [DCTdbNextField] }
          pack $w.item$i -side top -fill x -expand true
     }

# tkwait needed for proper timing of widget display ... without it code fails

     if {"$i" > "0"} {catch {  tkwait visibility $w.item[expr $i - 1] } }
     APSScrollAdjust .userFrame.fieldDialog.userFrame.scrolledFieldList -numVisible 11 

     set status [DCTdbFirstField]
     DCTFieldSelectCallback 
}
 
# xxxxxx  ... output MENUFORM ...............................
# the APS Button for MENUFORM (which calls next proc) has been commented out 8/19/96
proc DCTSetFormFieldList {i} {
     global GLOBALfieldName$i
     global GLOBALfieldValue$i
     global GLOBALfieldValueGood$i

# puts "fieldName = [subst \$GLOBALfieldName$i]"

     set fieldValue  [subst \$GLOBALfieldValue$i]
     set origValue   [subst \$GLOBALfieldValueGood$i]

# puts "current value= $fieldValue   orig. value= $origValue "

     if { [string compare $fieldValue $origValue] != 0 } {
	   # puts " current menu index not = orig. menu index"

	   DCTReviseMenuFormYN $i
     }  else {	 
	      DCTDrawMenuForm $i
     }	
}


# xxxxxx  ... select new device type (DTYP) from pull-down menu.............
proc DCTpickNewDeviceYN {i} {
     global GLOBALfieldName$i
     global GLOBALfieldValue$i
     global GLOBALfieldValueGood$i
     global GLOBALrelFieldName
     global GLOBALwhereFrom
     global GLOBALuserValue$i

 # puts " ... proc DCTpickNewDeviceYN entered ...where from=$GLOBALwhereFrom \n"

     set fieldValue  [subst \$GLOBALfieldValue$i]
     set origValue   [subst \$GLOBALfieldValueGood$i]
# puts "current device= $fieldValue   orig. value= $origValue "

     if { [string compare $fieldValue $origValue] != 0 } {
	   # puts " current device not = orig. device"

    set widget .wiper
    APSDialogBoxR $widget -name "Overwrite Device MenuForm ?" \
         -contextHelp "overwrite current menuForm" -width 50
    $widget.buttonRow.ok.button configure -text "Yes" -state normal \
         -command "DCTReviseDTYP $i; destroy $widget "
    $widget.buttonRow.cancel.button configure -text "No"  \
	 -command "DCTResetDTYP  $i; destroy $widget "
    grab set $widget
    focus $widget
    set parent $widget.userFrame
    global GLOBALselectedRecordName
    label $parent.replMFormConfirm\
     -text "  In order to carry out the requested device type change    
     (in the DTYP field), current values in the related    
     field $GLOBALrelFieldName will be overwritten (replaced / lost)

  ******   DO YOU WANT TO CONTINUE ? "
    pack  $parent.replMFormConfirm 
    }
}


# xxxxxx  ... select new device type (DTYP) from user entry field........
proc DCTpickNewDeviceYN2  {i} {
     global GLOBALfieldName$i
     global GLOBALfieldValue$i
     global GLOBALfieldValueGood$i
     global GLOBALrelFieldName
     global GLOBALwhereFrom
     global GLOBALuserValue$i

 # puts " ... proc DCTpickNewDeviceYN2  entered ...where from=$GLOBALwhereFrom \n"

     set fieldValue  [subst \$GLOBALuserValue$i]
     set origValue   [subst \$GLOBALfieldValueGood$i]
# puts "current device= $fieldValue   orig. value= $origValue "

     if { [string compare $fieldValue $origValue] != 0 } {
	   # puts " current device not = orig. device"

    if {[winfo exists .wiper2]} { return }
    set widget .wiper2
    APSDialogBoxR $widget -name "Overwrite Device MenuForm2 ?" \
         -contextHelp "overwrite current menuForm" -width 50
    $widget.buttonRow.ok.button configure -text "Yes" -state normal \
         -command "DCTReviseDTYP2 $i; destroy $widget "
    $widget.buttonRow.cancel.button configure -text "No"  \
	 -command "DCTResetDTYP2  $i; destroy $widget "
    set parent $widget.userFrame
    global GLOBALselectedRecordName
    label $parent.replMFormConfirm\
     -text "  In order to carry out the requested device type change    
     (in the DTYP field), current values in the related    
     field $GLOBALrelFieldName will be overwritten (replaced / lost)
    focus $widget
    grab set $widget

  ******   DO YOU WANT TO CONTINUE ? "
    pack  $parent.replMFormConfirm 
    }

}
proc  DCTDrawMenuForm {i} {
     global  GLOBALfieldName$i
     global  GLOBALfieldValue$i
     global  GLOBALselectedRecordName
     global  GLOBALfieldListFrame
     global  GLOBALnumFormLines

     set fname [subst \$GLOBALfieldName$i]
     set status [DCTdbFindField $fname]
# puts "after DCTdbFindField status $status"
# puts "*** calling DCTdbAllocForm from DCTDrawMenuForm"

     set numFields "[DCTdbAllocForm]"
# puts "    DrawMenuForm: i=$i name=$fname  from dbAllocForm: no.of lines=$numFields"

#----------------------------------
#   set xlink  [DCTdbGetLinkType]
#   puts "    DrawMenuForm: i=$i name=$fname  from dbGetLinkType:  linktype=$xlink"
#----------------------------------

     set GLOBALnumFormLines  $numFields

  #    puts "... tcl. DrawMenuForm: device= [subst \$GLOBALfieldValue$i]"
  #    puts "... tcl. DrawMenuForm: no. of menuform fields= $numFields"

     if { $numFields == 0 } {
	  set device [subst \$GLOBALfieldValue$i]
	  # puts "*** tcl.DrawMenuForm: zero menuform fields for device=$device" 
	  # puts "*** no associated form ... use string entry field"
	  tk_dialog .noform {No form } {No associated form ...
	  use string entry field.}  {} 0 OK
          return 
     }

     set  widget .formEntry

     APSDialogBoxR $widget  -name FormEntry
     set zzz $widget.buttonRow.ok.button
     $zzz  configure -text "Ok"  -state normal \
	  -command "DCTUpdateFromMenu $i; destroy $widget"

     set w .formEntry.userFrame

 # puts " ... calling  DCTdbGetFormPrompt  from DCTDrawMenuForm"
     set promptList [DCTdbGetFormPrompt]
# puts " promptListi=$promptList"

 # puts " ... calling  DCTdbGetFormValue   from DCTDrawMenuForm \n"
     set  fieldValueList [DCTdbGetFormValue]
     
     for {set j 0} {$j<$numFields} {incr j} {
          set promptValue  [lindex $promptList $j]
	  # puts "tcl... j=$j  Menuform prompt =$promptValue"

	  set promptString  [format "%-20s"  $promptValue ]

          global GLOBALformValue$j
          set  fvalue   [lindex $fieldValueList $j]
          set GLOBALformValue$j  $fvalue
 
          frame $w.item$j
          label $w.item$j.prompt -text $promptString
          pack  $w.item$j.prompt -side left -fill x

#   puts "in DrawMenuForm: form line$j contents =|[subst \$GLOBALformValue$j ]| "

          entry $w.item$j.entry -textvariable GLOBALformValue$j -relief sunken -width 50
	  bind  $w.item$j.entry <Key-Return> "DCTPutNVerifyForm  $j "

          pack  $w.item$j.entry -side right -fill x -expand true 
          pack  $w.item$j       -side top   -fill x -expand true

     }
}


proc  DCTUpdateFromMenu {i}  {
     global  GLOBALfieldName$i
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueGood$i
     global  GLOBALfieldValueCancel$i
     global  GLOBALselectedRecordName
     global  GLOBALnumFormLines


 # puts " *********** DCTUpdateFromMenu has been entered **********"

     set  fieldname  [subst \$GLOBALfieldName$i]

     set status [DCTdbFindField $fieldname ]
 # puts "... tcl.DCTUpdateFromMenu: after DCTdbFindField status $status"

     set numLines  $GLOBALnumFormLines
# puts " ... no.of form lines = $numLines"
     if { $numLines == 0} {
	return
     }

     for {set j 0} {$j<$numLines } {incr j} {
          set  formLine  $j
	  global  GLOBALformValue$j
	  set userValue  [subst \$GLOBALformValue$j ]
 # puts " for j=$j the value to be output by dbPutForm =$userValue"

	  set errlist [DCTdbPutFormLine $formLine  $userValue]
     }
 
# puts " ***tcl.DCTUpdateFromMenu: dbPutForm(of n lines 1 at a time) is DONE"
# all output to menuform is completed, therefore free form

   DCTdbFreeForm
# ..........................................................................

     set  fieldname  [subst \$GLOBALfieldName$i]

     set status [DCTdbFindField $fieldname ]
 # puts "... tcl.DCTUpdateFromMenu: after DCTdbFindField status $status"

      set fieldValue  [DCTdbGetString]
 # puts "... the value returned by dbGetString on $fieldname is:$fieldValue"

     global GLOBALdatabaseChanged
     set    GLOBALdatabaseChanged  1

     set GLOBALfieldValue$i       $fieldValue
     set GLOBALfieldValueGood$i   $fieldValue
     set GLOBALfieldValueCancel$i $fieldValue

     # jjjj new item saved to memory. Setup fake $GLOBALnewData for Cancel
     global  GLOBALnewData
     set     GLOBALnewData  2 
}

proc  DCTPutNVerifyForm  { i }  {

     global GLOBALformValue$i

     set formLine  $i 
     set userValue  [subst \$GLOBALformValue$i ]

      set errlist  [DCTdbPutFormLine   $formLine  $userValue ]

# DCTdbPutFormLineX tried unsuccessfully to manage dbAllocForm ...8/15/96
#     set errlist  [DCTdbPutFormLineX  $formLine  $userValue ]
#     if {errlist}
}

 proc DCTReviseMenuFormYN {i } {

    set widget .wipe

    APSDialogBoxR $widget -name "Overwrite MenuForm ?" \
         -contextHelp "overwrite current menuForm" -width 50
    $widget.buttonRow.ok.button configure -text "Yes" -state normal \
         -command "DCTReviseMF $i;destroy $widget "
    $widget.buttonRow.cancel.button configure -text "No"  \
	 -command "DCTResetMF  $i; destroy $widget "

    grab set $widget
    focus $widget

    set parent $widget.userFrame
    global GLOBALselectedRecordName
    label $parent.replMFormConfirm\
        -text "In order to execute the requested FORM command,
	the previous values on the (DTYP) menuForm 
	will be overwritten (replaced / lost ) ...

  ******   DO YOU WANT TO CONTINUE ? "
    pack  $parent.replMFormConfirm 
}


 proc DCTcvtLinkYN {i } {
    global  GLOBALfieldName$i
    set fieldName [subst \$GLOBALfieldName$i]

    set widget .wipe3

    APSDialogBoxR $widget -name "Complete Conversion Request ?" \
         -contextHelp "complete requested conversion ?" -width 50
    $widget.buttonRow.ok.button configure -text "Yes" -state normal \
         -command "DCTcvtLinkYes $i;destroy $widget "
    $widget.buttonRow.cancel.button configure -text "No"  \
	 -command "DCTcvtLinkNo  $i; destroy $widget "

    grab set $widget
    focus $widget

    set parent $widget.userFrame
    global GLOBALselectedRecordName
    label $parent.replMFormConfirm\
    -text "   In order to complete the requested coversion command,  
    the current values of the entry field & Form for $fieldName   
    will be overwritten (replaced / lost ) ...

  ******   DO YOU WANT TO CONTINUE ? "
    pack  $parent.replMFormConfirm 
}


proc DCTReviseMF {i} {
     global  GLOBALfieldValue$i
     global GLOBALfieldValueCancel$i
     global GLOBALfieldValueGood$i
     global GLOBALfieldName$i

     set fieldValue  [subst \$GLOBALfieldValue$i]

     DCTdbFindField [subst \$GLOBALfieldName$i ]
     set old1  [DCTdbGetString ]
     # puts "... in ReviseMF:  current device b4 putString= $old1"

# load most recent user supplied device type to memory
     DCTdbPutString  $fieldValue

     set new1  [DCTdbGetString ]
     # puts "... in ReviseMF: current device after  putString= $new1"

     set GLOBALfieldValueCancel$i [subst \$GLOBALfieldValue$i] 
     set GLOBALfieldValueGood$i [subst \$GLOBALfieldValue$i] 

# generate new menuform for new device type
     DCTDrawMenuForm $i

}

proc DCTResetMF {i} {
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueCancel$i

     set GLOBALfieldValue$i  [subst \$GLOBALfieldValueCancel$i]
}

proc DCTcvtLinkNo {i} {
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueCancel$i

     return 
}

# for DTYP related field logic .........................................
proc DCTReviseDTYP {i} {
     global  GLOBALfieldValue$i
     global GLOBALfieldValueCancel$i
     global GLOBALfieldValueGood$i
     global GLOBALfieldName$i
     global GLOBALrelFieldName
     global GLOBALrelFieldIndex
     global GLOBALuserValue$i
     global GLOBALwhereFrom
# puts "in DCTReviseDTYP:  whereFrom=$GLOBALwhereFrom"
     if {$GLOBALwhereFrom == 1} {
          set fieldValue  [subst \$GLOBALfieldValue$i]
     }  else {
          set fieldValue  [subst \$GLOBALuserValue$i]
     }
 # puts "... fieldValue start= $fieldValue"
# position at current #i field (DTYP) ..............
     DCTdbFindField [subst \$GLOBALfieldName$i ]
     set old1  [DCTdbGetString ]
      # puts "... in ReviseDTYP:  current device b4 putString= $old1"

# load most recent user supplied device type (DTYP) to memory
     DCTdbPutString  $fieldValue

     set new1  [DCTdbGetString ]
      # puts "... in ReviseDTYP: current device after  putString= $new1"
# the  $i field position  is the DTYP field itself
     set GLOBALfieldValueCancel$i $fieldValue 
     set GLOBALfieldValueGood$i   $fieldValue 

     if {$GLOBALwhereFrom == 1} {
          set GLOBALuserValue$i        $fieldValue  
     }  else {
          set GLOBALfieldValue$i        $fieldValue  
     }
# puts "...at end: fieldValue=$fieldValue"

     global  GLOBALdatabaseChanged
     set     GLOBALdatabaseChanged  1

     # jjjj new item saved to memory. Setup fake $GLOBALnewData for Cancel
     global  GLOBALnewData
     set     GLOBALnewData  2 

# prepare to update related field (the $j field position) contents
     set  j  $GLOBALrelFieldIndex
     DCTupdateRFValue $j
}

# for DTYP related field logic (where user types in device info)..................
proc DCTReviseDTYP2 {i} {
     global  GLOBALfieldValue$i
     global GLOBALfieldValueCancel$i
     global GLOBALfieldValueGood$i
     global GLOBALfieldName$i
     global GLOBALrelFieldName
     global GLOBALrelFieldIndex
     global GLOBALuserValue$i
     global GLOBALwhereFrom
# puts "in DCTReviseDTYP2:  whereFrom=$GLOBALwhereFrom"

          set fieldValue  [subst \$GLOBALuserValue$i]

 # puts "... fieldValue start= $fieldValue"
# position at current #i field (DTYP) ..............
     DCTdbFindField [subst \$GLOBALfieldName$i ]
     set old1  [DCTdbGetString ]
      # puts "... in ReviseDTYP2:  current device b4 putString= $old1"

# load most recent user supplied device type (DTYP) to memory
     DCTdbPutString  $fieldValue

     set new1  [DCTdbGetString ]
      # puts "... in ReviseDTYP2: current device after  putString= $new1"
# the  $i field position  is the DTYP field itself
     set GLOBALfieldValueCancel$i $fieldValue 
     set GLOBALfieldValueGood$i   $fieldValue 

     set GLOBALfieldValue$i        $fieldValue  
# puts "...at end: fieldValue=$fieldValue"

     global  GLOBALdatabaseChanged
     set     GLOBALdatabaseChanged  1

     # jjjj new item saved to memory. Setup fake $GLOBALnewData for Cancel
     global  GLOBALnewData
     set     GLOBALnewData  2 

# prepare to update related field (the $j field position) contents
     set  j  $GLOBALrelFieldIndex
     DCTupdateRFValue $j
     global GLOBALnogo
     set GLOBALnogo  2
}

proc DCTResetDTYP {i} {
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueCancel$i
     global GLOBALuserValue$i
     set GLOBALfieldValue$i  [subst \$GLOBALfieldValueCancel$i]
     set GLOBALuserValue$i  [subst \$GLOBALfieldValueCancel$i]
}

proc DCTResetDTYP2 {i} {
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueCancel$i
     global GLOBALuserValue$i
     global GLOBALnogo
 # puts "ResetDTYP2 entered..."
     set GLOBALuserValue$i  [subst \$GLOBALfieldValue$i]
     set GLOBALnogo  1
}

proc DCTupdateRFValue {i}  {
     global GLOBALfieldValue$i
     global GLOBALfieldValueCancel$i
     global GLOBALfieldValueGood$i
     global GLOBALrelFieldName
     global GLOBALrelFieldIndex

# position at related field .......................
     DCTdbFindField  $GLOBALrelFieldName

# obtain newly loaded value of related field from memory ....
     set  newContents  [DCTdbGetString]
     set GLOBALfieldValue$i       $newContents
     set GLOBALfieldValueGood$i   $newContents
     set GLOBALfieldValueCancel$i $newContents

     global  GLOBALdatabaseChanged
     set     GLOBALdatabaseChanged  1
     # jjjj new item saved to memory. Setup fake $GLOBALnewData for Cancel
     global  GLOBALnewData
     set     GLOBALnewData  2 
}

proc DCTEditFieldsCancel { } {
# this routine sets as many fields as possible back to their orig. values
     global  GLOBALdatabaseChanged
     global  GLOBALnewData

     set numFields "[DCTdbGetNFields]"
     set status [DCTdbFirstField]
     for {set i 0} {$i<$numFields} {incr i} {
          if {"$status" != "0" } break
          global GLOBALfieldName$i
          global GLOBALfieldValue$i
          global GLOBALfieldValueCancel$i
          set a [subst \$GLOBALfieldValue$i]
          set b [subst \$GLOBALfieldValueCancel$i]
          if { [string compare $a $b] != 0 } {
	         set GLOBALfieldValue$i [subst \$GLOBALfieldValueCancel$i] 
          }
          set status [DCTdbNextField]
     }

    # GLOBALnewData= 2, some new data already put in memory, cant cancel it
     if {$GLOBALnewData > 0 } {
	  set GLOBALdatabaseChanged  1
     } else { 
	  set GLOBALdatabaseChanged  0 
     }
     set GLOBALnewData  0
}

proc DCTEditFieldsOK { } {
   # puts "...DCTEditFieldsOK: entered ... 2 "

   focus .userFrame.fieldDialog.buttonRow.ok.button  

     global  GLOBALverifyError
     global  DCT_MENUFORM

     # puts "in EditFieldsOK  ... GLOBALverifyError=$GLOBALverifyError"

     set numFields "[DCTdbGetNFields]"
     set status [DCTdbFirstField]

     for {set i 0} {$i<$numFields} {incr i} {
          if {"$status" != "0" } break
          global GLOBALfieldName$i
	  set fieldname [subst \$GLOBALfieldName$i]

          global GLOBALfieldValue$i
          global GLOBALfieldValueCancel$i
          global GLOBALfieldType$i
	  global GLOBALuserValue$i
          global GLOBALwhereFrom
	  global GLOBALnogo

	  set GLOBALnogo 0

          if { [subst \$GLOBALfieldType$i] == $DCT_MENUFORM}  {
	      set teston  [subst \$GLOBALuserValue$i] 
          }  else {
	      set teston  [subst \$GLOBALfieldValue$i]
          } 

          set b [subst \$GLOBALfieldValueCancel$i]
          if { [string compare $teston $b] != 0 } {
	       # puts "EditFieldsOk: calls VerifyField for: $fieldname"
               DCTVerifyField $i
	       if {$GLOBALverifyError != 0} {
		   # puts "...EditFieldsOK: VerifyField error on: $fieldname"
		   return
               } else {
                       if { [subst \$GLOBALfieldType$i] == $DCT_MENUFORM}  {
		       # puts "pickND YN2 called from EditF OK nogo=$GLOBALnogo"
	     	           DCTpickNewDeviceYN2 $i
			   # if DCTpickNewDeviceYN2 says NO, then GLOBALnogo=1 
			   tkwait variable GLOBALnogo
		       # puts "pickND YN2 returns to EditF OK nogo=$GLOBALnogo"
			   if {$GLOBALnogo == 2} { set GLOBALnogo 0}
                       }
                   if {$GLOBALnogo == 0} {
		       # puts "putString i=$i** nogo=$GLOBALnogo teston=$teston**"

	               DCTdbPutString  $teston
	               set GLOBALfieldValueCancel$i   $teston 
	               # puts " cancel=[subst \$GLOBALfieldValueCancel$i] " 
                   }
               }
          }
          set status [DCTdbNextField]
     }

     # jjjj reset to 0 after PutString (save) to memory of new user supplied input
     global  GLOBALnewData
     set     GLOBALnewData 0
     global GLOBALdatabaseChanged
     set GLOBALdatabaseChanged 1
}


# Verify acceptability of user supplied  input value ....................
proc DCTVerifyField { i } {
     global GLOBALfieldName$i
     global GLOBALfieldValue$i
     global GLOBALfieldValueGood$i
     global GLOBALfieldValueCancel$i
     global GLOBALnoDevice$i
     global GLOBALfieldIndex
     global GLOBALwidget$i
     global GLOBALverifyError
     global GLOBALnewData
     global GLOBALdatabaseChanged
     global GLOBALuserprob
     global GLOBALfocus$i
     global GLOBALfieldType$i
     global GLOBALwhereFrom
     global GLOBALuserValue$i
     global GLOBALlinkField$i
     global DCT_MENUFORM

 #  puts "DCTVerifyField called ================================"
     if {$GLOBALverifyError != 0} {
       # puts "DCTVerifyFieldG: called but is quitting due to already finding error"
       return
     }
  
  #   set GLOBALfieldValue$i  $inputValue

     set fieldType [subst \$GLOBALfieldType$i]

  # for DTYP field (fieldType=4) ... for all other fields (fieldType not=4) ..... 
     if {$fieldType == $DCT_MENUFORM} {
          set inputValue  [string trim [subst \$GLOBALuserValue$i]]
     }  else {
          set inputValue  [string trim [subst \$GLOBALfieldValue$i]]
     }

  # for links (fieldType is > 4, i.e., 5,6,7 .... No verify is done ............
     set linkOK  [subst \$GLOBALlinkField$i]
     if { $linkOK == 99} {
	# set  GLOBALfieldValueCancel$i  $inputValue
	set  GLOBALfieldValueGood$i    $inputValue
        set GLOBALnewData 1 
	set GLOBALdatabaseChanged 1 
	return
     }

  # remaining logic is for fieldType 0,1,2,3,4  (where 4 is a DTYP field) .......
     set cancelValue  [string trim [subst \$GLOBALfieldValueCancel$i]]
     set GLOBALfieldValueCancel$i  $cancelValue

     if { [string compare $inputValue $cancelValue] == 0 } {
	 # puts "in DCTVerifyField: no verify needed since values unchanged"
	 return
     }

     set  GLOBALverifyError 0
     set  fieldname  [subst \$GLOBALfieldName$i]

     set status [DCTdbFindField $fieldname ]
     if {$status != 0 } { puts "dbFindField failed" }

     set device  [subst \$GLOBALnoDevice$i]
     if  { $device == 999 }  {
	 set status "is not allowable input value "
     }  else {
              # .......................................................................
              # puts "VerifyField: Check on field=$i name=$fieldname value=$inputValue"
              set status [DCTdbVerify $inputValue ]
     }

       # when status is non-blank, then error has been found ..............
     if {$status != ""}  {
           set GLOBALverifyError 99
           # reset proper field to last good value depending on DTYP or not ... 
           if {$fieldType == $DCT_MENUFORM} {
               set GLOBALuserValue$i [subst \$GLOBALfieldValueGood$i] 
           }  else {
               set GLOBALfieldValue$i [subst \$GLOBALfieldValueGood$i] 
           }

           set GLOBALnewData 0 
           set GLOBALdatabaseChanged 0

	   set widget  .userprob
	   set GLOBALuserprob   $widget

	   set errmsg [concat $inputValue " "  $status " in field "  $fieldname]
	   # output error message to screen ....................
           tk_dialog $widget {User input error} $errmsg  {} 0 OK 

 	   focus [subst \$GLOBALfocus$i]
          set GLOBALverifyError 0

     } else {
          if {$fieldType == $DCT_MENUFORM} { return }

          set GLOBALfieldValueGood$i    $inputValue 
          set GLOBALnewData 1 
	  set GLOBALdatabaseChanged 1 
     }
      # puts " ... AFTER tkdialog"
}

proc DCTsetFieldNum {i} {
  global  GLOBALfieldIndex

  set  GLOBALfieldIndex  $i
  # puts "...DCTsetFieldNum: GLOBALfieldIndex from FocusIn = $GLOBALfieldIndex"

}

proc DCTuserValue {i} {
  global  GLOBALuserValue$i
  global  GLOBALverifyError
  global  GLOBALfieldValue$i
  global  GLOBALwhereFrom


# puts "*** DCTuserValue entered : i=$i   userValue=[subst \$GLOBALuserValue$i]"
               DCTVerifyField  $i
	       if {$GLOBALverifyError != 0} {
		    # puts "...DCTuserValue: VerifyField error on: $i"
		   return
               } else {
     #              set  GLOBALwhereFrom 2
		   DCTpickNewDeviceYN2  $i
         #          set  GLOBALwhereFrom 1 
               }
}

proc DCTFieldListEntry {parent type i} {
    global  GLOBALfieldValue$i
    global  GLOBALwidget$i
    global  GLOBALfieldIndex
    global  GLOBALfocus$i
    global  GLOBALfieldType$i
    global  GLOBALuserValue$i
    global  GLOBALwhereFrom 

   global  DCT_STRING
   global  DCT_INTEGER
   global  DCT_REAL
   global  DCT_MENU
   global  DCT_MENUFORM
   global  DCT_INLINK
   global  DCT_OUTLINK
   global  DCT_FWDLINK

    set  GLOBALwhereFrom    1
    set  GLOBALfieldType$i   $type

#    switch $type {
#        0 { # DCT_STRING
          if { $type == $DCT_STRING} {	
            entry $parent.entry -relief sunken -textvariable GLOBALfieldValue$i  \
	    -width 50 
        #    bind $parent.entry <Tab>        "DCTVerifyField $i"
        #    bind $parent.entry <FocusIn>    "DCTsetFieldNum $i"
	    bind $parent.entry <Key-Return> "DCTVerifyField $i"
	    bind $parent.entry <FocusOut>   "DCTVerifyField $i"
            pack $parent.entry -fill x -expand 1 

            set  GLOBALfocus$i $parent.entry 
          } 

#        1 { # DCT_INTEGER
          if { $type == $DCT_INTEGER} {	
            entry $parent.entry -relief sunken -textvariable GLOBALfieldValue$i
         #   verify the contents of input field at position $i after a Return

	    bind $parent.entry <KeyRelease>        "DCTVerifyField $i"
	    bind $parent.entry <Key-Return> "DCTVerifyField $i"
	    bind $parent.entry <FocusOut>   "DCTVerifyField $i"
            pack $parent.entry -fill x -expand 1 

            set  GLOBALfocus$i $parent.entry 
          }

#        2 { # DCT_REAL
          if { $type == $DCT_REAL} {	
            entry $parent.entry -relief sunken  -textvariable GLOBALfieldValue$i
 
	    bind $parent.entry <KeyRelease>        "DCTVerifyField $i"
	    bind $parent.entry <Key-Return> "DCTVerifyField $i"
	    bind $parent.entry <FocusOut>   "DCTVerifyField $i"
            pack $parent.entry -fill x -expand 1 

            set  GLOBALfocus$i $parent.entry 
          }

#        3 { # DCT_MENU
          if { $type == $DCT_MENU} {	
            set choices [DCTdbGetMenuChoices]
#	    puts "MENU choices=$choices=======\n"
#            global GLOBALmenuChoice$i
#            set GLOBALchoice$type [DCTdbGetMenuIndex]

           # next 2 stmts are reqd to allow  subst on choices to convert it from
	   # a single string consisting of concatenated choices  to a list of
	   # n  separate choices to be displayed in a column . 
            set zzz "tk_optionMenu $parent.entry GLOBALfieldValue$i [subst $choices] "
            set menu [eval $zzz]
            $parent.entry configure -pady 0

#	    bind $parent.entry <Configure>     "DCTpickNewMenuItem $i"
            pack $parent.entry -side left

            entry $parent.entry2 -relief sunken -textvariable GLOBALfieldValue$i -width 10
	    bind $parent.entry2 <Key-Return> "DCTVerifyField $i"
	    bind $parent.entry2 <FocusOut>   "DCTVerifyField $i"
            pack $parent.entry2 -fill x -expand 1  -side right 

            set  GLOBALfocus$i $parent.entry2
          }

#        4 { # DCT_MENUFORM
          if { $type == $DCT_MENUFORM} {	
            set choices [DCTdbGetMenuChoices]
#	    puts " ****** MENUFORM choices=$choices=======\n"

	    if {$choices == 0}  {
		entry $parent.entry -relief sunken -textvar GLOBALfieldValue$i -width 40 
		pack  $parent.entry -fill x -expand true
		global GLOBALnoDevice$i
		set  GLOBALnoDevice$i  999
		bind $parent.entry <Key-Return> "DCTVerifyField $i"
		bind $parent.entry <Tab>        "DCTVerifyField $i"
		return
	    }

            set zzz "tk_optionMenu $parent.entry GLOBALfieldValue$i [subst $choices] "
            set menu [eval $zzz]

            $parent.entry configure -pady 0 

        # None of the next 5 stmts will bind to DCTpickNewDeviceYN
        #   $parent.entry configure   -command "DCTpickNewDeviceYN $i"
        #   bind $parent.entry <ButtonRelease> "DCTpickNewDeviceYN $i"
	#   bind $parent.entry <Button-1>      "DCTpickNewDeviceYN $i"
        #   bind $parent.entry <Motion>        "DCTpickNewDeviceYN $i"
        #   bind $parent.entry <Leave>         "DCTpickNewDeviceYN $i"

	    bind $parent.entry <Configure>     "DCTpickNewDeviceYN $i"
            pack $parent.entry -side left 

            entry $parent.entry2 -relief sunken -textvariable GLOBALuserValue$i -width 10
	    bind $parent.entry2 <Key-Return>      "DCTuserValue $i"
	    bind $parent.entry2 <FocusOut>        "DCTuserValue $i"
            pack $parent.entry2 -fill x -expand 1  -side right 

            set  GLOBALfocus$i $parent.entry2
           }


# ALL varieties of  link types are processed here .............................
#        5 { # DCT_INLINK ,  DCT_OUTLINK, DCT_FWDLINK
	  global GLOBALlinkField$i
	  set  ftype  [subst \$GLOBALlinkField$i]
          if {$ftype == 99 } {	
	    global  GLOBALlinkType$i
	    global  GLOBALlinkTitle$i
	    global  GLOBALfieldName$i
	    global  GLOBALrelFieldName
            global  DCT_LINK_CONSTANT
            global  DCT_LINK_FORM
            global  DCT_LINK_PV

            entry $parent.entry -relief sunken -textvariable GLOBALfieldValue$i -width 10
	
	    bind $parent.entry <KeyRelease> "DCTVerifyField $i"
	    bind $parent.entry <Key-Return> "DCTVerifyField $i"
	    bind $parent.entry <FocusOut>   "DCTVerifyField $i"

            pack $parent.entry -fill x -expand 1  -side left

	    set field     [subst \$GLOBALfieldName$i]
	    set linkval  [subst \$GLOBALlinkType$i ]

#  puts "... in tcl.DCTFieldListEntry: linkType=$linkval for field=$field "

# check here to see if current link field is for hdw (ie, linktype=1) ...
	    
            if { $linkval != $DCT_LINK_FORM }  {

# puts "... in tcl.DCTFieldListEntry: GLOBALrelFieldName=$GLOBALrelFieldName "
# puts "... in tcl.DCTFieldListEntry: GLOBALfieldName$i=$field \n"

                 if {$linkval == $DCT_LINK_CONSTANT}  {
                      set GLOBALlinkTitle$i  "to PV"
                 }
	         if {$linkval == $DCT_LINK_PV}  {
		     set GLOBALlinkTitle$i  "to const."
                 }

                 button $parent.button \
	     	       -textvariable  GLOBALlinkTitle$i \
		       -command "DCTcvtLinkYN  $i"
                 pack $parent.button  -side left
            }

             APSButton .button4 -parent $parent -text "FORM" \
             -command "DCTDrawMenuForm $i"
             pack $parent.button4  

            set  GLOBALfocus$i $parent.entry
          }

#    }
#puts "******END i=$i **************"
}


proc DCTpickNewMenuItem {i} {
     global  GLOBALfieldValue$i
     global  GLOBALfieldValueGood$i
     global  GLOBALnewData 
     global  GLOBALdatabaseChanged
     global  GLOBALuserValue$i
#
#puts "...DCTpickNewMenuItem: New pull-down menu choice=[subst \$GLOBALfieldValue$i]"
     set fieldValue  [subst \$GLOBALfieldValue$i]
     set origValue   [subst \$GLOBALfieldValueGood$i]

     if { [string compare $fieldValue $origValue] != 0 } {
	   # puts " current menu index not = orig. menu index"
         # jjjj user input change is OK
           set GLOBALnewData 1 
           set GLOBALdatabaseChanged  1
	   set GLOBALfieldValueGood$i  $fieldValue 
     } else {
           set GLOBALnewData 0 
           set GLOBALdatabaseChanged 0 
     }

}


proc DCTcvtLinkYes {i} {
     global  GLOBALlinkType$i
     global  GLOBALlinkTitle$i
     global  GLOBALfieldValue$i
     global  GLOBALfieldName$i
     global  GLOBALfieldValueGood$i
     global  GLOBALfieldValueCancel$i
     global  DCT_LINK_PV
     global  DCT_LINK_CONSTANT

     set name [subst \$GLOBALfieldName$i ]
     set status [DCTdbFindField $name ]
     set linkval [subst \$GLOBALlinkType$i]

     if { $linkval == $DCT_LINK_CONSTANT } { # CONSTANT
          set status [DCTdbCvtLinkToPvlink]
          set  GLOBALlinkTitle$i  "to const."
          set  GLOBALlinkType$i $DCT_LINK_PV 
     } else {
	  if { $linkval == $DCT_LINK_PV } { # PV
               set status [DCTdbCvtLinkToConstant]
               set  GLOBALlinkTitle$i  "to PV"
               set  GLOBALlinkType$i  $DCT_LINK_CONSTANT 
	  }
     }

     set fieldValue [DCTdbGetString]
     
     set GLOBALfieldValueCancel$i $fieldValue
     set GLOBALfieldValue$i $fieldValue
     set GLOBALfieldValueGood$i $fieldValue

     global  GLOBALdatabaseChanged
     set     GLOBALdatabaseChanged  1
     
     # jjjj new item saved to memory. Setup fake $GLOBALnewData for Cancel
     global  GLOBALnewData
     set     GLOBALnewData  2 
}

proc DCTSetScrolledRecordList {} {
     set w .userFrame.scrolledRecordList.listbox
     while {"[$w get 0]" != ""} { $w delete 0 }
     set numRecdes "[DCTdbGetNRecordTypes]"
     set quit  [expr $numRecdes -1]
     if { "$numRecdes" > "0" } {
          set status [DCTdbFirstRecordType]
     }
     global GLOBALselectedRecordName
     set GLOBALselectedRecordName ""
     for {set j 0} {$j<$numRecdes} {incr j} {
          set type [DCTdbGetRecordTypeName]
          set numRecords "[DCTdbGetNRecords]"
	  set quit2 [expr $numRecords -1]
          if { "$numRecords" > "0" } {
               set status [DCTdbFirstRecord]
          }
          for {set i 0} {$i<$numRecords} {incr i} {
               set name [DCTdbGetRecordName]
               if { "$GLOBALselectedRecordName" == "" } {
                    set GLOBALselectedRecordName [string trimright $name ]
               }
               set recordString ""
               set recordString [format "%-28s %-25s" $name $type]
               $w insert end $recordString
               if {$i < $quit2 }  { set status [DCTdbNextRecord]
	       }
          }
	  if {$j < $quit}  {
               set status [DCTdbNextRecordType]
          }	       
     }
     $w configure -selectmode single
     if { "$GLOBALselectedRecordName" != "" } {
          set status [DCTdbFindRecord $GLOBALselectedRecordName]
          $w selection set 0
     }
     DCTSetRecordSelection 
}

proc DCTRecordTypeOptionMenu {parent} {
     set w $parent.optionMenu
     set numRecdes [DCTdbGetNRecordTypes]
     set string ""
     if { "$numRecdes" > "0" } {
         set status [DCTdbFirstRecordType]
         for {set i 0} {$i<$numRecdes} {incr i} {
              set type [DCTdbGetRecordTypeName]
              set string [concat $string "$type" ]
              set status [DCTdbNextRecordType]
         }
	 # setup list of available record types .....
	 tk_optionMenu $w ADDRecordType "$string"
         pack $w  
     } else {
           tk_dialog .nodbd {Error ... no .dbd} {You must first open a database 
       defintion file (.dbd) before manipulating record instances} {} 0 OK
     }
}

proc DCTSetScrolledRecordTypeList {parent} {
     set w $parent.listbox
     while {"[$w get 0]" != ""} { $w delete 0 }
     set numRecdes [DCTdbGetNRecordTypes]

     set quit [ expr $numRecdes - 1 ]

     if { "$numRecdes" > "0" } {
         set status [DCTdbFirstRecordType]

         for {set i 0} {$i<$numRecdes} {incr i} {
              set type [DCTdbGetRecordTypeName]
              $w insert end $type
              if {$i < $quit}  { set status [DCTdbNextRecordType] }

         }
         $w configure -selectmode single
         $w selection set 0
         DCTSetRecordSelection 
     } else {
           tk_dialog .nodbd {Error ... no .dbd} {You must first open a database 
       defintion file (.dbd) before manipulating record instances} {} 0 OK

     }
}

proc DCTPrefixEntryOK {widget} {
     global GLOBALrecordNamePrefix
     global ADDRecordName
     set GLOBALrecordNamePrefix [$widget.userFrame.prefix.entry get]
     set ADDRecordName [string trimright $GLOBALrecordNamePrefix]
}

proc DCTCreateRecordOK {} {
     global ADDRecordName
     global ADDRecordType
     if {"$ADDRecordName" == ""} {
           tk_dialog .norec {No record name} {No record name given ... you must
             specify a record name before pressing the Create button} {} 0 OK
	   return
     }
    
     set status [DCTdbFindRecordType $ADDRecordType]
     if {"$status" == "0"} {
          set status [DCTdbCreateRecord $ADDRecordName]
# puts "status=$status"
# puts "ADDRecordName=$ADDRecordName"
     }
     if {"$status" == "0"} {
          set status [DCTdbFindRecord $ADDRecordName]
          set name [DCTdbGetRecordName]
          set type [DCTdbGetRecordTypeName]
          set recordListString [format "%-28s %-25s" $name $type]
          set w .userFrame.scrolledRecordList.listbox
          set index [$w curselection]
          if { "$index" != "" } {
               $w selection clear $index
          }
          $w insert end $recordListString

#     puts " w=$w   name=$recordListString"
#	  DCTinsertAlfa $recordListString  $w 

          $w selection set end
          DCTSetRecordSelection 
          $w yview end
          global GLOBALrecordNamePrefix
          set ADDRecordName  [string trimright $GLOBALrecordNamePrefix]
          # let recordType be default for next record
          #set ADDRecordType ""
          global GLOBALdatabaseChanged
          set GLOBALdatabaseChanged 1
     } else {
           tk_dialog .badname {Record name error} {  Invalid or duplicate
             record name ... Re-enter   } {} 0 OK
     }
}

proc DCTraise {popup} {
    if {[winfo exists $popup]} {
         if {[winfo ismapped $popup ]} {
	      raise $popup
	      focus $popup
	 }
    }
}

proc DCTEditCreateRecord {widget} {
    set status "[DCTdbFirstRecordType]"
    if {"$status" != "0"} {
           tk_dialog .nodbd {Error ... no .dbd} {You must first open a database 
       defintion file (.dbd) before manipulating record instances} {} 0 OK
	   return
    }
    APSDialogBox $widget -name "Create Record" -contextHelp "Context Help" \
     -width 50
    $widget.buttonRow.ok.button configure -text "Create" -state normal \
      -command { DCTCreateRecordOK }
    $widget.buttonRow.cancel.button configure -text "Dismiss"

    global ADDRecordName
    global ADDRecordType
    global GLOBALrecordNamePrefix
    set ADDRecordName [string trimright $GLOBALrecordNamePrefix]
    set type [DCTdbGetRecordTypeName]
    set ADDRecordType [string trimright $type]
    set parent $widget.userFrame
    
    frame $parent.frame0
    label $parent.frame0.label -text  "Record name:"
    pack $parent.frame0.label -side left
    entry $parent.frame0.recordNameEntry -width 29 -relief sunken \
        -textvariable ADDRecordName
    pack $parent.frame0.recordNameEntry -fill x
    pack $parent.frame0 -side top -fill x 
    frame $parent.frame
    label $parent.frame.label -text  "Record type:"
    pack $parent.frame.label -side left
    label $parent.frame.recordTypeEntry  -relief sunken \
        -textvariable ADDRecordType
    pack $parent.frame.recordTypeEntry -fill x
    pack $parent.frame -fill x 
     APSScrolledList .scrolledRecordTypeList -parent $parent \
          -callback "DCTRecordTypeSelectCallback" \
          -height 5 \
          -name RecordType
     DCTSetScrolledRecordTypeList $parent.scrolledRecordTypeList 
    pack $parent.scrolledRecordTypeList 
#    DCTRecordTypeOptionMenu $parent
}

proc DCTRecordTypeSelectCallback {listboxItem doubleClick} {
     global ADDRecordType
     set ADDRecordType [string trimright $listboxItem]
}

proc DCTRecordSelectCallback {listboxItem doubleClick} {
 global  GLOBALnewData
 global  GLOBALfieldIndex

 # puts "... RecordSelectCallback: listboxItem=$listboxItem "
  if {$GLOBALnewData == 1 } {
    set widget .wipe4
    APSDialogBoxR $widget -name "Save display field values ?" \
         -contextHelp "do you want to save display values" -width 50
    $widget.buttonRow.ok.button configure -text "Yes" -state normal \
         -command "DCTEditFieldsOK; destroy $widget; DCTSetRecordSelection "
    $widget.buttonRow.cancel.button configure -text "No"  \
	 -command " destroy $widget; DCTSetRecordSelection "
    grab set $widget
    focus $widget
    set parent $widget.userFrame
    label $parent.replMFormConfirm\
        -text "You have asked for another record to 
	be displayed without saving  your newly 
        entered data of the current record.	

      ****** Do you want to  SAVE  the current Display field values ? "
    pack  $parent.replMFormConfirm 
  } else {
      DCTSetRecordSelection
  }      
      
}

proc DCTSetRecordSelection {} {
     global GLOBALselectedRecordName
     global newRenameRecordName
     global newCopyRecordName
     global GLOBALnewData

     set    GLOBALnewData  0
#     puts "... SetRecordSelection: entered ......."
     set w .userFrame.scrolledRecordList.listbox
     set index "[$w curselection]"
     if {"$index" != ""} {
         set text "[$w get $index]"
         scan $text "%28s" name
         set GLOBALselectedRecordName [string trimright $name]
         set status [DCTdbFindRecord $GLOBALselectedRecordName]
         if {"$status" != "0"} {
             set errmsg [concat "Cannot find record " $GLOBALselectedRecordName]	
	     tk_dialog .norec {Error...record name} $errmsg {} 0 OK
         }

     } else {
         set GLOBALselectedRecordName  ""
     }
     if { "$GLOBALselectedRecordName" != "" } {
	  set newRenameRecordName   $GLOBALselectedRecordName
	  set newCopyRecordName     $GLOBALselectedRecordName
          set type [DCTdbGetRecordTypeName]
          set type [string trimright $type ]
     } else {
          set type ""
     }
     .userFrame.labelNAME configure -text "Record name:    $GLOBALselectedRecordName"
     .userFrame.labelTYPE configure -text "Record Type:    $type"
     DCTSetScrolledFieldList
}

proc DCTunresLinks {} {

     global GLOBALselectedRecordName
     global DCT_LINK_PV

     set  widget .unLinks

     APSScrolledList  $widget  -name "Listing of Unresolved Links" -height 10 \
	 -packOption "-side left -fill x -expand true" -callback "destroy $widget"

#     set w $widget.button.Row.ok.button
#     $w configure -text "Dimiss" -state normal

     set w $widget.listbox
     $w  configure  -width 70

     set  reread  0
     set numRecdes "[DCTdbGetNRecordTypes]"

     if { "$numRecdes" > "0" } {
          set status [DCTdbFirstRecordType]
	  set quit1  [expr $numRecdes - 1 ]
	  set num  -1
	  set name "Record name ..............."
	  set fieldName "Field"
	  set contents  "Unresolved link"
  set outline [format "%-29s %-6s  %-29s" $name $fieldName $contents]
	#	  puts "  $outline"
		  set  num  [expr $num + 1]
	#	  puts "num= $num"
		  $w insert end $outline
     }

    
#     puts "\n***   Unresolved Links Listing  ...........................................  "
#     puts "  Record name                  Field   Contents"
     for {set j 0} {$j<$numRecdes} {incr j} {
          set type [DCTdbGetRecordTypeName]
          set numRecords "[DCTdbGetNRecords]"
          if { "$numRecords" > "0" } {
               set status [DCTdbFirstRecord]
	       set quit2  [expr $numRecords - 1 ]
          }
          for {set i 0} {$i<$numRecords} {incr i} {
               set name [DCTdbGetRecordName]
	       set numLinks [DCTdbGetNLinks]
               # puts "i=$i  record is:|$name| which has $numLinks links"
	       for {set k 0} {$k<$numLinks} {incr k} {
		  set linkField$k [DCTdbGetLinkField  $k]
		  set linkType$k  [DCTdbGetLinkType]
		  set  link  [subst \$linkType$k]
		  set fieldName$k [DCTdbGetFieldName]
		  set contents$k  [DCTdbGetString]
          # puts "... contents=|[subst \$contents$k]| linktype=$link"
	       }	  
	       for {set m 0} {$m<$numLinks} {incr m} {
		  set  link  [subst \$linkType$m]
                  if {$link == $DCT_LINK_PV } {
		    set xxx  [subst \$contents$m]
        #	      puts " test in unResLinks: ... DCT_LINK_PV=$DCT_LINK_PV"
		    set period "."
		    set last1 [string first $period $xxx ]
          # puts "... period (.) at position=$last1"
		    if {$last1 > 0}  {
  		        set last1 [expr $last1 - 1]
		        set recname  [string range $xxx 0  $last1]
                    }  else  {
			set blank  " "
			set last1 [string first  $blank  $xxx]
		        if {$last1 > 0}  {
  		              set last1 [expr $last1 - 1]
		              set recname  [string range $xxx 0  $last1]
                        }  else  {
		              set recname  $xxx
                        }
                    }

          # puts " ***call dbFindRecord for:|$recname|"
                    set status [DCTdbFindRecord $recname ]
		    set reread  1
		    if {$status != 0}  {
		      puts "unresolved link has been found ... "
		      set field [subst \$fieldName$m]
		      set pvlink  [subst \$contents$m ]
                      set outline [format "%-29s %-6s  %-29s" $name $field $pvlink]
#		      puts "  $outline"
		      set  num  [expr $num + 1]
#		      puts "num= $num"
		      $w insert end $outline
		    } 
		  }
               }
	       if {$reread == 1}   {
		    set status [DCTdbFindRecord $name ]
		    set reread  0 
               }		    

               if {$i < $quit2 }  {
	            set status [DCTdbNextRecord]
               }

          }
	  if {$j < $quit1 }  {
               set status [DCTdbNextRecordType]
	  }
     }
# go back to last picked record since lotsa dbFindRecord,etc have been done

 set status [DCTdbFindRecord  $GLOBALselectedRecordName]
}

proc DCTallLinks {} {

     global GLOBALselectedRecordName
     set  widget .allLinks

     APSScrolledList  $widget  -name "Listing of All Links (by record)" -height 10 \
	 -packOption "-side left -fill x -expand true" -callback "destroy $widget"

#     set w $widget.button.Row.ok.button
#     $w configure -text "Dimiss" -state normal

     set w $widget.listbox
     $w  configure  -width 80

     set blnk " "
     set numRecdes "[DCTdbGetNRecordTypes]"
     if { "$numRecdes" > "0" } {
          set status [DCTdbFirstRecordType]
	  set quit1  [expr $numRecdes - 1 ]
     }

#     puts "\n***   All Links Listing ...................................................."

     for {set j 0} {$j<$numRecdes} {incr j} {
          set type [DCTdbGetRecordTypeName]
          set numRecords "[DCTdbGetNRecords]"
          if { "$numRecords" > "0" } {
               set status [DCTdbFirstRecord]
	       set quit2  [expr $numRecords - 1 ]
          }
          for {set i 0} {$i<$numRecords} {incr i} {
               set name [DCTdbGetRecordName]
#     puts "\n  Record name: $name "
               set outline [format "%1s"  $blnk ]
	       $w  insert  end  $outline
               set outline [format "%-28s"  $name]
	       $w  insert  end  $outline

	       set numLinks [DCTdbGetNLinks]
	       
#     puts "\       Field prompt .......... Name   Type      Contents"
     set outline [format "%-60s"  "     Field prompt            Name   Type      Contents"]
     $w  insert  end  $outline
	       for {set k 0} {$k<$numLinks} {incr k} {
		  set linkField [DCTdbGetLinkField $k]
		  set linkType  [DCTdbGetLinkType]
		  set prompt    [DCTdbGetPrompt]
		  set fieldName [DCTdbGetFieldName]
		  set kind      [DCTdbGetRange]
		  set contents  [DCTdbGetString]

  set outline [format "%-4s %-18s  %-6s %-7s  %-28s" $blnk $prompt  $fieldName $kind  $contents]
#		  puts "  $outline"
		  $w  insert end  $outline
               }

               if {$i < $quit2 }  { set status [DCTdbNextRecord]
               }
          }
	  if {$j < $quit1 }  {
               set status [DCTdbNextRecordType]
	  }
     }
# go back to last picked record since lotsa dbFindRecord,etc have been done

 set status [DCTdbFindRecord  $GLOBALselectedRecordName]
}


proc DCTsortByName  {} {

     set nrecs 0
     set blnk " "
     set numRecdes "[DCTdbGetNRecordTypes]"
     if { "$numRecdes" > "0" } {
          set status [DCTdbFirstRecordType]
	  set quit1  [expr $numRecdes - 1 ]
     }
     # puts "\n***   Listing of records sorted by record name....................."

     for {set j 0} {$j<$numRecdes} {incr j} {
          set type [DCTdbGetRecordTypeName]
          set numRecords "[DCTdbGetNRecords]"
          if { "$numRecords" > "0" } {
               set status [DCTdbFirstRecord]
	       set quit2  [expr $numRecords - 1 ]
          }
          for {set i 0} {$i<$numRecords} {incr i} {
	       set nrecs [expr $nrecs + 1 ]
               set name [DCTdbGetRecordName]
	       if {$nrecs == 1} {
		   linsert sortlist 1  $name
               } else {

	            for {set k 0} {$k<$nrecs} {incr k}  {
                    }
               }		    

               set outline [format " %-28s" $name ]
	       # puts "  $outline"

               if {$i < $quit2 }  { set status [DCTdbNextRecord]
               }

          }
	  if {$j < $quit1 }  {
               set status [DCTdbNextRecordType]
	  }
     }

}

proc DCTinsertAlfa  {newName  listBoxName} {

   set num [$listBoxName  size]

   for {set j 0} {$j<=$num} {incr j}  {
       set name [$listBoxName  get  $j]
       if { [string compare $newName  $name]  == -1 }  {break}
   }

   $listBoxName  insert  $j  $newName

}
       
proc DCTgetConstants  {} {
   global  DCT_STRING
   global  DCT_INTEGER
   global  DCT_REAL
   global  DCT_MENU
   global  DCT_MENUFORM
   global  DCT_INLINK
   global  DCT_OUTLINK
   global  DCT_FWDLINK

   set DCT_STRING    [DCTdbGetST]
#   puts "in DCTgetConstants: DCT_STRING= $DCT_STRING"
   set DCT_INTEGER   [DCTdbGetIN]
#   puts "in DCTgetConstants: DCT_INTEGER= $DCT_INTEGER"
   set DCT_REAL      [DCTdbGetRL]
#   puts "in DCTgetConstants: DCT_REAL= $DCT_REAL"
   
   # set nREAL  DCT_REAL
   # set nREAL $DCT_REAL
   # puts "in DCTgetConstants: DCT_REAL= $nREAL"

   set DCT_MENU      [DCTdbGetME]
#   puts "in DCTgetConstants: DCT_MENU= $DCT_MENU"
   set DCT_MENUFORM  [DCTdbGetMF]
#   puts "in DCTgetConstants: DCT_MENUFORM= $DCT_MENUFORM"
   set DCT_INLINK    [DCTdbGetNL]
#   puts "in DCTgetConstants: DCT_INLINK= $DCT_INLINK"
   set DCT_OUTLINK   [DCTdbGetOL]
#   puts "in DCTgetConstants: DCT_OUTLINK= $DCT_OUTLINK"
   set DCT_FWDLINK   [DCTdbGetFL]
#   puts "in DCTgetConstants: DCT_FWDLINK= $DCT_FWDLINK"

   global  DCT_LINK_CONSTANT
   global  DCT_LINK_FORM
   global  DCT_LINK_PV

   set DCT_LINK_CONSTANT  [DCTdbGetLkC]
#   puts "in DCTgetConstants: DCT_LINK_CONSTANT= $DCT_LINK_CONSTANT"
   set DCT_LINK_FORM  [DCTdbGetLkF]
#   puts "in DCTgetConstants: DCT_LINK_FORM= $DCT_LINK_FORM"
   set DCT_LINK_PV  [DCTdbGetLkPV]
#   puts "in DCTgetConstants: DCT_LINK_PV= $DCT_LINK_PV"
}

#----------------------------------------------------


#------------------------------------------
# Phase 0: create the front end application
#------------------------------------------

DCTgetConstants

option add *font 7x13

APSApplication . -name "DCT - Database Configuration Tool - for 3.13" \
     -version "\nDatabase Configuration Tool\n\nDCT for 3.13(1May97)\n" \
     -overview "\nDCT is a tool for creating EPICS ioc databases.\n" \
     -contextHelp "This is the main DCT application."
     
#------------------------------------------
# Phase 1:  Add menus
#------------------------------------------
 global GLOBALsavetype

# delete the Quit command created by APSApplication
.menu.file.menu delete 1 1


# add the following 7 text entries under the File menu heading which
# was generated by  APSApplication

.menu.file.menu add command -label "Clear" \
     -command "DCTChangedDialog DCTClearOK" -underline 0
.menu.file.menu add command -label "New" \
     -command "DCTChangedDialog DCTNewOK" -underline 0
.menu.file.menu add command -label "Open..." \
     -command "DCTChangedDialog DCTOpenOK" -underline 0
.menu.file.menu add command -label "Append..." \
     -command "DCTChangedDialog DCTAppendOK" -underline 0

# .menu.file.menu add command -label "Save..."\
#      -command "DCTSaveRecords" -underline 1

 .menu.file.menu add cascade -label "Save..." \
      -menu .menu.file.menu.save -underline 0
.menu.file.menu add separator
.menu.file.menu add command -label "Print..."  \
     -command DCTFilePrintAction -underline 0
.menu.file.menu add separator
.menu.file.menu add command -label "Quit" \
     -command "DCTChangedDialog exit" -underline 0

#  Remove  next 2 option choices for  Save 29jul96 
 menu .menu.file.menu.save
 .menu.file.menu.save add radiobutton -label "Ouput only non-default values" \
      -command "set GLOBALsavetype 0; DCTSaveRecords" -value 0
 .menu.file.menu.save add radiobutton -label "Output all values" \
      -command "set GLOBALsavetype 1; DCTSaveRecords" -value 1

APSMenubarAddMenu .edit -parent .menu -text Edit
.menu.edit.menu add command -label "Create record" \
     -command "DCTEditCreateAction .createRecord" -underline 0
.menu.edit.menu add command -label "Copy record" \
     -command "DCTEditNameAction .copyRecord Copy" \
     -underline 0
.menu.edit.menu add command -label "Rename record" \
     -command "DCTEditNameAction .renameRecord Rename"  \
     -underline 0
.menu.edit.menu add command -label "Delete record" \
     -command "DCTEditDeleteAction .deleteRecord" -underline 0
.menu.edit.menu add separator
.menu.edit.menu add cascade -label "Sort records..." \
     -menu .menu.edit.menu.sort -underline 0
.menu.edit.menu add command -label "Prefix" \
     -command "DCTEditPrefixAction .prefixWindow" -underline 0

menu .menu.edit.menu.sort
.menu.edit.menu.sort add radiobutton -label "By record name" \
     -command DCTEditSortAction  -value 0
#    -command DCTsortByName  -value 0
.menu.edit.menu.sort add radiobutton -label "By record type" \
     -command DCTEditSortAction  -value 1

APSMenubarAddMenu .view -parent .menu -text View
.menu.view.menu add command -label "Unresolved links" \
     -command DCTunresLinks -underline 0
.menu.view.menu add command -label "All link references" \
     -command DCTallLinks  -underline 0
.menu.view.menu add command -label "Database consistency check" \
     -command DCTViewAction -underline 0
.menu.view.menu add command -label "Link tree for a record" \
     -command DCTViewAction -underline 0
.menu.view.menu add separator
.menu.view.menu add command -label "Record description ..." \
     -command DCTViewAction -underline 0
.menu.view.menu add command -label "Field description ..." \
     -command DCTViewAction -underline 0



#----------------------------------------------------
# Phase 2: Accelerators, mnemonics, command-line info
#----------------------------------------------------

# File Menu bindings
bind . <Control-n> {DCTChangedDialog DCTNewOK}
bind . <Control-o> {DCTChangedDialog DCTOpenOK}
bind . <Control-a> {DCTChangedDialog DCTAppendOK}
bind . <Control-s> {DCTSaveRecords}
bind . <Control-p> {DCTFilePrintAction}
bind . <Control-x> {destroy .}
bind . <Control-q> {DCTChangedDialog "destroy ."}

# Edit Menu bindings
bind . <Control-c> {DCTEditCreateAction .createRecord}
bind . <Control-r> {DCTEditNameAction .renameRecord Rename}
bind . <Control-d> {DCTEditDeleteAction .deleteRecord}

focus .menu.file

#------------------------------------------
# Phase 3: Create Record List Window
#------------------------------------------
global GLOBALselectedRecordName

APSScrolledList .scrolledRecordList -parent .userFrame -height 20 \
     -callback DCTRecordSelectCallback -name RecordList  \
     -packOption "-side left -fill both" 

.userFrame.scrolledRecordList.listbox configure -width 35

#------------------------------------------
# Phase 4: Create Field List Window
#------------------------------------------
label .userFrame.labelNAME -relief ridge -bd 4 -anchor w -width 90
label .userFrame.labelTYPE -relief ridge -bd 4 -anchor w
pack  .userFrame.labelNAME -side top -fill x
pack  .userFrame.labelTYPE -side top -fill x

APSDialogBox .fieldDialog -parent .userFrame -name "Edit Record" \
     -contextHelp "Context Help" 
set widget .userFrame.fieldDialog
$widget.buttonRow.ok.button configure -command DCTEditFieldsOK \
     -state normal -text "Save field values" 
$widget.buttonRow.cancel.button configure -command DCTEditFieldsCancel

global GLOBALfieldListFrame
set GLOBALfieldListFrame [APSScroll .scrolledFieldList -parent $widget.userFrame \
     -name FieldList -packOption "-expand 1 -fill both" ]

pack .userFrame.fieldDialog -side right -expand 1 -fill both


#------------------------------------------
# Phase 5: Set and display globals
#------------------------------------------
global GLOBALrecordNamePrefix
set GLOBALrecordNamePrefix ""

global GLOBALtotalRecords
set GLOBALtotalRecords 0
global GLOBALfilename
set GLOBALfilename "no file name"
frame .fileFrame
label .fileFrame.fileText -borderwidth 1 -text "File Name:  "
label .fileFrame.fileName -borderwidth 1 -textvariable GLOBALfilename
pack .fileFrame.fileText .fileFrame.fileName  -side left -fill x -anchor center
pack .fileFrame -side bottom -fill x -anchor s

global GLOBALdatabaseChanged
set GLOBALdatabaseChanged  0

global GLOBALselectedRecordName
set GLOBALselectedRecordName  ""

set apsFileSelect(pattern) "*.db*"

#------------------------------------------
# Phase 5: Populate record and field lists
#------------------------------------------

DCTSetScrolledRecordList

#-- the end of  dct.tcl  source code  ------------------------------------
